@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -7,10 +7,9 @@
  */
 
 #include "Chan.h"
-#include "znc.h"
-#include "User.h"
-#include "Utils.h"
 #include "IRCSock.h"
+#include "User.h"
+#include "znc.h"
 
 CChan::CChan(const CString& sName, CUser* pUser, bool bInConfig) {
 	m_sName = sName.Token(0);
@@ -56,20 +55,44 @@ bool CChan::WriteConfig(CFile& File) {
 		return false;
 	}
 
-	File.Write("\t<Chan " + GetName() + ">\r\n");
+	File.Write("\t<Chan " + GetName() + ">\n");
 
-	File.Write("\t\tBuffer     = " + CString(GetBufferCount()) + "\r\n");
-	File.Write("\t\tKeepBuffer = " + CString((KeepBuffer()) ? "true" : "false") + "\r\n");
-	File.Write("\t\tDetached   = " + CString((IsDetached()) ? "true" : "false") + "\r\n");
-	File.Write("\t\tAutoCycle  = " + CString((AutoCycle()) ? "true" : "false") + "\r\n");
-	if (!GetKey().empty()) { File.Write("\t\tKey        = " + GetKey() + "\r\n"); }
-	if (!GetDefaultModes().empty()) { File.Write("\t\tModes      = " + GetDefaultModes() + "\r\n"); }
+	if (m_pUser->GetBufferCount() != GetBufferCount())
+		File.Write("\t\tBuffer     = " + CString(GetBufferCount()) + "\n");
+	if (m_pUser->KeepBuffer() != KeepBuffer())
+		File.Write("\t\tKeepBuffer = " + CString((KeepBuffer()) ? "true" : "false") + "\n");
+	if (IsDetached())
+		File.Write("\t\tDetached   = true\n");
+	if (m_pUser->AutoCycle() != AutoCycle())
+		File.Write("\t\tAutoCycle  = " + CString((AutoCycle()) ? "true" : "false") + "\n");
+	if (!GetKey().empty()) { File.Write("\t\tKey        = " + GetKey() + "\n"); }
+	if (!GetDefaultModes().empty()) { File.Write("\t\tModes      = " + GetDefaultModes() + "\n"); }
 
-	File.Write("\t</Chan>\r\n");
+	File.Write("\t</Chan>\n");
 	return true;
 }
 
-void CChan::Joined() {
+void CChan::Clone(CChan& chan) {
+	// We assume that m_sName and m_pUser are equal
+	SetBufferCount(chan.GetBufferCount());
+	SetKeepBuffer(chan.KeepBuffer());
+	SetAutoCycle(chan.AutoCycle());
+	SetKey(chan.GetKey());
+	SetDefaultModes(chan.GetDefaultModes());
+
+	if (IsDetached() != chan.IsDetached()) {
+		// Only send something if it makes sense
+		// (= Only detach if client is on the channel
+		//    and only attach if we are on the channel)
+		if (IsOn()) {
+			if (IsDetached()) {
+				JoinUser(false, "");
+			} else {
+				DetachUser();
+			}
+		}
+		SetDetached(chan.IsDetached());
+	}
 }
 
 void CChan::Cycle() const {
@@ -104,7 +127,7 @@ void CChan::JoinUser(bool bForce, const CString& sKey, CClient* pClient) {
 			pThisClient = pClient;
 
 		for (map<CString,CNick*>::iterator a = m_msNicks.begin(); a != m_msNicks.end(); a++) {
-			if(pThisClient->HasNamesx()) {
+			if (pThisClient->HasNamesx()) {
 				sPerm = a->second->GetPermStr();
 			} else {
 				char c = a->second->GetPermChar();
@@ -113,7 +136,7 @@ void CChan::JoinUser(bool bForce, const CString& sKey, CClient* pClient) {
 					sPerm += c;
 				}
 			}
-			if(pThisClient->HasUHNames() && !a->second->GetIdent().empty() && !a->second->GetHost().empty()) {
+			if (pThisClient->HasUHNames() && !a->second->GetIdent().empty() && !a->second->GetHost().empty()) {
 				sNick = a->first + "!" + a->second->GetIdent() + "@" + a->second->GetHost();
 			} else {
 				sNick = a->first;
@@ -169,7 +192,7 @@ CString CChan::GetModeString() const {
 
 CString CChan::GetModeForNames() const {
 	CString sMode;
-	
+
 	for (map<unsigned char, CString>::const_iterator it = m_musModes.begin(); it != m_musModes.end(); it++) {
 		if (it->first == 's') {
 		    sMode = "@";
@@ -180,7 +203,7 @@ CString CChan::GetModeForNames() const {
 
 	return (sMode.empty() ? "=" : sMode);
 }
-	
+
 void CChan::SetModes(const CString& sModes) {
 	m_musModes.clear();
 	m_uLimit = 0;
@@ -441,19 +464,19 @@ bool CChan::AddNick(const CString& sNick) {
 		pNick->SetUser(m_pUser);
 	}
 
-	if(!sIdent.empty())
+	if (!sIdent.empty())
 		pNick->SetIdent(sIdent);
-	if(!sHost.empty())
+	if (!sHost.empty())
 		pNick->SetHost(sHost);
 
-	for(CString::size_type i = 0; i < sPrefix.length(); i++) {
+	for (CString::size_type i = 0; i < sPrefix.length(); i++) {
 		if (pNick->AddPerm(sPrefix[i])) {
 			IncPermCount(sPrefix[i]);
 		}
 	}
 
 	if (pNick->GetNick().CaseCmp(m_pUser->GetCurNick()) == 0) {
-		for(CString::size_type i = 0; i < sPrefix.length(); i++) {
+		for (CString::size_type i = 0; i < sPrefix.length(); i++) {
 			AddPerm(sPrefix[i]);
 		}
 	}
@@ -491,15 +514,18 @@ void CChan::IncPermCount(unsigned char uPerm) {
 }
 
 bool CChan::RemNick(const CString& sNick) {
-	map<CString,CNick*>::iterator it = m_msNicks.find(sNick);
+	map<CString,CNick*>::iterator it;
+	set<unsigned char>::iterator it2;
+
+	it = m_msNicks.find(sNick);
 	if (it == m_msNicks.end()) {
 		return false;
 	}
 
 	const set<unsigned char>& suPerms = it->second->GetChanPerms();
 
-	for (set<unsigned char>::iterator it = suPerms.begin(); it != suPerms.end(); it++) {
-		DecPermCount(*it);
+	for (it2 = suPerms.begin(); it2 != suPerms.end(); it2++) {
+		DecPermCount(*it2);
 	}
 
 	delete it->second;
