@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,20 +9,20 @@
 #ifndef _CHAN_H
 #define _CHAN_H
 
-#include "main.h"
+#include "FileUtils.h"
 #include "Nick.h"
 #include "String.h"
-#include "FileUtils.h"
-#include "Client.h"
-#include <vector>
 #include <map>
 #include <set>
+#include <vector>
+
 using std::vector;
 using std::map;
 using std::set;
 
 // Forward Declarations
 class CUser;
+class CClient;
 // !Forward Declarations
 
 class CChan {
@@ -55,7 +55,7 @@ public:
 
 	void Reset();
 	bool WriteConfig(CFile& File);
-	void Joined();
+	void Clone(CChan& chan);
 	void Cycle() const;
 	void JoinUser(bool bForce = false, const CString& sKey = "", CClient* pClient = NULL);
 	void DetachUser();
@@ -104,7 +104,7 @@ public:
 	// Setters
 	void IncPermCount(unsigned char uPerm);
 	void DecPermCount(unsigned char uPerm);
-	void SetIsOn(bool b) { m_bIsOn = b; if (!b) { Reset(); } else { Joined(); } }
+	void SetIsOn(bool b) { m_bIsOn = b; if (!b) { Reset(); } }
 	void SetKey(const CString& s) { m_sKey = s; }
 	void SetTopic(const CString& s) { m_sTopic = s; }
 	void SetTopicOwner(const CString& s) { m_sTopicOwner = s; }
