@@ -1,19 +1,20 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
 #include "Client.h"
-#include "znc.h"
-#include "User.h"
-#include "IRCSock.h"
+#include "Chan.h"
 #include "DCCBounce.h"
 #include "DCCSock.h"
+#include "IRCSock.h"
 #include "Server.h"
+#include "Timers.h"
+#include "User.h"
+#include "znc.h"
 
 CClient::~CClient() {
 	if (!m_spAuth.IsNull()) {
@@ -24,6 +25,9 @@ CClient::~CClient() {
 		m_pUser->AddBytesRead(GetBytesRead());
 		m_pUser->AddBytesWritten(GetBytesWritten());
 	}
+	if (m_pTimeout) {
+		CZNC::Get().GetManager().DelCronByAddr(m_pTimeout);
+	}
 }
 
 void CClient::ReadLine(const CString& sData) {
@@ -36,7 +40,7 @@ void CClient::ReadLine(const CString& sData) {
 	DEBUG_ONLY(cout << "(" << ((m_pUser) ? m_pUser->GetUserName() : CString("")) << ") CLI -> ZNC [" << sLine << "]" << endl);
 
 #ifdef _MODULES
-	if (m_bAuthed) {
+	if (IsAttached()) {
 		MODULECALL(OnUserRaw(sLine), m_pUser, this, return);
 	}
 #endif
@@ -44,7 +48,7 @@ void CClient::ReadLine(const CString& sData) {
 	CString sCommand = sLine.Token(0);
 
 	if (sCommand.CaseCmp("PASS") == 0) {
-		if (!m_bAuthed) {
+		if (!IsAttached()) {
 			m_bGotPass = true;
 			m_sPass = sLine.Token(1);
 
@@ -65,7 +69,7 @@ void CClient::ReadLine(const CString& sData) {
 			sNick.LeftChomp();
 		}
 
-		if (!m_bAuthed) {
+		if (!IsAttached()) {
 			m_sNick = sNick;
 			m_bGotNick = true;
 
@@ -97,7 +101,7 @@ void CClient::ReadLine(const CString& sData) {
 			}
 		}
 	} else if (sCommand.CaseCmp("USER") == 0) {
-		if (!m_bAuthed) {
+		if (!IsAttached()) {
 			if (m_sUser.empty()) {
 				m_sUser = sLine.Token(1);
 			}
@@ -117,8 +121,7 @@ void CClient::ReadLine(const CString& sData) {
 	}
 
 	if (!m_pUser) {
-		PutClient("ERROR :You need to send your password first!");
-		Close();
+		// Only NICK, USER and PASS are allowed before login
 		return;
 	}
 
@@ -146,10 +149,13 @@ void CClient::ReadLine(const CString& sData) {
 		}
 	} else if (sCommand.CaseCmp("PING") == 0) {
 		CString sTarget = sLine.Token(1);
-		
-		if (sTarget.CaseCmp("irc.znc.com") == 0) {
-		    PutClient("PONG " + sLine.substr(5));
-		    return;
+
+		// If the client meant to ping us or we can be sure the server
+		// won't answer the ping (=no server connected) -> PONG back.
+		// else: It's the server's job to send a PONG.
+		if (sTarget.CaseCmp("irc.znc.com") == 0 || !m_pIRCSock) {
+			PutClient("PONG " + sLine.substr(5));
+			return;
 		}
 	} else if (sCommand.CaseCmp("PONG") == 0) {
 		return;	// Block pong replies, we already responded to the pings
@@ -168,9 +174,7 @@ void CClient::ReadLine(const CString& sData) {
 
 			for (unsigned int a = 0; a < vChans.size(); a++) {
 				CString sChannel = vChans[a];
-#ifdef _MODULES
 				MODULECALL(OnUserJoin(sChannel, sKey), m_pUser, this, continue);
-#endif
 
 				CChan* pChan = m_pUser->FindChan(sChannel);
 
@@ -202,9 +206,7 @@ void CClient::ReadLine(const CString& sData) {
 			sMessage.LeftChomp();
 		}
 
-#ifdef _MODULES
 		MODULECALL(OnUserPart(sChan, sMessage), m_pUser, this, return);
-#endif
 
 		if (m_pUser) {
 			CChan* pChan = m_pUser->FindChan(sChan);
@@ -221,6 +223,24 @@ void CClient::ReadLine(const CString& sData) {
 		if (!sMessage.empty()) {
 			sLine += " :" + sMessage;
 		}
+	} else if (sCommand.CaseCmp("TOPIC") == 0) {
+		CString sChan = sLine.Token(1);
+		CString sTopic = sLine.Token(2, true);
+		bool bUnset = false;
+
+		if (sTopic.Left(1) == ":") {
+			sTopic.LeftChomp();
+			if (sTopic.empty())
+				bUnset = true;
+		}
+
+		MODULECALL(OnUserTopic(sChan, sTopic), m_pUser, this, return);
+
+		sLine = "TOPIC " + sChan;
+
+		if (!sTopic.empty() || bUnset) {
+			sLine += " :" + sTopic;
+		}
 	} else if (sCommand.CaseCmp("MODE") == 0) {
 		CString sTarget = sLine.Token(1);
 		CString sModes = sLine.Token(2, true);
@@ -245,10 +265,10 @@ void CClient::ReadLine(const CString& sData) {
 		return;		// Don't forward this msg.  We don't want the client getting us disconnected.
 	} else if (sCommand.CaseCmp("PROTOCTL") == 0) {
 		unsigned int i = 1;
-		while(!sLine.Token(i).empty()) {
-			if(sLine.Token(i).CaseCmp("NAMESX") == 0) {
+		while (!sLine.Token(i).empty()) {
+			if (sLine.Token(i).CaseCmp("NAMESX") == 0) {
 				m_bNamesx = true;
-			} else if(sLine.Token(i).CaseCmp("UHNAMES") == 0) {
+			} else if (sLine.Token(i).CaseCmp("UHNAMES") == 0) {
 				m_bUHNames = true;
 			}
 			i++;
@@ -308,6 +328,12 @@ void CClient::ReadLine(const CString& sData) {
 		}
 #endif
 
+		if (!m_pIRCSock) {
+			PutStatus("Your message to [" + sTarget + "] got lost, "
+					"you are not connected to IRC!");
+			return;
+		}
+
 		CChan* pChan = m_pUser->FindChan(sTarget);
 
 		if ((pChan) && (pChan->KeepBuffer())) {
@@ -385,10 +411,8 @@ void CClient::ReadLine(const CString& sData) {
 							if (m_pUser) {
 								m_pUser->GetFile(GetNick(), CUtils::GetIP(uLongIP), uPort, sLocalFile, uFileSize);
 							}
-#ifdef _MODULES
 						} else {
 							MODULECALL(OnDCCUserSend(sTarget, uLongIP, uPort, sFile, uFileSize), m_pUser, this, return);
-#endif
 						}
 					} else {
 						unsigned short uBNCPort = CDCCBounce::DCCRequest(sTarget, uLongIP, uPort, sFile, false, m_pUser, (m_pIRCSock) ? m_pIRCSock->GetLocalIP() : GetLocalIP(), "");
@@ -436,10 +460,16 @@ void CClient::ReadLine(const CString& sData) {
 			}
 
 			if (strncasecmp(sTarget.c_str(), m_pUser->GetStatusPrefix().c_str(), m_pUser->GetStatusPrefix().length()) == 0) {
-#ifdef _MODULES
 				CString sModule = sTarget;
 				sModule.LeftChomp(m_pUser->GetStatusPrefix().length());
 
+				if (sModule == "status") {
+					StatusCTCP(sCTCP);
+
+					return;
+				}
+
+#ifdef _MODULES
 				CModule* pModule = m_pUser->GetModules().FindModule(sModule);
 				if (pModule) {
 					pModule->SetClient(this);
@@ -484,9 +514,7 @@ void CClient::ReadLine(const CString& sData) {
 		}
 
 		if ((m_pUser) && (sTarget.CaseCmp(CString(m_pUser->GetStatusPrefix() + "status")) == 0)) {
-#ifdef _MODULES
 			MODULECALL(OnStatusCommand(sMsg), m_pUser, this, return);
-#endif
 			UserCommand(sMsg);
 			return;
 		}
@@ -520,9 +548,13 @@ void CClient::ReadLine(const CString& sData) {
 			return;
 		}
 
-#ifdef _MODULES
 		MODULECALL(OnUserMsg(sTarget, sMsg), m_pUser, this, return);
-#endif
+
+		if (!m_pIRCSock) {
+			PutStatus("Your message to [" + sTarget + "] got lost, "
+					"you are not connected to IRC!");
+			return;
+		}
 
 		CChan* pChan = m_pUser->FindChan(sTarget);
 
@@ -571,6 +603,16 @@ bool CClient::DecKeepNickCounter() {
 	return true;
 }
 
+void CClient::StatusCTCP(const CString& sLine) {
+	CString sCommand = sLine.Token(0);
+
+	if (sCommand.CaseCmp("PING") == 0) {
+		PutStatusNotice("\001PING " + sLine.Token(1, true) + "\001");
+	} else if (sCommand.CaseCmp("VERSION") == 0) {
+		PutStatusNotice("\001VERSION " + CZNC::GetTag() + "\001");
+	}
+}
+
 void CClient::UserCommand(const CString& sLine) {
 	if (!m_pUser) {
 		return;
@@ -631,7 +673,7 @@ void CClient::UserCommand(const CString& sLine) {
 			for (unsigned int b = 0; b < sPerms.size(); b++) {
 				if (a->second->HasPerm(sPerms[b])) {
 					CString sPerm;
-				   	sPerm += sPerms[b];
+					sPerm += sPerms[b];
 					Table.SetCell(sPerm, sPerm);
 				}
 			}
@@ -642,10 +684,10 @@ void CClient::UserCommand(const CString& sLine) {
 		}
 
 		unsigned int uTableIdx = 0;
-		CString sLine;
+		CString sTmp;
 
-		while (Table.GetLine(uTableIdx++, sLine)) {
-			PutStatus(sLine);
+		while (Table.GetLine(uTableIdx++, sTmp)) {
+			PutStatus(sTmp);
 		}
 	} else if (sCommand.CaseCmp("DETACH") == 0) {
 		if (m_pUser) {
@@ -671,6 +713,14 @@ void CClient::UserCommand(const CString& sLine) {
 		if (!SendMotd()) {
 			PutStatus("There is no MOTD set.");
 		}
+	} else if (m_pUser->IsAdmin() && sCommand.CaseCmp("Rehash") == 0) {
+		CString sRet;
+
+		if (CZNC::Get().RehashConfig(sRet)) {
+			PutStatus("Rehashing succeeded!");
+		} else {
+			PutStatus("Rehashing failed: " + sRet);
+		}
 	} else if (m_pUser->IsAdmin() && sCommand.CaseCmp("SaveConfig") == 0) {
 		if (CZNC::Get().WriteConfig()) {
 			PutStatus("Wrote config to [" + CZNC::Get().GetConfigFile() + "]");
@@ -713,10 +763,10 @@ void CClient::UserCommand(const CString& sLine) {
 
 			if (Table.size()) {
 				unsigned int uTableIdx = 0;
-				CString sLine;
+				CString sTmp;
 
-				while (Table.GetLine(uTableIdx++, sLine)) {
-					PutStatus(sLine);
+				while (Table.GetLine(uTableIdx++, sTmp)) {
+					PutStatus(sTmp);
 				}
 			}
 		}
@@ -744,10 +794,10 @@ void CClient::UserCommand(const CString& sLine) {
 
 		if (Table.size()) {
 			unsigned int uTableIdx = 0;
-			CString sLine;
+			CString sTmp;
 
-			while (Table.GetLine(uTableIdx++, sLine)) {
-				PutStatus(sLine);
+			while (Table.GetLine(uTableIdx++, sTmp)) {
+				PutStatus(sTmp);
 			}
 		}
 	} else if (m_pUser->IsAdmin() && sCommand.CaseCmp("SetMOTD") == 0) {
@@ -787,6 +837,11 @@ void CClient::UserCommand(const CString& sLine) {
 	} else if (sCommand.CaseCmp("JUMP") == 0 ||
 			sCommand.CaseCmp("CONNECT") == 0) {
 		if (m_pUser) {
+			if (!m_pUser->HasServers()) {
+				PutStatus("You don't have any servers added.");
+				return;
+			}
+
 			if (m_pIRCSock) {
 				m_pIRCSock->Quit();
 				PutStatus("Jumping to the next server in the list...");
@@ -817,7 +872,7 @@ void CClient::UserCommand(const CString& sLine) {
 				PutStatus("Channel [" + sChan + "] not found.");
 				return;
 			}
-			
+
 			pChan->Enable();
 			PutStatus("Channel [" + sChan + "] enabled.");
 		}
@@ -865,10 +920,10 @@ void CClient::UserCommand(const CString& sLine) {
 
 			if (Table.size()) {
 				unsigned int uTableIdx = 0;
-				CString sLine;
+				CString sTmp;
 
-				while (Table.GetLine(uTableIdx++, sLine)) {
-					PutStatus(sLine);
+				while (Table.GetLine(uTableIdx++, sTmp)) {
+					PutStatus(sTmp);
 				}
 			}
 		}
@@ -880,7 +935,7 @@ void CClient::UserCommand(const CString& sLine) {
 			return;
 		}
 
-		if (m_pUser->FindServer(sLine.Token(1))) {
+		if (m_pUser->FindServer(sServer)) {
 			PutStatus("That server already exists");
 			return;
 		}
@@ -931,10 +986,10 @@ void CClient::UserCommand(const CString& sLine) {
 
 			if (Table.size()) {
 				unsigned int uTableIdx = 0;
-				CString sLine;
+				CString sTmp;
 
-				while (Table.GetLine(uTableIdx++, sLine)) {
-					PutStatus(sLine);
+				while (Table.GetLine(uTableIdx++, sTmp)) {
+					PutStatus(sTmp);
 				}
 			}
 		    } else {
@@ -959,10 +1014,10 @@ void CClient::UserCommand(const CString& sLine) {
 
 			if (Table.size()) {
 				unsigned int uTableIdx = 0;
-				CString sLine;
+				CString sTmp;
 
-				while (Table.GetLine(uTableIdx++, sLine)) {
-					PutStatus(sLine);
+				while (Table.GetLine(uTableIdx++, sTmp)) {
+					PutStatus(sTmp);
 				}
 			}
 		}
@@ -977,7 +1032,7 @@ void CClient::UserCommand(const CString& sLine) {
 			return;
 		}
 
-		sAbsolutePath = CUtils::ChangeDir(m_pUser->GetDLPath(), sFile, CZNC::Get().GetHomePath());
+		sAbsolutePath = CDir::ChangeDir(m_pUser->GetDLPath(), sFile, CZNC::Get().GetHomePath());
 
 		if (sAbsolutePath.Left(sAllowedPath.length()) != sAllowedPath) {
 			PutStatus("Illegal path.");
@@ -997,7 +1052,7 @@ void CClient::UserCommand(const CString& sLine) {
 			return;
 		}
 
-		sAbsolutePath = CUtils::ChangeDir(m_pUser->GetDLPath(), sFile, CZNC::Get().GetHomePath());
+		sAbsolutePath = CDir::ChangeDir(m_pUser->GetDLPath(), sFile, CZNC::Get().GetHomePath());
 
 		if (sAbsolutePath.Left(sAllowedPath.length()) != sAllowedPath) {
 			PutStatus("Illegal path.");
@@ -1036,7 +1091,7 @@ void CClient::UserCommand(const CString& sLine) {
 					Table.AddRow();
 					Table.SetCell("Type", "Sending");
 					Table.SetCell("State", CString::ToPercent(pSock->GetProgress()));
-					Table.SetCell("Speed", CString::ToKBytes(pSock->GetAvgWrite() / 1000.0));
+					Table.SetCell("Speed", CString((int)(pSock->GetAvgWrite() / 1024.0)) + " KiB/s");
 					Table.SetCell("Nick", pSock->GetRemoteNick());
 					Table.SetCell("IP", pSock->GetRemoteIP());
 					Table.SetCell("File", pSock->GetFileName());
@@ -1046,7 +1101,7 @@ void CClient::UserCommand(const CString& sLine) {
 					Table.AddRow();
 					Table.SetCell("Type", "Getting");
 					Table.SetCell("State", CString::ToPercent(pSock->GetProgress()));
-					Table.SetCell("Speed", CString::ToKBytes(pSock->GetAvgRead() / 1000.0));
+					Table.SetCell("Speed", CString((int)(pSock->GetAvgRead() / 1024.0)) + " KiB/s");
 					Table.SetCell("Nick", pSock->GetRemoteNick());
 					Table.SetCell("IP", pSock->GetRemoteIP());
 					Table.SetCell("File", pSock->GetFileName());
@@ -1098,10 +1153,10 @@ void CClient::UserCommand(const CString& sLine) {
 
 		if (Table.size()) {
 			unsigned int uTableIdx = 0;
-			CString sLine;
+			CString sTmp;
 
-			while (Table.GetLine(uTableIdx++, sLine)) {
-				PutStatus(sLine);
+			while (Table.GetLine(uTableIdx++, sTmp)) {
+				PutStatus(sTmp);
 			}
 		} else {
 			PutStatus("You have no active DCCs.");
@@ -1124,10 +1179,11 @@ void CClient::UserCommand(const CString& sLine) {
 					GTable.SetCell("Description", GModules[b]->GetDescription().Ellipsize(128));
 				}
 
-				unsigned int uTableIdx = 0; CString sLine;
+				unsigned int uTableIdx = 0;
+				CString sTmp;
 
-				while (GTable.GetLine(uTableIdx++, sLine)) {
-					PutStatus(sLine);
+				while (GTable.GetLine(uTableIdx++, sTmp)) {
+					PutStatus(sTmp);
 				}
 			}
 		}
@@ -1150,9 +1206,76 @@ void CClient::UserCommand(const CString& sLine) {
 				Table.SetCell("Description", Modules[b]->GetDescription().Ellipsize(128));
 			}
 
-			unsigned int uTableIdx = 0; CString sLine;
-			while (Table.GetLine(uTableIdx++, sLine)) {
-				PutStatus(sLine);
+			unsigned int uTableIdx = 0;
+			CString sTmp;
+			while (Table.GetLine(uTableIdx++, sTmp)) {
+				PutStatus(sTmp);
+			}
+		}
+#else
+		PutStatus("Modules are not enabled.");
+#endif
+		return;
+	} else if ((sCommand.CaseCmp("LISTAVAILMODS") == 0) || (sCommand.CaseCmp("LISTAVAILABLEMODULES") == 0)) {
+#ifdef _MODULES
+		if (m_pUser->DenyLoadMod()) {
+			PutStatus("Access Denied.");
+			return;
+		}
+
+		if (m_pUser->IsAdmin()) {
+			set<CModInfo> ssGlobalMods;
+			CZNC::Get().GetModules().GetAvailableMods(ssGlobalMods, true);
+
+			if (!ssGlobalMods.size()) {
+				PutStatus("No global modules available.");
+			} else {
+				CTable GTable;
+				GTable.AddColumn("Name");
+				GTable.AddColumn("Description");
+				set<CModInfo>::iterator it;
+
+				for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); it++) {
+					const CModInfo& Info = *it;
+					GTable.AddRow();
+					GTable.SetCell("Name", (CZNC::Get().GetModules().FindModule(Info.GetName()) ? "*" : " ") + Info.GetName());
+					GTable.SetCell("Description", Info.GetDescription().Ellipsize(128));
+				}
+
+				unsigned int uTableIdx = 0;
+				CString sTmp;
+
+				while (GTable.GetLine(uTableIdx++, sTmp)) {
+					PutStatus(sTmp);
+				}
+			}
+		}
+
+		if (m_pUser) {
+			set<CModInfo> ssUserMods;
+			CZNC::Get().GetModules().GetAvailableMods(ssUserMods);
+
+			if (!ssUserMods.size()) {
+				PutStatus("No user modules available.");
+				return;
+			}
+
+			CTable Table;
+			Table.AddColumn("Name");
+			Table.AddColumn("Description");
+			set<CModInfo>::iterator it;
+
+			for (it = ssUserMods.begin(); it != ssUserMods.end(); it++) {
+				const CModInfo& Info = *it;
+				Table.AddRow();
+				Table.SetCell("Name", (m_pUser->GetModules().FindModule(Info.GetName()) ? "*" : " ") + Info.GetName());
+				Table.SetCell("Description", Info.GetDescription().Ellipsize(128));
+			}
+
+			unsigned int uTableIdx = 0;
+			CString sTmp;
+			while (Table.GetLine(uTableIdx++, sTmp)) {
+				PutStatus(sTmp);
 			}
 		}
 #else
@@ -1289,15 +1412,19 @@ void CClient::UserCommand(const CString& sLine) {
 		PutStatus("Unable to unload [" + sMod + "] Modules are not enabled.");
 #endif
 		return;
-	} else if (sCommand.CaseCmp("SETVHOST") == 0) {
+	} else if (sCommand.CaseCmp("SETVHOST") == 0 && (m_pUser->IsAdmin() || !m_pUser->DenySetVHost())) {
 		CString sVHost = sLine.Token(1);
 
 		if (sVHost.empty()) {
 			PutStatus("Usage: SetVHost <VHost>");
+			return;
 		}
 
 		m_pUser->SetVHost(sVHost);
 		PutStatus("Set VHost to [" + m_pUser->GetVHost() + "]");
+	} else if (sCommand.CaseCmp("CLEARVHOST") == 0 && (m_pUser->IsAdmin() || !m_pUser->DenySetVHost())) {
+		m_pUser->SetVHost("");
+		PutStatus("VHost Cleared");
 	} else if (sCommand.CaseCmp("PLAYBUFFER") == 0) {
 		CString sChan = sLine.Token(1);
 
@@ -1346,6 +1473,14 @@ void CClient::UserCommand(const CString& sLine) {
 
 		pChan->ClearBuffer();
 		PutStatus("The buffer for [" + sChan + "] has been cleared");
+	} else if (sCommand.CaseCmp("CLEARALLCHANNELBUFFERS") == 0) {
+		vector<CChan*>::const_iterator it;
+		const vector<CChan*>& vChans = m_pUser->GetChans();
+
+		for (it = vChans.begin(); it != vChans.end(); it++) {
+			(*it)->ClearBuffer();
+		}
+		PutStatus("All channel buffers have been cleared");
 	} else if (sCommand.CaseCmp("SETBUFFER") == 0) {
 		CString sChan = sLine.Token(1);
 
@@ -1376,50 +1511,52 @@ void CClient::UserCommand(const CString& sLine) {
 		pChan->SetBufferCount(uLineCount);
 
 		PutStatus("BufferCount for [" + sChan + "] set to [" + CString(pChan->GetBufferCount()) + "]");
-        } else if (m_pUser->IsAdmin() && sCommand.CaseCmp("TRAFFIC") == 0) {
-                CZNC::Get().UpdateTrafficStats();
-                const map<CString, CUser*>& msUsers = CZNC::Get().GetUserMap();
-                CTable Table;
-                Table.AddColumn("Username");
-                Table.AddColumn("In");
-                Table.AddColumn("Out");
-                Table.AddColumn("Total");
-                unsigned long long users_total_in = 0;
-                unsigned long long users_total_out = 0;
-                for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
-                    Table.AddRow();
-                    Table.SetCell("Username", it->first);
-                    Table.SetCell("In", CString(it->second->BytesRead()));
-                    Table.SetCell("Out", CString(it->second->BytesWritten()));
-                    Table.SetCell("Total", CString(it->second->BytesRead() + it->second->BytesWritten()));
-                    users_total_in += it->second->BytesRead();
-                    users_total_out += it->second->BytesWritten();
-                }
-                Table.AddRow();
-                Table.SetCell("Username", "<Users>");
-                Table.SetCell("In", CString(users_total_in));
-                Table.SetCell("Out", CString(users_total_out));
-                Table.SetCell("Total", CString(users_total_in + users_total_out));
+	} else if (m_pUser->IsAdmin() && sCommand.CaseCmp("TRAFFIC") == 0) {
+		CZNC::Get().UpdateTrafficStats();
+		const map<CString, CUser*>& msUsers = CZNC::Get().GetUserMap();
+		CTable Table;
+		Table.AddColumn("Username");
+		Table.AddColumn("In");
+		Table.AddColumn("Out");
+		Table.AddColumn("Total");
+		unsigned long long users_total_in = 0;
+		unsigned long long users_total_out = 0;
+		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
+			Table.AddRow();
+			Table.SetCell("Username", it->first);
+			Table.SetCell("In", CString::ToByteStr(it->second->BytesRead()));
+			Table.SetCell("Out", CString::ToByteStr(it->second->BytesWritten()));
+			Table.SetCell("Total", CString::ToByteStr(it->second->BytesRead() + it->second->BytesWritten()));
+			users_total_in += it->second->BytesRead();
+			users_total_out += it->second->BytesWritten();
+		}
+		Table.AddRow();
+		Table.SetCell("Username", "<Users>");
+		Table.SetCell("In", CString::ToByteStr(users_total_in));
+		Table.SetCell("Out", CString::ToByteStr(users_total_out));
+		Table.SetCell("Total", CString::ToByteStr(users_total_in + users_total_out));
 
-                Table.AddRow();
-                Table.SetCell("Username", "<ZNC>");
-                Table.SetCell("In", CString(CZNC::Get().BytesRead()));
-                Table.SetCell("Out", CString(CZNC::Get().BytesWritten()));
-                Table.SetCell("Total", CString(CZNC::Get().BytesRead() + CZNC::Get().BytesWritten()));
+		Table.AddRow();
+		Table.SetCell("Username", "<ZNC>");
+		Table.SetCell("In", CString::ToByteStr(CZNC::Get().BytesRead()));
+		Table.SetCell("Out", CString::ToByteStr(CZNC::Get().BytesWritten()));
+		Table.SetCell("Total", CString::ToByteStr(CZNC::Get().BytesRead() + CZNC::Get().BytesWritten()));
 
-                Table.AddRow();
-                Table.SetCell("Username", "<Total>");
-                Table.SetCell("In", CString(users_total_in + CZNC::Get().BytesRead()));
-                Table.SetCell("Out", CString(users_total_out + CZNC::Get().BytesWritten()));
-                Table.SetCell("Total", CString(users_total_in + CZNC::Get().BytesRead() + users_total_out + CZNC::Get().BytesWritten()));
+		Table.AddRow();
+		Table.SetCell("Username", "<Total>");
+		Table.SetCell("In", CString::ToByteStr(users_total_in + CZNC::Get().BytesRead()));
+		Table.SetCell("Out", CString::ToByteStr(users_total_out + CZNC::Get().BytesWritten()));
+		Table.SetCell("Total", CString::ToByteStr(users_total_in + CZNC::Get().BytesRead() + users_total_out + CZNC::Get().BytesWritten()));
 
-                if (Table.size()) {
-                    unsigned int uTableIdx = 0;
-                    CString sLine;
-                    while (Table.GetLine(uTableIdx++, sLine)) {
-                        PutStatus(sLine);
-                    }
-                }
+		if (Table.size()) {
+			unsigned int uTableIdx = 0;
+			CString sTmp;
+			while (Table.GetLine(uTableIdx++, sTmp)) {
+				PutStatus(sTmp);
+			}
+		}
+	} else if (m_pUser->IsAdmin() && sCommand.CaseCmp("UPTIME") == 0) {
+		PutStatus("Running for " + CZNC::Get().GetUptime());
 	} else {
 		PutStatus("Unknown command [" + sCommand + "] try 'Help'");
 	}
@@ -1445,46 +1582,208 @@ void CClient::HelpUser() {
 	Table.AddColumn("Arguments");
 	Table.AddColumn("Description");
 
-	Table.AddRow(); Table.SetCell("Command", "Version");	Table.SetCell("Arguments", "");						Table.SetCell("Description", "Prints which version of znc this is");
-	Table.AddRow(); Table.SetCell("Command", "ListDCCs");	Table.SetCell("Arguments", "");						Table.SetCell("Description", "List all active DCCs");
-	Table.AddRow(); Table.SetCell("Command", "ListMods");	Table.SetCell("Arguments", "");						Table.SetCell("Description", "List all loaded modules");
-	Table.AddRow(); Table.SetCell("Command", "ListChans");	Table.SetCell("Arguments", "");						Table.SetCell("Description", "List all channels");
-	Table.AddRow(); Table.SetCell("Command", "ListNicks");	Table.SetCell("Arguments", "<#chan>");				Table.SetCell("Description", "List all nicks on a channel");
+	Table.AddRow();
+	Table.SetCell("Command", "Version");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "Prints which version of znc this is");
+
+	Table.AddRow();
+	Table.SetCell("Command", "MOTD");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "Show the message of the day");
+
+	Table.AddRow();
+	Table.SetCell("Command", "ListDCCs");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "List all active DCCs");
+
+	Table.AddRow();
+	Table.SetCell("Command", "ListMods");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "List all loaded modules");
+
+	Table.AddRow();
+	Table.SetCell("Command", "ListAvailMods");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "List all available modules");
+
+	Table.AddRow();
+	Table.SetCell("Command", "ListChans");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "List all channels");
+
+	Table.AddRow();
+	Table.SetCell("Command", "ListNicks");
+	Table.SetCell("Arguments", "<#chan>");
+	Table.SetCell("Description", "List all nicks on a channel");
 
 	if (!m_pUser->IsAdmin()) { // If they are an admin we will add this command below with an argument
-		Table.AddRow(); Table.SetCell("Command", "ListClients");Table.SetCell("Arguments", "");					Table.SetCell("Description", "List all clients connected to your znc user");
+		Table.AddRow();
+		Table.SetCell("Command", "ListClients");
+		Table.SetCell("Arguments", "");
+		Table.SetCell("Description", "List all clients connected to your znc user");
 	}
 
-	Table.AddRow(); Table.SetCell("Command", "ListServers");Table.SetCell("Arguments", "");						Table.SetCell("Description", "List all servers");
-	Table.AddRow(); Table.SetCell("Command", "AddServer");	Table.SetCell("Arguments", "<host> [[+]port] [pass]");	Table.SetCell("Description", "Add a server to the list");
-	Table.AddRow(); Table.SetCell("Command", "RemServer");	Table.SetCell("Arguments", "<host>");				Table.SetCell("Description", "Remove a server from the list");
-	Table.AddRow(); Table.SetCell("Command", "Topics");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Show topics in all channels");
-	Table.AddRow(); Table.SetCell("Command", "PlayBuffer");	Table.SetCell("Arguments", "<#chan>");				Table.SetCell("Description", "Play back the buffer for a given channel");
-	Table.AddRow(); Table.SetCell("Command", "ClearBuffer");Table.SetCell("Arguments", "<#chan>");				Table.SetCell("Description", "Clear the buffer for a given channel");
-	Table.AddRow(); Table.SetCell("Command", "SetBuffer");	Table.SetCell("Arguments", "<#chan> [linecount]");	Table.SetCell("Description", "Set the buffer count for a channel");
-	Table.AddRow(); Table.SetCell("Command", "SetVHost");	Table.SetCell("Arguments", "<vhost (ip preferred)>");Table.SetCell("Description", "Set the VHost for this connection");
-	Table.AddRow(); Table.SetCell("Command", "Jump");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Jump to the next server in the list");
-	Table.AddRow(); Table.SetCell("Command", "Disconnect");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Disconnect from IRC");
-	Table.AddRow(); Table.SetCell("Command", "Connect");		Table.SetCell("Arguments", "");						Table.SetCell("Description", "Reconnect to IRC");
-	Table.AddRow(); Table.SetCell("Command", "Send");		Table.SetCell("Arguments", "<nick> <file>");		Table.SetCell("Description", "Send a shell file to a nick on IRC");
-	Table.AddRow(); Table.SetCell("Command", "Get");		Table.SetCell("Arguments", "<file>");				Table.SetCell("Description", "Send a shell file to yourself");
+	Table.AddRow();
+	Table.SetCell("Command", "ListServers");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "List all servers");
+
+	Table.AddRow();
+	Table.SetCell("Command", "AddServer");
+	Table.SetCell("Arguments", "<host> [[+]port] [pass]");
+	Table.SetCell("Description", "Add a server to the list");
+
+	Table.AddRow();
+	Table.SetCell("Command", "RemServer");
+	Table.SetCell("Arguments", "<host>");
+	Table.SetCell("Description", "Remove a server from the list");
+
+	Table.AddRow();
+	Table.SetCell("Command", "Enablechan");
+	Table.SetCell("Arguments", "<#chan>");
+	Table.SetCell("Description", "Enable the channel");
+
+	Table.AddRow();
+	Table.SetCell("Command", "Detach");
+	Table.SetCell("Arguments", "<#chan>");
+	Table.SetCell("Description", "Detach from the channel");
+
+	Table.AddRow();
+	Table.SetCell("Command", "Topics");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "Show topics in all channels");
+
+	Table.AddRow();
+	Table.SetCell("Command", "PlayBuffer");
+	Table.SetCell("Arguments", "<#chan>");
+	Table.SetCell("Description", "Play back the buffer for a given channel");
+
+	Table.AddRow();
+	Table.SetCell("Command", "ClearBuffer");
+	Table.SetCell("Arguments", "<#chan>");
+	Table.SetCell("Description", "Clear the buffer for a given channel");
+
+	Table.AddRow();
+	Table.SetCell("Command", "ClearAllChannelBuffers");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "Clear the channel buffers");
+
+	Table.AddRow();
+	Table.SetCell("Command", "SetBuffer");
+	Table.SetCell("Arguments", "<#chan> [linecount]");
+	Table.SetCell("Description", "Set the buffer count for a channel");
+
+	if (m_pUser->IsAdmin() || !m_pUser->DenySetVHost()) {
+		Table.AddRow();
+		Table.SetCell("Command", "SetVHost");
+		Table.SetCell("Arguments", "<vhost (ip preferred)>");
+		Table.SetCell("Description", "Set the VHost for this connection");
+
+		Table.AddRow();
+		Table.SetCell("Command", "ClearVHost");
+		Table.SetCell("Arguments", "");
+		Table.SetCell("Description", "Clear the VHost for this connection");
+	}
+
+	Table.AddRow();
+	Table.SetCell("Command", "Jump");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "Jump to the next server in the list");
+
+	Table.AddRow();
+	Table.SetCell("Command", "Disconnect");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "Disconnect from IRC");
+
+	Table.AddRow();
+	Table.SetCell("Command", "Connect");
+	Table.SetCell("Arguments", "");
+	Table.SetCell("Description", "Reconnect to IRC");
+
+	Table.AddRow();
+	Table.SetCell("Command", "Send");
+	Table.SetCell("Arguments", "<nick> <file>");
+	Table.SetCell("Description", "Send a shell file to a nick on IRC");
+
+	Table.AddRow();
+	Table.SetCell("Command", "Get");
+	Table.SetCell("Arguments", "<file>");
+	Table.SetCell("Description", "Send a shell file to yourself");
 
 	if (m_pUser) {
 		if (!m_pUser->DenyLoadMod()) {
-			Table.AddRow(); Table.SetCell("Command", "LoadMod");	Table.SetCell("Arguments", "<module>");		Table.SetCell("Description", "Load a module");
-			Table.AddRow(); Table.SetCell("Command", "UnloadMod");	Table.SetCell("Arguments", "<module>");		Table.SetCell("Description", "Unload a module");
-			Table.AddRow(); Table.SetCell("Command", "ReloadMod");	Table.SetCell("Arguments", "<module>");		Table.SetCell("Description", "Reload a module");
+			Table.AddRow();
+			Table.SetCell("Command", "LoadMod");
+			Table.SetCell("Arguments", "<module>");
+			Table.SetCell("Description", "Load a module");
+
+			Table.AddRow();
+			Table.SetCell("Command", "UnloadMod");
+			Table.SetCell("Arguments", "<module>");
+			Table.SetCell("Description", "Unload a module");
+
+			Table.AddRow();
+			Table.SetCell("Command", "ReloadMod");
+			Table.SetCell("Arguments", "<module>");
+			Table.SetCell("Description", "Reload a module");
 		}
 
 		if (m_pUser->IsAdmin()) {
-			Table.AddRow(); Table.SetCell("Command", "SaveConfig");	Table.SetCell("Arguments", "");				Table.SetCell("Description", "Save the current settings to disk");
-			Table.AddRow(); Table.SetCell("Command", "ListUsers");	Table.SetCell("Arguments", "");				Table.SetCell("Description", "List all users/clients connected to znc");
-			Table.AddRow(); Table.SetCell("Command", "ListClients");Table.SetCell("Arguments", "[User]");		Table.SetCell("Description", "List all clients connected to your znc user");
-			Table.AddRow(); Table.SetCell("Command", "SetMOTD");	Table.SetCell("Arguments", "<Message>");	Table.SetCell("Description", "Set the message of the day");
-			Table.AddRow(); Table.SetCell("Command", "AddMOTD");	Table.SetCell("Arguments", "<Message>");	Table.SetCell("Description", "Append <Message> to MOTD");
-			Table.AddRow(); Table.SetCell("Command", "ClearMOTD");	Table.SetCell("Arguments", "");				Table.SetCell("Description", "Clear the MOTD");
-			Table.AddRow(); Table.SetCell("Command", "Broadcast");	Table.SetCell("Arguments", "[message]");	Table.SetCell("Description", "Broadcast a message to all users");
-			Table.AddRow(); Table.SetCell("Command", "Shutdown");	Table.SetCell("Arguments", "[message]");	Table.SetCell("Description", "Shutdown znc completely");
+			Table.AddRow();
+			Table.SetCell("Command", "Rehash");
+			Table.SetCell("Arguments", "");
+			Table.SetCell("Description", "Reload znc.conf from disk");
+
+			Table.AddRow();
+			Table.SetCell("Command", "SaveConfig");
+			Table.SetCell("Arguments", "");
+			Table.SetCell("Description", "Save the current settings to disk");
+
+			Table.AddRow();
+			Table.SetCell("Command", "ListUsers");
+			Table.SetCell("Arguments", "");
+			Table.SetCell("Description", "List all users/clients connected to znc");
+
+			Table.AddRow();
+			Table.SetCell("Command", "ListClients");
+			Table.SetCell("Arguments", "[User]");
+			Table.SetCell("Description", "List all clients connected to your znc user");
+
+			Table.AddRow();
+			Table.SetCell("Command", "Traffic");
+			Table.SetCell("Arguments", "");
+			Table.SetCell("Description", "Show basic traffic stats for all znc users");
+
+			Table.AddRow();
+			Table.SetCell("Command", "Uptime");
+			Table.SetCell("Arguments", "");
+			Table.SetCell("Description", "Show how long ZNC is already running");
+
+			Table.AddRow();
+			Table.SetCell("Command", "SetMOTD");
+			Table.SetCell("Arguments", "<Message>");
+			Table.SetCell("Description", "Set the message of the day");
+
+			Table.AddRow();
+			Table.SetCell("Command", "AddMOTD");
+			Table.SetCell("Arguments", "<Message>");
+			Table.SetCell("Description", "Append <Message> to MOTD");
+
+			Table.AddRow();
+			Table.SetCell("Command", "ClearMOTD");
+			Table.SetCell("Arguments", "");
+			Table.SetCell("Description", "Clear the MOTD");
+
+			Table.AddRow();
+			Table.SetCell("Command", "Broadcast");
+			Table.SetCell("Arguments", "[message]");
+			Table.SetCell("Description", "Broadcast a message to all users");
+
+			Table.AddRow();
+			Table.SetCell("Command", "Shutdown");
+			Table.SetCell("Arguments", "[message]");
+			Table.SetCell("Description", "Shutdown znc completely");
 		}
 	}
 
@@ -1526,27 +1825,12 @@ void CClient::AuthUser() {
 
 	m_spAuth = new CClientAuth(this, m_sUser, m_sPass);
 
-	CClientAuth::AuthUser(m_spAuth);
+	CZNC::Get().AuthUser(m_spAuth);
 }
 
-void CAuthBase::AuthUser(CSmartPtr<CAuthBase> AuthClass) {
-#ifdef _MODULES
-	if (CZNC::Get().GetModules().OnLoginAttempt(AuthClass)) {
-		return;
-	}
-#endif
-
-	CUser* pUser = CZNC::Get().GetUser(AuthClass->GetUsername());
-
-	if (pUser && pUser->CheckPass(AuthClass->GetPassword())) {
-		AuthClass->AcceptLogin(*pUser);
-	} else {
-		if (pUser) {
-			pUser->PutStatus("Another client attempted to login as you, with a bad password.");
-		}
-
-		AuthClass->RefuseLogin("Invalid Password");
-	}
+CClientAuth::CClientAuth(CClient* pClient, const CString& sUsername, const CString& sPassword)
+		: CAuthBase(sUsername, sPassword, pClient->GetRemoteIP()) {
+	m_pClient = pClient;
 }
 
 void CClientAuth::RefuseLogin(const CString& sReason) {
@@ -1554,11 +1838,16 @@ void CClientAuth::RefuseLogin(const CString& sReason) {
 		m_pClient->RefuseLogin(sReason);
 	}
 #ifdef _MODULES
-	CZNC::Get().GetModules().OnFailedLogin(GetUsername(), ((m_pClient) ? m_pClient->GetRemoteIP() : CString("")));
+	CZNC::Get().GetModules().OnFailedLogin(GetUsername(), GetRemoteIP());
 #endif
 }
 
 void CClient::RefuseLogin(const CString& sReason) {
+	if (m_pTimeout) {
+		m_pTimeout->Stop();
+		m_pTimeout = NULL;
+	}
+
 	PutStatus("Bad username and/or password.");
 	PutClient(":irc.znc.com 464 " + GetNick() + " :" + sReason);
 	Close();
@@ -1574,14 +1863,11 @@ void CClient::AcceptLogin(CUser& User) {
 	m_sPass = "";
 	m_pUser = &User;
 
-	if (!m_pUser->IsHostAllowed(GetRemoteIP())) {
-		PutClient(":irc.znc.com 463 " + GetNick() + " :Your host ("
-				+ GetRemoteIP() + ") is not allowed");
-		Close();
-		return;
+	if (m_pTimeout) {
+		m_pTimeout->Stop();
+		m_pTimeout = NULL;
 	}
 
-	m_bAuthed = true;
 	SetSockName("USR::" + m_pUser->GetUserName());
 
 	m_pIRCSock = (CIRCSock*) CZNC::Get().FindSockByName("IRC::" + m_pUser->GetUserName());
@@ -1589,9 +1875,21 @@ void CClient::AcceptLogin(CUser& User) {
 
 	SendMotd();
 
-#ifdef _MODULES
 	MODULECALL(OnUserAttached(), m_pUser, this, );
-#endif
+}
+
+void CClient::StartLoginTimeout() {
+	m_pTimeout = new CClientTimeout(this);
+	CZNC::Get().GetManager().AddCron(m_pTimeout);
+}
+
+void CClient::LoginTimeout() {
+	PutClient("ERROR :Closing link [Timeout]");
+	Close(Csock::CLT_AFTERWRITE);
+	if (m_pTimeout) {
+		m_pTimeout->Stop();
+		m_pTimeout = NULL;
+	}
 }
 
 void CClient::Connected() {
@@ -1610,9 +1908,7 @@ void CClient::Disconnected() {
 
 	m_pIRCSock = NULL;
 
-#ifdef _MODULES
 	MODULECALL(OnUserDetached(), m_pUser, this, );
-#endif
 }
 
 void CClient::IRCConnected(CIRCSock* pIRCSock) {
@@ -1631,6 +1927,7 @@ void CClient::IRCDisconnected() {
 
 Csock* CClient::GetSockObj(const CString& sHost, unsigned short uPort) {
 	CClient* pSock = new CClient(sHost, uPort);
+	pSock->StartLoginTimeout();
 	return pSock;
 }
 
@@ -1674,7 +1971,7 @@ void CClient::PutModule(const CString& sModule, const CString& sLine) {
 CString CClient::GetNick(bool bAllowIRCNick) const {
 	CString sRet;
 
-	if ((bAllowIRCNick) && (m_bAuthed) && (m_pIRCSock)) {
+	if ((bAllowIRCNick) && (IsAttached()) && (m_pIRCSock)) {
 		sRet = m_pIRCSock->GetNick();
 	}
 
