@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.61 $
+* $Revision: 1.80 $
 */
 
 #include "Csocket.h"
@@ -36,6 +36,8 @@
 #include <sys/param.h>
 #endif /* __NetBSD__ */
 
+#include <list>
+
 #define CS_SRANDBUFFER 128
 
 using namespace std;
@@ -51,6 +53,52 @@ int GetCsockClassIdx()
 	return( g_iCsockSSLIdx );
 }
 
+#ifdef _WIN32
+static inline void set_non_blocking(int fd)
+{
+	u_long iOpts = 1;
+	ioctlsocket( fd, FIONBIO, &iOpts );
+}
+
+static inline void set_blocking(int fd)
+{
+	u_long iOpts = 0;
+	ioctlsocket( fd, FIONBIO, &iOpts );
+}
+
+static inline void set_close_on_exec(int fd)
+{
+	// TODO add this for windows
+	// see http://gcc.gnu.org/ml/java-patches/2002-q1/msg00696.html
+	// for infos on how to do this
+}
+#else
+static inline void set_non_blocking(int fd)
+{
+	int fdflags = fcntl(fd, F_GETFL, 0);
+	if ( fdflags < 0 )
+		return; // Ignore errors
+	fcntl( fd, F_SETFL, fdflags|O_NONBLOCK );
+}
+
+static inline void set_blocking(int fd)
+{
+	int fdflags = fcntl(fd, F_GETFL, 0);
+	if ( fdflags < 0 )
+		return; // Ignore errors
+	fdflags &= ~O_NONBLOCK;
+	fcntl( fd, F_SETFL, fdflags );
+}
+
+static inline void set_close_on_exec(int fd)
+{
+	int fdflags = fcntl(fd, F_GETFD, 0);
+	if ( fdflags < 0 )
+		return; // Ignore errors
+	fcntl( fd, F_SETFD, fdflags|FD_CLOEXEC);
+}
+#endif /* _WIN32 */
+
 #ifdef HAVE_LIBSSL
 Csock *GetCsockFromCTX( X509_STORE_CTX *pCTX )
 {
@@ -126,12 +174,13 @@ static int __GetHostByName( const CS_STRING & sHostName, struct in_addr *paddr,
 int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockAddr )
 {
 #ifndef HAVE_IPV6
+	// if ipv6 is not enabled, then simply use gethostbyname, nothing special outside of this is done
 	if( pSock )
 		pSock->SetIPv6( false );
 	csSockAddr.SetIPv6( false );
 	if( __GetHostByName( sHostname, csSockAddr.GetAddr(), 3 ) == 0 )
 		return( 0 );
-	
+
 #else /* HAVE_IPV6 */
 	struct addrinfo *res = NULL;
 	struct addrinfo hints;
@@ -140,197 +189,102 @@ int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockA
 
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_protocol = IPPROTO_TCP;
+#ifdef AI_ADDRCONFIG
+	// this is suppose to eliminate host from appearing that this system can not support
+	hints.ai_flags = AI_ADDRCONFIG;
+#endif /* AI_ADDRCONFIG */
+
+	if( pSock && ( pSock->GetType() == Csock::LISTENER || pSock->GetConState() == Csock::CST_BINDVHOST ) )
+	{ // when doing a dns for bind only, set the AI_PASSIVE flag as suggested by the man page
+		hints.ai_flags |= AI_PASSIVE;
+	}
+
 	int iRet = getaddrinfo( sHostname.c_str(), NULL, &hints, &res );
 	if( iRet == EAI_AGAIN )
-		return( EAGAIN );
+		return( EAGAIN ); // need to return telling the user to try again
 	else if( ( iRet == 0 ) && ( res ) )
-	{
-		struct addrinfo *pUseAddr = NULL;
-#ifdef __RANDOMIZE_SOURCE_ADDRESSES
-		std::vector< struct addrinfo *> vHostCanidates;
-#endif /* __RANDOMIZE_SOURCE_ADDRESSES */
+	{ 
+		std::list<struct addrinfo *> lpTryAddrs;
+		bool bFound = false;
 		for( struct addrinfo *pRes = res; pRes; pRes = pRes->ai_next )
-		{
+		{ // pass through the list building out a lean list of candidates to try. AI_CONFIGADDR doesn't always seem to work
 #ifdef __sun
 			if( ( pRes->ai_socktype != SOCK_STREAM ) || ( pRes->ai_protocol != IPPROTO_TCP && pRes->ai_protocol != IPPROTO_IP ) )
 #else
 			if( ( pRes->ai_socktype != SOCK_STREAM ) || ( pRes->ai_protocol != IPPROTO_TCP ) )
 #endif /* __sun work around broken impl of getaddrinfo */
 				continue;
-
+			
 			if( ( csSockAddr.GetAFRequire() != CSSockAddr::RAF_ANY ) && ( pRes->ai_family != csSockAddr.GetAFRequire() ) )
 				continue; // they requested a special type, so be certain we woop past anything unwanted
-
+			lpTryAddrs.push_back( pRes );
+		}
+		for( std::list<struct addrinfo *>::iterator it = lpTryAddrs.begin(); it != lpTryAddrs.end();  )
+		{ // cycle through these, leaving the last iterator for the outside caller to call, so if there is an error it can call the events
+			struct addrinfo * pRes = *it;
+			bool bTryConnect = false;
 			if( pRes->ai_family == AF_INET )
 			{
-#ifdef __RANDOMIZE_SOURCE_ADDRESSES
-				vHostCanidates.push_back( pRes );
-#else
-				pUseAddr = pRes;
-#endif /* __RANDOMIZE_SOURCE_ADDRESSES */
+				if( pSock )
+					pSock->SetIPv6( false );
+				csSockAddr.SetIPv6( false );
+				struct sockaddr_in *pTmp = (struct sockaddr_in *)pRes->ai_addr;
+				memcpy( csSockAddr.GetAddr(), &(pTmp->sin_addr), sizeof( *(csSockAddr.GetAddr()) ) );
+				if( pSock && pSock->GetConState() == Csock::CST_DESTDNS && pSock->GetType() == Csock::OUTBOUND )
+				{
+					bTryConnect = true;
+				}
+				else
+				{
+					bFound = true;
+					break;
+				}
 			}
 			else if( pRes->ai_family == AF_INET6 )
 			{
-#ifdef __RANDOMIZE_SOURCE_ADDRESSES
-				vHostCanidates.push_back( pRes );
-#else
-				pUseAddr = pRes;
-#endif /* __RANDOMIZE_SOURCE_ADDRESSES */
+				if( pSock )
+					pSock->SetIPv6( true );
+				csSockAddr.SetIPv6( true );
+				struct sockaddr_in6 *pTmp = (struct sockaddr_in6 *)pRes->ai_addr;
+				memcpy( csSockAddr.GetAddr6(), &(pTmp->sin6_addr), sizeof( *(csSockAddr.GetAddr6()) ) );
+				if( pSock && pSock->GetConState() == Csock::CST_DESTDNS && pSock->GetType() == Csock::OUTBOUND )
+				{
+					bTryConnect = true;
+				}
+				else
+				{
+					bFound = true;
+					break;
+				}
 			}
-		}
-#ifdef __RANDOMIZE_SOURCE_ADDRESSES
-		if( vHostCanidates.size() > 1 )
-		{
-			// this is basically where getaddrinfo() sorts the list of results. basically to help out what round robin dns does,
-			// pick a random canidate to make this work
-			struct random_data cRandData;
-			char chState[CS_SRANDBUFFER];
-			int32_t iNumber = 0; 
-			if( initstate_r( (u_int)millitime(), chState, CS_SRANDBUFFER, &cRandData ) == 0 && random_r( &cRandData, &iNumber ) == 0 )
-			{
-		   		iNumber %= (int)vHostCanidates.size();
-		   		pUseAddr = vHostCanidates[iNumber];
-			}
-			else
-			{
-				CS_DEBUG( "initstate_r/random_r failed" );
-				pUseAddr = vHostCanidates[0];
-			}
-		}
-		else if( vHostCanidates.size() )
-		{
-			pUseAddr = vHostCanidates[0];
-		}
-#endif /* __RANDOMIZE_SOURCE_ADDRESSES */
 
-		if( pUseAddr && pUseAddr->ai_family == AF_INET )
-		{
-			if( pSock )
-				pSock->SetIPv6( false );
-			csSockAddr.SetIPv6( false );
-			struct sockaddr_in *pTmp = (struct sockaddr_in *)pUseAddr->ai_addr;
-			memcpy( csSockAddr.GetAddr(), &(pTmp->sin_addr), sizeof( *(csSockAddr.GetAddr()) ) );
-		}
-		else if( pUseAddr )
-		{
-			if( pSock )
-				pSock->SetIPv6( true );
-			csSockAddr.SetIPv6( true );
-			struct sockaddr_in6 *pTmp = (struct sockaddr_in6 *)pUseAddr->ai_addr;
-			memcpy( csSockAddr.GetAddr6(), &(pTmp->sin6_addr), sizeof( *(csSockAddr.GetAddr6()) ) );
+			it++; // increment the iterator her so we know if its the last element or not
+
+			if( bTryConnect && it != lpTryAddrs.end() )
+			{ // save the last attempt for the outer loop, the issue then becomes that the error is thrown on the last failure
+				if( pSock->CreateSocksFD() && pSock->Connect( pSock->GetBindHost(), true ) )
+				{
+					pSock->SetSkipConnect( true ); // this tells the socket that the connection state has been started
+					bFound = true;
+					break;
+				}
+			}
+			else if( bTryConnect )
+			{
+				bFound = true;
+			}
 		}
+
 		freeaddrinfo( res );
-		if( pUseAddr ) // the data pointed to here is invalid now, but the pointer itself is a good test
+		if( bFound ) // the data pointed to here is invalid now, but the pointer itself is a good test
+		{
 			return( 0 );
+		}
 	}
 #endif /* ! HAVE_IPV6 */
 	return( ETIMEDOUT );
 }
 
-#ifdef ___DO_THREADS
-CSMutex::CSMutex() 
-{
-	pthread_mutexattr_init( &m_mattrib );
-	if ( pthread_mutexattr_settype( &m_mattrib, PTHREAD_MUTEX_FAST_NP ) != 0 )
-		throw CS_STRING( "ERROR: pthread_mutexattr_settype failed!" );
-		
-	if ( pthread_mutex_init( &m_mutex, &m_mattrib ) != 0 )
-		throw CS_STRING( "ERROR: pthread_mutex_init failed!" );
-}
-
-CSMutex::~CSMutex() 
-{
-	pthread_mutexattr_destroy( &m_mattrib );
-	pthread_mutex_destroy( &m_mutex );
-}
-
-bool CSThread::start()
-{
-	// mark the job as running
-	lock();
-	m_eStatus = RUNNING;
-	unlock();
-
-	pthread_attr_t attr;
-	if ( pthread_attr_init( &attr ) != 0 )
-	{
-		WARN( "pthread_attr_init failed" );
-		lock();
-		m_eStatus = FINISHED;
-		unlock();
-		return( false );
-	}
-
-	if ( pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED ) != 0 )
-	{
-		WARN( "pthread_attr_setdetachstate failed" );
-		lock();
-		m_eStatus = FINISHED;
-		unlock();
-		return( false );
-	}
-
-	int iRet = pthread_create( &m_ppth, &attr, start_thread, this );
-	if ( iRet != 0 )
-	{
-		WARN( "pthread_create failed " );
-		lock();
-		m_eStatus = FINISHED;
-		unlock();
-		return( false );
-	}
-
-	return( true );
-}		
-
-void CSThread::wait()
-{
-	while( true )
-	{
-		lock();
-		EStatus e = Status();
-		unlock();
-		if ( e == FINISHED )
-			break;
-		usleep( 100 );
-	}
-}
-
-void *CSThread::start_thread( void *args )
-{
-	CSThread *curThread = (CSThread *)args;
-	curThread->run();
-	curThread->lock();
-	curThread->SetStatus( CSThread::FINISHED );
-	curThread->unlock();
-	pthread_exit( NULL );   
-}
-
-void CDNSResolver::Lookup( const CS_STRING & sHostname )
-{
-	m_bSuccess = false;
-	m_sHostname = sHostname;
-	start();
-}
-
-void CDNSResolver::run()
-{
-	m_bSuccess = false;
-	if( GetAddrInfo( m_sHostname, NULL, m_cSockAddr ) == 0 )
-		m_bSuccess = true;
-}
-
-bool CDNSResolver::IsCompleted()
-{
-	lock();
-	EStatus e = Status();
-	unlock();
-	if ( e == FINISHED )
-		return( true );
-	return( false );
-}
-
-#endif /* ___DO_THREADS */
 #ifdef HAVE_LIBSSL
 bool InitSSL( ECompType eCompressionType )
 {
@@ -498,9 +452,6 @@ Csock::Csock( int itimeout )
 #ifdef HAVE_LIBSSL
 	m_pCerVerifyCB = NULL;
 #endif /* HAVE_LIBSSL */
-#ifdef ___DO_THREADS
-	m_pResolver = NULL;
-#endif /* ___DO_THREADS */
 	Init( "", 0, itimeout );
 }
 
@@ -509,9 +460,6 @@ Csock::Csock( const CS_STRING & sHostname, u_short iport, int itimeout )
 #ifdef HAVE_LIBSSL
 	m_pCerVerifyCB = NULL;
 #endif /* HAVE_LIBSSL */
-#ifdef ___DO_THREADS
-	m_pResolver = NULL;
-#endif /* ___DO_THREADS */
 	Init( sHostname, iport, itimeout );
 }
 
@@ -529,17 +477,10 @@ Csock *Csock::GetSockObj( const CS_STRING & sHostname, u_short iPort )
 
 Csock::~Csock()
 {
-#ifdef ___DO_THREADS
-	if( m_pResolver )
-	{
-		m_pResolver->lock();
-		CDNSResolver::EStatus eStatus = m_pResolver->Status();
-		m_pResolver->unlock();
-		if ( eStatus == CDNSResolver::RUNNING )
-			m_pResolver->cancel();
-		Zzap( m_pResolver );
-	}
-#endif /* __DO_THREADS_ */
+#ifdef HAVE_LIBSSL
+	FREE_SSL();
+	FREE_CTX();
+#endif /* HAVE_LIBSSL */
 
 	if ( m_iReadSock != m_iWriteSock )
 	{
@@ -553,11 +494,6 @@ Csock::~Csock()
 	m_iReadSock = -1;
 	m_iWriteSock = -1;
 
-#ifdef HAVE_LIBSSL
-	FREE_SSL();
-	FREE_CTX();
-
-#endif /* HAVE_LIBSSL */
 	// delete any left over crons
 	for( vector<CCron *>::size_type i = 0; i < m_vcCrons.size(); i++ )
 		CS_Delete( m_vcCrons[i] );
@@ -565,9 +501,6 @@ Csock::~Csock()
 
 void Csock::Dereference()
 {
-#ifdef ___DO_THREADS
-	m_pResolver = NULL;
-#endif /* __DO_THREADS_ */
 	m_iWriteSock = m_iReadSock = -1;
 
 #ifdef HAVE_LIBSSL
@@ -583,49 +516,51 @@ void Csock::Copy( const Csock & cCopy )
 	m_iTcount		= cCopy.m_iTcount;
 	m_iLastCheckTimeoutTime	=	cCopy.m_iLastCheckTimeoutTime;
 	m_iport 		= cCopy.m_iport;
-   	m_iRemotePort	= cCopy.m_iRemotePort;
+	m_iRemotePort	= cCopy.m_iRemotePort;
 	m_iLocalPort	= cCopy.m_iLocalPort;
 	m_iReadSock		= cCopy.m_iReadSock;
 	m_iWriteSock	= cCopy.m_iWriteSock;
-   	m_itimeout		= cCopy.m_iWriteSock;
-   	m_iConnType		= cCopy.m_iConnType;
+	m_itimeout		= cCopy.m_iWriteSock;
+	m_iConnType		= cCopy.m_iConnType;
 	m_iMethod		= cCopy.m_iMethod;
 	m_bssl			= cCopy.m_bssl;
 	m_bIsConnected	= cCopy.m_bIsConnected;
 	m_bBLOCK		= cCopy.m_bBLOCK;
 	m_bFullsslAccept	= cCopy.m_bFullsslAccept;
 	m_bsslEstablished	= cCopy.m_bsslEstablished;
-   	m_bEnableReadLine	= cCopy.m_bEnableReadLine;
-   	m_bRequireClientCert = cCopy.m_bRequireClientCert;
-   	m_bPauseRead		= cCopy.m_bPauseRead;
+	m_bEnableReadLine	= cCopy.m_bEnableReadLine;
+	m_bRequireClientCert = cCopy.m_bRequireClientCert;
+	m_bPauseRead		= cCopy.m_bPauseRead;
 	m_shostname		= cCopy.m_shostname;
-   	m_sbuffer		= cCopy.m_sbuffer;
-   	m_sSockName		= cCopy.m_sSockName;
-   	m_sPemFile		= cCopy.m_sPemFile;
-   	m_sCipherType	= cCopy.m_sCipherType;
-   	m_sParentName	= cCopy.m_sParentName;
+	m_sbuffer		= cCopy.m_sbuffer;
+	m_sSockName		= cCopy.m_sSockName;
+	m_sPemFile		= cCopy.m_sPemFile;
+	m_sCipherType	= cCopy.m_sCipherType;
+	m_sParentName	= cCopy.m_sParentName;
 	m_sSend			= cCopy.m_sSend;
-   	m_sSSLBuffer	= cCopy.m_sSSLBuffer;
-   	m_sPemPass		= cCopy.m_sPemPass;
-   	m_sLocalIP		= cCopy.m_sLocalIP;
-   	m_sRemoteIP		= cCopy.m_sRemoteIP;
+	m_sPemPass		= cCopy.m_sPemPass;
+	m_sLocalIP		= cCopy.m_sLocalIP;
+	m_sRemoteIP		= cCopy.m_sRemoteIP;
 	m_eCloseType	= cCopy.m_eCloseType;
 
 	m_iMaxMilliSeconds	= cCopy.m_iMaxMilliSeconds;
-   	m_iLastSendTime		= cCopy.m_iLastSendTime;
-   	m_iBytesRead		= cCopy.m_iBytesRead;
-   	m_iBytesWritten		= cCopy.m_iBytesWritten;
-   	m_iStartTime		= cCopy.m_iStartTime;
+	m_iLastSendTime		= cCopy.m_iLastSendTime;
+	m_iBytesRead		= cCopy.m_iBytesRead;
+	m_iBytesWritten		= cCopy.m_iBytesWritten;
+	m_iStartTime		= cCopy.m_iStartTime;
 	m_iMaxBytes			= cCopy.m_iMaxBytes;
-   	m_iLastSend			= cCopy.m_iLastSend;
-   	m_iMaxStoredBufferLength	= cCopy.m_iMaxStoredBufferLength;
-   	m_iTimeoutType		= cCopy.m_iTimeoutType;
+	m_iLastSend			= cCopy.m_iLastSend;
+	m_iMaxStoredBufferLength	= cCopy.m_iMaxStoredBufferLength;
+	m_iTimeoutType		= cCopy.m_iTimeoutType;
 
 	m_address			= cCopy.m_address;
-   	m_bindhost			= cCopy.m_bindhost;
+	m_bindhost			= cCopy.m_bindhost;
 	m_bIsIPv6			= cCopy.m_bIsIPv6;
+	m_bSkipConnect		= cCopy.m_bSkipConnect;
 
 #ifdef HAVE_LIBSSL
+	m_sSSLBuffer	= cCopy.m_sSSLBuffer;
+
 	FREE_SSL();
 	FREE_CTX(); // be sure to remove anything that was already here
 	m_ssl				= cCopy.m_ssl;
@@ -649,13 +584,8 @@ void Csock::Copy( const Csock & cCopy )
 	m_eConState			= cCopy.m_eConState;
 	m_sBindHost			= cCopy.m_sBindHost;
 	m_iCurBindCount		= cCopy.m_iCurBindCount;
-   	m_iDNSTryCount		= cCopy.m_iDNSTryCount;
+	m_iDNSTryCount		= cCopy.m_iDNSTryCount;
 
-#ifdef ___DO_THREADS
-	if( m_pResolver )
-		CS_Delete( m_pResolver );
-	m_pResolver			= cCopy.m_pResolver;
-#endif /* ___DO_THREADS */
 }
 
 Csock & Csock::operator<<( const CS_STRING & s )
@@ -722,22 +652,18 @@ Csock & Csock::operator<<( double i )
 
 bool Csock::Connect( const CS_STRING & sBindHost, bool bSkipSetup )
 {
+	if( m_bSkipConnect )
+	{ // this was already called, so skipping now. this is to allow easy pass through
+		if ( m_eConState != CST_OK )
+		{
+			m_eConState = ( GetSSL() ? CST_CONNECTSSL : CST_OK );
+		}
+		return( true );
+	}
 	// bind to a hostname if requested
 	m_sBindHost = sBindHost;
 	if ( !bSkipSetup )
 	{
-		int iDNSRet = ETIMEDOUT;
-		while( true )
-		{
-			iDNSRet = DNSLookup( DNS_VHOST );
-			if ( iDNSRet == EAGAIN )
-				continue;
-
-			break;
-		}
-		if ( iDNSRet != 0 )
-			return( false );
-
 		if ( !sBindHost.empty() )
 		{
 			// try to bind 3 times, otherwise exit failure
@@ -759,16 +685,23 @@ bool Csock::Connect( const CS_STRING & sBindHost, bool bSkipSetup )
 				return( false );
 			}
 		}
+
+		int iDNSRet = ETIMEDOUT;
+		while( true )
+		{
+			iDNSRet = DNSLookup( DNS_VHOST );
+			if ( iDNSRet == EAGAIN )
+				continue;
+
+			break;
+		}
+		if ( iDNSRet != 0 )
+			return( false );
+
 	}
 
 	// set it none blocking
-#ifdef _WIN32
-	u_long iOpts = 1;
-	ioctlsocket( m_iReadSock, FIONBIO, &iOpts );
-#else
-	int fdflags = fcntl (m_iReadSock, F_GETFL, 0);
-	fcntl( m_iReadSock, F_SETFL, fdflags|O_NONBLOCK );
-#endif /* _WIN32 */
+	set_non_blocking( m_iReadSock );
 
 	m_iConnType = OUTBOUND;
 
@@ -792,19 +725,13 @@ bool Csock::Connect( const CS_STRING & sBindHost, bool bSkipSetup )
 
 	if ( m_bBLOCK )
 	{
-#ifdef _WIN32
-		u_long iOpts = 0;
-		ioctlsocket( m_iReadSock, FIONBIO, &iOpts );
-#else
-		// unset the flags afterwords, rather than have connect block
-		int fdflags = fcntl (m_iReadSock, F_GETFL, 0);
-		fdflags &= ~O_NONBLOCK;
-		fcntl( m_iReadSock, F_SETFL, fdflags );
-#endif /* _WIN32 */
+		set_blocking( m_iReadSock );
 	}
 
 	if ( m_eConState != CST_OK )
-		m_eConState = CST_OK;
+	{
+		m_eConState = ( GetSSL() ? CST_CONNECTSSL : CST_OK );
+	}
 
 	return( true );
 }
@@ -908,13 +835,7 @@ bool Csock::Listen( u_short iPort, int iMaxConns, const CS_STRING & sBindHost, u
 	if ( !m_bBLOCK )
 	{
 		// set it none blocking
-#ifdef _WIN32
-		u_long iOpts = 1;
-		ioctlsocket( m_iReadSock, FIONBIO, &iOpts );
-#else
-		int fdflags = fcntl ( m_iReadSock, F_GETFL, 0);
-		fcntl( m_iReadSock, F_SETFL, fdflags|O_NONBLOCK );
-#endif /* _WIN32 */
+		set_non_blocking( m_iReadSock );
 	}
 
 	return( true );
@@ -948,19 +869,15 @@ int Csock::Accept( CS_STRING & sHost, u_short & iRPort )
 	}
 #endif /* HAVE_IPV6 */
 
-	
 	if ( iSock != -1 )
 	{
+		// Make it close-on-exec
+		set_close_on_exec( iSock );
+
 		if ( !m_bBLOCK )
 		{
-			// make it none blocking
-#ifdef _WIN32
-			u_long iOpts = 1;
-			ioctlsocket( m_iReadSock, FIONBIO, &iOpts );
-#else
-			int fdflags = fcntl (iSock, F_GETFL, 0);
-			fcntl( iSock, F_SETFL, fdflags|O_NONBLOCK );
-#endif /* _WIN32 */
+			// make it none blocking 
+			set_non_blocking( iSock );
 		}
 
 		if ( !ConnectionFrom( sHost, iRPort ) )
@@ -1203,7 +1120,6 @@ bool Csock::ConnectSSL( const CS_STRING & sBindhost )
 	if ( m_iReadSock == -1 )
 		if ( !Connect( sBindhost ) )
 			return( false );
-
 	if ( !m_ssl )
 		if ( !SSLClientSetup() )
 			return( false );
@@ -1212,13 +1128,7 @@ bool Csock::ConnectSSL( const CS_STRING & sBindhost )
 
 	if ( m_bBLOCK )
 	{
-#ifdef _WIN32
-		u_long iOpts = 1;
-		ioctlsocket( m_iReadSock, FIONBIO, &iOpts );
-#else
-		int fdflags = fcntl ( m_iReadSock, F_GETFL, 0);
-		fcntl( m_iReadSock, F_SETFL, fdflags|O_NONBLOCK );
-#endif /* _WIN32 */
+		set_non_blocking( m_iReadSock );
 	}
 
 	int iErr = SSL_connect( m_ssl );
@@ -1234,17 +1144,11 @@ bool Csock::ConnectSSL( const CS_STRING & sBindhost )
 	if ( m_bBLOCK )
 	{
 		// unset the flags afterwords, rather then have connect block
-#ifdef _WIN32
-		u_long iOpts = 0;
-		ioctlsocket( m_iReadSock, FIONBIO, &iOpts );
-#else
-		int fdflags = fcntl (m_iReadSock, F_GETFL, 0);
-		fdflags &= ~O_NONBLOCK;
-		fcntl( m_iReadSock, F_SETFL, fdflags );
-#endif /* _WIN32 */
-
+		set_blocking( m_iReadSock );
 	}
 
+	if ( m_eConState != CST_OK )
+		m_eConState = CST_OK;
 	return( bPass );
 #else
 	return( false );
@@ -1412,7 +1316,6 @@ bool Csock::Write( const CS_STRING & sData )
 int Csock::Read( char *data, int len )
 {
 	int bytes = 0;
-	memset( (char *)data, '\0', len );
 
 	if ( ( IsReadPaused() ) && ( SslIsEstablished() ) )
 		return( READ_EAGAIN ); // allow the handshake to complete first
@@ -1499,7 +1402,7 @@ CS_STRING Csock::GetLocalIP()
 		char straddr[INET6_ADDRSTRLEN];
 		struct sockaddr_in6 mLocalAddr;
 		socklen_t mLocalLen = sizeof( mLocalAddr );
-		if ( ( getsockname( iSock, (struct sockaddr *) &mLocalAddr, &mLocalLen ) == 0 ) 
+		if ( ( getsockname( iSock, (struct sockaddr *) &mLocalAddr, &mLocalLen ) == 0 )
 			&& ( inet_ntop( AF_INET6, &mLocalAddr.sin6_addr, straddr, sizeof(straddr) ) ) )
 		{
 			m_sLocalIP = straddr;
@@ -1536,7 +1439,7 @@ CS_STRING Csock::GetRemoteIP()
 		char straddr[INET6_ADDRSTRLEN];
 		struct sockaddr_in6 mRemoteAddr;
 		socklen_t mRemoteLen = sizeof( mRemoteAddr );
-		if ( ( getpeername( iSock, (struct sockaddr *) &mRemoteAddr, &mRemoteLen ) == 0 ) 
+		if ( ( getpeername( iSock, (struct sockaddr *) &mRemoteAddr, &mRemoteLen ) == 0 )
 			&& ( inet_ntop( AF_INET6, &mRemoteAddr.sin6_addr, straddr, sizeof(straddr) ) ) )
 		{
 			m_sRemoteIP = straddr;
@@ -1596,7 +1499,7 @@ bool Csock::CheckTimeout( time_t iNow )
 		// this is weird, but its possible if someone changes a clock and it went back in time, this essentially has to reset the last check
 		// the worst case scenario is the timeout is about to it and the clock changes, it would then cause
 		// this to pass over the last half the time
-		m_iLastCheckTimeoutTime = iNow; 
+		m_iLastCheckTimeoutTime = iNow;
 	}
 
 	if ( m_itimeout > 0 )
@@ -1632,7 +1535,7 @@ void Csock::PushBuff( const char *data, int len, bool bStartAtZero )
 	if ( data )
 		m_sbuffer.append( data, len );
 
-	while( !m_bPauseRead )
+	while( !m_bPauseRead && GetCloseType() == CLT_DONT  )
 	{
 		CS_STRING::size_type iFind = m_sbuffer.find( "\n", iStartPos );
 
@@ -1652,7 +1555,8 @@ void Csock::PushBuff( const char *data, int len, bool bStartAtZero )
 
 }
 
-CS_STRING & Csock::GetInternalBuffer() { return( m_sbuffer ); }
+CS_STRING & Csock::GetInternalReadBuffer() { return( m_sbuffer ); }
+CS_STRING & Csock::GetInternalWriteBuffer() { return( m_sSend ); }
 void Csock::SetMaxBufferThreshold( u_int iThreshold ) { m_iMaxStoredBufferLength = iThreshold; }
 u_int Csock::GetMaxBufferThreshold() { return( m_iMaxStoredBufferLength ); }
 int Csock::GetType() { return( m_iConnType ); }
@@ -1751,30 +1655,18 @@ u_short Csock::GetLocalPort()
 u_short Csock::GetPort() { return( m_iport ); }
 void Csock::SetPort( u_short iPort ) { m_iport = iPort; }
 void Csock::Close( ECloseType eCloseType )
-{  
+{
 	m_eCloseType = eCloseType;
 }
 void Csock::BlockIO( bool bBLOCK ) { m_bBLOCK = bBLOCK; }
 
 void Csock::NonBlockingIO()
 {
-#ifdef _WIN32
-	u_long iOpts = 1;
-	ioctlsocket( m_iReadSock, FIONBIO, &iOpts );
-#else
-	int fdflags = fcntl ( m_iReadSock, F_GETFL, 0);
-	fcntl( m_iReadSock, F_SETFL, fdflags|O_NONBLOCK );
-#endif /* _WIN32 */
+	set_non_blocking( m_iReadSock );
 
 	if ( m_iReadSock != m_iWriteSock )
 	{
-#ifdef _WIN32
-		iOpts = 1;
-		ioctlsocket( m_iReadSock, FIONBIO, &iOpts );
-#else
-		fdflags = fcntl ( m_iWriteSock, F_GETFL, 0);
-		fcntl( m_iWriteSock, F_SETFL, fdflags|O_NONBLOCK );
-#endif /* _WIN32 */
+		set_non_blocking( m_iWriteSock );
 	}
 
 	BlockIO( false );
@@ -2045,7 +1937,7 @@ int Csock::DNSLookup( EDNSLType eDNSLType )
 		if ( m_sBindHost.empty() )
 		{
 			if ( m_eConState != CST_OK )
-				m_eConState = CST_BINDVHOST;
+				m_eConState = CST_DESTDNS; // skip binding, there is no vhost
 			return( 0 );
 		}
 
@@ -2053,70 +1945,20 @@ int Csock::DNSLookup( EDNSLType eDNSLType )
 		m_bindhost.SinPort( 0 );
 	}
 
-#ifdef ___DO_THREADS
-	if( !m_pResolver )
-		m_pResolver = new CDNSResolver;
-
-	if ( m_iDNSTryCount == 0 )
-	{
-		m_pResolverLookup( ( eDNSLType == DNS_VHOST ) ? m_sBindHost : m_shostname );
-		m_iDNSTryCount++;
-	}
-   
-	if ( m_pResolverIsCompleted() )
-	{
-		m_iDNSTryCount = 0;
-		if ( m_pResolverSuceeded() )
-		{
-			if ( eDNSLType == DNS_VHOST )
-			{
-				if( !m_pResolverGetSockAddr()->GetIPv6() )
-				{
-					SetIPv6( false );
-					memcpy( m_bindhost.GetAddr(), m_pResolverGetSockAddr()->GetAddr(), sizeof( *(m_bindhost.GetAddr()) ) );
-				}
-#ifdef HAVE_IPV6
-				else
-				{
-					SetIPv6( true );
-					memcpy( m_bindhost.GetAddr6(), m_pResolverGetSockAddr()->GetAddr6(), sizeof( *(m_bindhost.GetAddr6()) ) );
-				}
-#endif /* HAVE_IPV6 */
-			}
-			else
-			{
-				if( m_pResolverGetSockAddr()->GetIPv6() )
-				{
-					SetIPv6( false );
-					memcpy( m_address.GetAddr(), m_pResolverGetSockAddr()->GetAddr(), sizeof( *(m_address.GetAddr()) ) );
-				}
-#ifdef HAVE_IPV6
-				else
-				{
-					SetIPv6( true );
-					memcpy( m_address.GetAddr6(), m_pResolverGetSockAddr()->GetAddr6(), sizeof( *(m_address.GetAddr6()) ) );
-				}
-#endif /* HAVE_IPV6 */
-			}
-
-			if ( m_eConState != CST_OK )
-				m_eConState = ( ( eDNSLType == DNS_VHOST ) ? CST_BINDVHOST : CST_VHOSTDNS );
-	
-			if( !CreateSocksFD() )
-				return( ETIMEDOUT );
-
-			return( 0 );
-		}
-
-		return( ETIMEDOUT );
-	}
-	return( EAGAIN );
-	
-#else
 	int iRet = ETIMEDOUT;
 	if ( eDNSLType == DNS_VHOST )
 	{
 		iRet = GetAddrInfo( m_sBindHost, this, m_bindhost );
+#ifdef HAVE_IPV6
+		if( m_bindhost.GetIPv6() )
+		{
+			SetAFRequire( CSSockAddr::RAF_INET6 );
+		}
+		else
+		{
+			SetAFRequire( CSSockAddr::RAF_INET );
+		}
+#endif /* HAVE_IPV6 */
 	}
 	else
 	{
@@ -2125,11 +1967,11 @@ int Csock::DNSLookup( EDNSLType eDNSLType )
 
 	if( !CreateSocksFD() )
 		iRet = ETIMEDOUT;
-	
+
 	if ( iRet == 0 )
 	{
 		if ( m_eConState != CST_OK )
-			m_eConState = ( ( eDNSLType == DNS_VHOST ) ? CST_BINDVHOST : CST_VHOSTDNS );
+			m_eConState = ( ( eDNSLType == DNS_VHOST ) ? CST_BINDVHOST : CST_CONNECT );
 		m_iDNSTryCount = 0;
 		return( 0 );
 	}
@@ -2145,7 +1987,6 @@ int Csock::DNSLookup( EDNSLType eDNSLType )
 	}
 	m_iDNSTryCount = 0;
 	return( ETIMEDOUT );
-#endif /* ___DO_THREADS */
 }
 
 bool Csock::SetupVHost()
@@ -2153,7 +1994,7 @@ bool Csock::SetupVHost()
 	if ( m_sBindHost.empty() )
 	{
 		if ( m_eConState != CST_OK )
-			m_eConState = CST_CONNECT;
+			m_eConState = CST_DESTDNS;
 		return( true );
 	}
 	int iRet = -1;
@@ -2167,7 +2008,7 @@ bool Csock::SetupVHost()
 	if ( iRet == 0 )
 	{
 		if ( m_eConState != CST_OK )
-			m_eConState = CST_CONNECT;
+			m_eConState = CST_DESTDNS;
 		return( true );
 	}
 	m_iCurBindCount++;
@@ -2206,19 +2047,20 @@ int Csock::SOCKET( bool bListen )
 #ifdef HAVE_IPV6
 	int iRet = socket( ( GetIPv6() ? PF_INET6 : PF_INET ), SOCK_STREAM, IPPROTO_TCP );
 #else
-	// missing wrapper around ipv6 for systems missing ipv6, Uli Schlachter <psycho@foex-gaming.com>
 	int iRet = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
 #endif /* HAVE_IPV6 */
 
+	if ( iRet >= 0 ) {
+		set_close_on_exec( iRet );
 
-	if ( ( iRet > -1 ) && ( bListen ) )
-	{
-		const int on = 1;
+		if ( bListen ) {
+			const int on = 1;
 
-		if ( setsockopt( iRet, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof( on ) ) != 0 )
-			PERROR( "setsockopt" );
-
-	} else if ( iRet == -1 )
+			if ( setsockopt( iRet, SOL_SOCKET, SO_REUSEADDR,
+						(char *)&on, sizeof( on ) ) != 0 )
+				PERROR( "setsockopt" );
+		}
+	} else
 		PERROR( "socket" );
 
 	return( iRet );
@@ -2264,6 +2106,7 @@ void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
 	m_iDNSTryCount = 0;
 	m_iCurBindCount = 0;
 	m_bIsIPv6 = false;
+	m_bSkipConnect = false;
 	m_iLastCheckTimeoutTime = 0;
 }
 
