@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.178 $
+* $Revision: 1.191 $
 */
 
 // note to compile with win32 need to link to winsock2, using gcc its -lws2_32
@@ -112,6 +112,7 @@ namespace Csocket
 {
 #endif /* _NO_CSOCKET_NS */
 
+
 /**
  * @class CSCharBuffer
  * @brief ease of use self deleting char * class
@@ -136,9 +137,9 @@ private:
 class CSSockAddr
 {
 public:
-	CSSockAddr() 
-	{ 
-		m_bIsIPv6 = false; 
+	CSSockAddr()
+	{
+		m_bIsIPv6 = false;
 		memset( (struct sockaddr_in *)&m_saddr, '\0', sizeof( m_saddr ) );
 #ifdef HAVE_IPV6
 		memset( (struct sockaddr_in6 *)&m_saddr6, '\0', sizeof( m_saddr6 ) );
@@ -180,8 +181,8 @@ public:
 		m_saddr.sin_port = htons( iPort );
 	}
 
-	void SetIPv6( bool b ) 
-	{ 
+	void SetIPv6( bool b )
+	{
 #ifndef HAVE_IPV6
 		if( b )
 		{
@@ -190,7 +191,7 @@ public:
 			return;
 		}
 #endif /* HAVE_IPV6 */
-		m_bIsIPv6 = b; 
+		m_bIsIPv6 = b;
 		SinFamily();
 	}
 	bool GetIPv6() const { return( m_bIsIPv6 ); }
@@ -218,6 +219,24 @@ private:
 };
 
 class Csock;
+
+/**
+ * @brief this function is a wrapper around gethostbyname and getaddrinfo (for ipv6)
+ *
+ * in the event this code is using ipv6, it calls getaddrinfo, and it tries to start the connection on each iteration
+ * in the linked list returned by getaddrinfo. if pSock is not NULL the following behavior happens.
+ * - if pSock is a listener, or if the connect state is in a bind vhost state (to be used with bind) AI_PASSIVE is sent to getaddrinfo
+ * - if pSock is an outbound connection, AI_ADDRCONFIG and the connection is started from within this function.
+ * getaddrinfo might return multiple (possibly invalid depending on system configuration) ip addresses, so connect needs to try them all.
+ * A classic example of this is a hostname that resolves to both ipv4 and ipv6 ip's. You still need to call Connect (and ConnectSSL) to finish
+ * up the connection state
+ * - NOTE ... Once threading is reimplemented, this function will spin off a thread to resolve and return EAGAIN until its done.
+ *
+ * @param sHostname the host to resolve
+ * @param pSock the sock being setup, this option can be NULL, if it is null csSockAddr is only setup
+ * @param csSockAddr the struct that sockaddr data is being copied to
+ * @return 0 on success, otherwise an error. EAGAIN if this needs to be called again at a later time, ETIMEDOUT if no host is found
+ */
 int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockAddr );
 
 //! used to retrieve the context position of the socket to its associated ssl connection. Setup once in InitSSL() via SSL_get_ex_new_index
@@ -229,84 +248,6 @@ Csock *GetCsockFromCTX( X509_STORE_CTX *pCTX );
 #endif /* HAVE_LIBSSL */
 
 
-#if defined( _REENTRANT ) && defined( _USE_THREADED_DNS )
-#define ___DO_THREADS
-#include <pthread.h>
-
-#ifndef PTHREAD_MUTEX_FAST_NP 
-#define PTHREAD_MUTEX_FAST_NP PTHREAD_MUTEX_NORMAL
-#endif /* PTHREAD_MUTEX_FAST_NP */
-
-class CSMutex
-{
-public:
-	CSMutex ();
-	virtual ~CSMutex();
-	int lock() { return( pthread_mutex_lock( &m_mutex ) ); }
-	int unlock() { return( pthread_mutex_unlock( &m_mutex ) ); }
-	int trylock() { return( pthread_mutex_trylock( &m_mutex ) ); }
-private:
-	pthread_mutex_t			m_mutex;
-	pthread_mutexattr_t		m_mattrib;
-	
-};
-
-class CSThread
-{
-public:
-	CSThread() { m_eStatus = WAITING; }
-	virtual ~CSThread() {}
-	
-	enum EStatus
-	{
-		WAITING = 1,
-		RUNNING = 2,
-		FINISHED = 3
-	};
-
-	bool start();
-	void wait();
-	virtual void run() = 0;
-	static void *start_thread( void *args );
-	EStatus Status() { return( m_eStatus ); }
-	void SetStatus( EStatus e ) { m_eStatus = e; }
-	int lock() { return( m_mutex.lock() ); }
-	int unlock() { return( m_mutex.unlock() ); }
-	int cancel() { return( pthread_cancel( m_ppth ) ); }
-
-private:
-	pthread_t	m_ppth;
-	EStatus m_eStatus;
-	CSMutex  m_mutex;
-};
-
-class CDNSResolver : public CSThread
-{
-public:
-	CDNSResolver() : CSThread() { m_bSuccess = false; }
-	virtual ~CDNSResolver() {}
-	//! returns imediatly, from here out check if IsCompleted() returns true before looking at ANY of the data
-	void Lookup( const CS_STRING & sHostname );
-
-	virtual void run();
-
-	//! true if dns entry was successfuly found
-	bool Suceeded() const { return( m_bSuccess ); }
-
-	//! true if task is finished, this function is thread safe
-	bool IsCompleted();
-
-	CSSockAddr * GetSockAddr() { return( &m_cSockAddr ); }
-
-private:
-	bool		m_bSuccess;
-	CS_STRING	m_sHostname;
-	CSSockAddr	m_cSockAddr;
-};
-
-
-#endif /* ___DO_THREADS */
-
 const u_int CS_BLOCKSIZE = 4096;
 template <class T> inline void CS_Delete( T * & p ) { if( p ) { delete p; p = NULL; } }
 
@@ -478,12 +419,12 @@ public:
 	 * @brief in the event you pass this class to Copy(), you MUST call this function or
 	 * on the original Csock other wise bad side effects will happen (double deletes, weird sock closures, etc)
 	 * if you call this function and have not handled the internal pointers, other bad things can happend (memory leaks, fd leaks, etc)
-	 * the whole point of this function is to allow this class to go away without shutting down 
+	 * the whole point of this function is to allow this class to go away without shutting down
 	 */
 	virtual void Dereference();
 	//! use this to copy a sock from one to the other, override it if you have special needs in the event of a copy
 	virtual void Copy( const Csock & cCopy );
-	
+
 	enum ETConn
 	{
 		OUTBOUND			= 0,		//!< outbound connection
@@ -520,10 +461,11 @@ public:
 	{
 		CST_START		= 0,
 		CST_DNS			= CST_START,
-		CST_VHOSTDNS	= 1,
-		CST_BINDVHOST	= 2,
+		CST_BINDVHOST	= 1,
+		CST_DESTDNS		= 2,
 		CST_CONNECT		= 3,
-		CST_OK			= 4
+		CST_CONNECTSSL	= 4,
+		CST_OK			= 5
 	};
 
 	enum ECloseType
@@ -683,10 +625,15 @@ public:
 	*/
 	virtual void PushBuff( const char *data, int len, bool bStartAtZero = false );
 
-	//! This gives access to the internal buffer, if your
+	//! This gives access to the internal read buffer, if your
 	//! not going to use ReadLine(), then you may want to clear this out
 	//! (if its binary data and not many '\\n')
-	CS_STRING & GetInternalBuffer();
+	CS_STRING & GetInternalReadBuffer();
+
+	//! This gives access to the internal write buffer.
+	//! If you want to check if the send queue fills up, check here.
+	CS_STRING & GetInternalWriteBuffer();
+
 	//! sets the max buffered threshold when EnableReadLine() is enabled
 	void SetMaxBufferThreshold( u_int iThreshold );
 	u_int GetMaxBufferThreshold();
@@ -930,11 +877,11 @@ public:
 
 
 	//! return how long it has been (in seconds) since the last read or successful write
-	int GetTimeSinceLastDataTransaction( time_t iNow = 0 ) 
-	{ 
+	int GetTimeSinceLastDataTransaction( time_t iNow = 0 )
+	{
 		if( m_iLastCheckTimeoutTime == 0 )
 			return( 0 );
-		return( ( iNow > 0 ? iNow : time( NULL ) ) - m_iLastCheckTimeoutTime ); 
+		return( ( iNow > 0 ? iNow : time( NULL ) ) - m_iLastCheckTimeoutTime );
 	}
 	time_t GetLastCheckTimeout() { return( m_iLastCheckTimeoutTime ); }
 
@@ -953,7 +900,7 @@ public:
 	{
 		if( m_iReadSock != -1 )
 			return( true );
-			
+
 		m_iReadSock = m_iWriteSock = SOCKET();
 		if ( m_iReadSock == -1 )
 			return( false );
@@ -966,13 +913,13 @@ public:
 
 	const CS_STRING & GetBindHost() const { return( m_sBindHost ); }
 	void SetBindHost( const CS_STRING & sBindHost ) { m_sBindHost = sBindHost; }
-		
+
 	enum EDNSLType
 	{
 		DNS_VHOST,
 		DNS_DEST
 	};
-	
+
 	/**
 	 * nonblocking dns lookup (when -pthread is set to compile)
 	 * @return 0 for success, EAGAIN to check back again (same arguments as before), ETIMEDOUT on failure
@@ -981,11 +928,11 @@ public:
 
 	//! this is only used on outbound connections, listeners bind in a different spot
 	bool SetupVHost();
-	
+
 	bool GetIPv6() const { return( m_bIsIPv6 ); }
-	void SetIPv6( bool b ) 
-	{ 
-		m_bIsIPv6 = b; 
+	void SetIPv6( bool b )
+	{
+		m_bIsIPv6 = b;
 		m_address.SetIPv6( b );
 		m_bindhost.SetIPv6( b );
 	}
@@ -1002,6 +949,8 @@ public:
 	//! returns a const reference to the crons associated to this socket
 	const std::vector<CCron *> & GetCrons() const { return( m_vcCrons ); }
 
+	void SetSkipConnect( bool b ) { m_bSkipConnect = b; }
+
 private:
 	//! making private for safety
 	Csock( const Csock & cCopy ) {}
@@ -1012,17 +961,18 @@ private:
 	bool		m_bssl, m_bIsConnected, m_bBLOCK, m_bFullsslAccept;
 	bool		m_bsslEstablished, m_bEnableReadLine, m_bRequireClientCert, m_bPauseRead;
 	CS_STRING	m_shostname, m_sbuffer, m_sSockName, m_sPemFile, m_sCipherType, m_sParentName;
-	CS_STRING	m_sSend, m_sSSLBuffer, m_sPemPass, m_sLocalIP, m_sRemoteIP;
+	CS_STRING	m_sSend, m_sPemPass, m_sLocalIP, m_sRemoteIP;
 	ECloseType	m_eCloseType;
 
 	unsigned long long	m_iMaxMilliSeconds, m_iLastSendTime, m_iBytesRead, m_iBytesWritten, m_iStartTime;
 	unsigned int		m_iMaxBytes, m_iLastSend, m_iMaxStoredBufferLength, m_iTimeoutType;
 
 	CSSockAddr 		m_address, m_bindhost;
-	bool			m_bIsIPv6;
+	bool			m_bIsIPv6, m_bSkipConnect;
 	time_t			m_iLastCheckTimeoutTime;
 
 #ifdef HAVE_LIBSSL
+	CS_STRING			m_sSSLBuffer;
 	SSL 				*m_ssl;
 	SSL_CTX				*m_ssl_ctx;
 	SSL_METHOD			*m_ssl_method;
@@ -1046,10 +996,6 @@ private:
 	CS_STRING		m_sBindHost;
 	u_int			m_iCurBindCount, m_iDNSTryCount;
 
-#ifdef ___DO_THREADS
-	CDNSResolver	m_pResolver;
-#endif /* ___DO_THREADS */
-
 };
 
 /**
@@ -1064,7 +1010,7 @@ public:
 	 * @param iPort port to connect to
 	 * @param iTimeout connection timeout
 	 */
-	CSConnection( const CS_STRING & sHostname, u_short iPort, int iTimeout = 60 ) 
+	CSConnection( const CS_STRING & sHostname, u_short iPort, int iTimeout = 60 )
 	{
 		m_sHostname = sHostname;
 		m_iPort = iPort;
@@ -1148,7 +1094,7 @@ public:
 	 * @param iPort port to listen on. Set to 0 to listen on a random port
 	 * @param sBindHost host to bind to
 	 */
-	CSListener( u_short iPort, const CS_STRING & sBindHost = "" ) 
+	CSListener( u_short iPort, const CS_STRING & sBindHost = "" )
 	{
 		m_iPort = iPort;
 		m_sBindHost = sBindHost;
@@ -1408,20 +1354,10 @@ public:
 				continue;
 
 			if ( pcSock->GetConState() == T::CST_DNS )
-			{
-				if ( pcSock->DNSLookup( T::DNS_DEST ) == ETIMEDOUT )
-				{
-					pcSock->SockError( EDOM );
-					DelSock( a-- );
-					continue;
-				}
-			}
-
-			if ( pcSock->GetConState() == T::CST_VHOSTDNS )
 			{
 				if ( pcSock->DNSLookup( T::DNS_VHOST ) == ETIMEDOUT )
 				{
-					pcSock->SockError( EADDRNOTAVAIL );
+					pcSock->SockError( EDOM );
 					DelSock( a-- );
 					continue;
 				}
@@ -1437,6 +1373,15 @@ public:
 				}
 			}
 
+			if ( pcSock->GetConState() == T::CST_DESTDNS )
+			{
+				if ( pcSock->DNSLookup( T::DNS_DEST ) == ETIMEDOUT )
+				{
+					pcSock->SockError( EADDRNOTAVAIL );
+					DelSock( a-- );
+					continue;
+				}
+			}
 			if ( pcSock->GetConState() == T::CST_CONNECT )
 			{
 				if ( !pcSock->Connect( pcSock->GetBindHost(), true ) )
@@ -1444,13 +1389,15 @@ public:
 					if ( GetSockError() == ECONNREFUSED )
 						pcSock->ConnectionRefused();
 					else
-						pcSock->SockError( ECONNABORTED );
+						pcSock->SockError( GetSockError() );
 
 					DelSock( a-- );
 					continue;
 				}
-
+			}
 #ifdef HAVE_LIBSSL
+			if( pcSock->GetConState() == T::CST_CONNECTSSL )
+			{
 				if ( pcSock->GetSSL() )
 				{
 					if ( !pcSock->ConnectSSL() )
@@ -1458,14 +1405,14 @@ public:
 						if ( GetSockError() == ECONNREFUSED )
 							pcSock->ConnectionRefused();
 						else
-							pcSock->SockError( ECONNABORTED );
+							pcSock->SockError( GetSockError() == 0 ? ECONNABORTED : GetSockError() );
 
 						DelSock( a-- );
 						continue;
 					}
 				}
-#endif /* HAVE_LIBSSL */
 			}
+#endif /* HAVE_LIBSSL */
 		}
 
 		std::map<T *, EMessages> mpeSocks;
@@ -1491,7 +1438,7 @@ public:
 
 						if ( iLen <= 0 )
 							iLen = CS_BLOCKSIZE;
-						
+
 						CSCharBuffer cBuff( iLen );
 
 						int bytes = pcSock->Read( cBuff(), iLen );
@@ -1586,14 +1533,14 @@ public:
 	 *	- by itself. I've tried to mitigate that as much as possible by not having it change the select if the previous call to select
 	 *	- was not a timeout. Anyways .... Caveat Emptor.
 	 *	- Sample useage is cFoo.DynamicSelectLoop( 500000, 5000000 ); which basically says min of 500ms and max of 5s
-	 * 
+	 *
 	 * @param iLowerBounds the lower bounds to use in MICROSECONDS
 	 * @param iUpperBounds the upper bounds to use in MICROSECONDS
 	 * @param iMaxResolution the maximum time to calculate overall in seconds
 	 */
 	void DynamicSelectLoop( u_long iLowerBounds, u_long iUpperBounds, time_t iMaxResolution = 3600 )
 	{
-		SetSelectTimeout( iLowerBounds );	
+		SetSelectTimeout( iLowerBounds );
 		if( m_errno == SELECT_TIMEOUT )
 		{ // only do this if the previous call to select was a timeout
 			time_t iNow = time( NULL );
@@ -1807,7 +1754,7 @@ public:
 		(*this)[iOrginalSockIdx] = (T *)pNewSock;
 		return( true );
 	}
-	
+
 	/**
 	 * @brief swaps out a sock with a copy of the original sock
 	 * @param pNewSock the new sock to change out with. (this should be constructed by you with the default ctor)
@@ -2138,7 +2085,7 @@ private:
 					iTimeoutInSeconds = 0;
 				else
 					iTimeoutInSeconds -= iDiff;
-			
+
 				iMinTimeout = std::min( iMinTimeout, iTimeoutInSeconds );
 			}
 
@@ -2182,9 +2129,6 @@ private:
 
 	////////
 	// Connection State Functions
-	
-
-
 
 	///////////
 	// members
