@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,10 +9,10 @@
 #ifndef _DCCBOUNCE_H
 #define _DCCBOUNCE_H
 
-#include "main.h"
+#include "Csocket.h"
 #include "Utils.h"
-#include "User.h"
-#include "znc.h"
+
+class CUser;
 
 class CDCCBounce : public Csock {
 public:
@@ -58,6 +58,7 @@ public:
 
 	void ReadLine(const CString& sData);
 	virtual void ReadData(const char* data, int len);
+	virtual void ReadPaused();
 	virtual void Timeout();
 	virtual void ConnectionRefused();
 	virtual void SockError(int iErrno);
@@ -93,9 +94,12 @@ protected:
 	CString					m_sFileName;
 	CUser*					m_pUser;
 	CDCCBounce*				m_pPeer;
-	unsigned short			m_uRemotePort;
+	unsigned short				m_uRemotePort;
 	bool					m_bIsChat;
 	bool					m_bIsRemote;
+
+	static const unsigned int		m_uiMaxDCCBuffer;
+	static const unsigned int		m_uiMinDCCBuffer;
 };
 
 #endif // !_DCCBOUNCE_H
