@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -7,13 +7,11 @@
  */
 
 #include "FileUtils.h"
-#include <iostream>
+#include "Utils.h"
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <sys/wait.h>
 
-using std::cout;
-using std::cerr;
-using std::endl;
-
 CFile::CFile() {
 	m_iFD = -1;
 	m_bClose = false;
@@ -121,7 +119,7 @@ bool CFile::Exists(const CString& sFile) {
 
 unsigned long long CFile::GetSize(const CString& sFile) {
 	struct stat st;
-	if(stat(sFile.c_str(), &st) != 0) {
+	if (stat(sFile.c_str(), &st) != 0) {
 		return 0;
 	}
 
@@ -159,7 +157,7 @@ int CFile::GetInfo(const CString& sFile, struct stat& st) {
 //
 // Functions to manipulate the file on the filesystem
 //
-int CFile::Delete() { return CFile::Delete(m_sLongName); }
+bool CFile::Delete() { return CFile::Delete(m_sLongName); }
 int CFile::Move(const CString& sNewFileName, bool bOverwrite) {
 	return CFile::Move(m_sLongName, sNewFileName, bOverwrite);
 }
@@ -169,7 +167,7 @@ int CFile::Copy(const CString& sNewFileName, bool bOverwrite) {
 }
 
 bool CFile::Delete(const CString& sFileName) {
-	if(!CFile::Exists(sFileName)) {
+	if (!CFile::Exists(sFileName)) {
 		return false;
 	}
 
@@ -177,7 +175,7 @@ bool CFile::Delete(const CString& sFileName) {
 }
 
 bool CFile::Move(const CString& sOldFileName, const CString& sNewFileName, bool bOverwrite) {
-	if((!bOverwrite) && (CFile::Exists(sNewFileName))) {
+	if ((!bOverwrite) && (CFile::Exists(sNewFileName))) {
 		return false;
 	}
 
@@ -186,7 +184,7 @@ bool CFile::Move(const CString& sOldFileName, const CString& sNewFileName, bool
 }
 
 bool CFile::Copy(const CString& sOldFileName, const CString& sNewFileName, bool bOverwrite) {
-	if((!bOverwrite) && (CFile::Exists(sNewFileName))) {
+	if ((!bOverwrite) && (CFile::Exists(sNewFileName))) {
 		return false;
 	}
 
@@ -254,6 +252,9 @@ bool CFile::Open(int iFlags, mode_t iMode) {
 	if (m_iFD < 0)
 		return false;
 
+	/* Make sure this FD isn't given to childs */
+	SetFdCloseOnExec(m_iFD);
+
 	m_bClose = true;
 	return true;
 }
@@ -276,7 +277,7 @@ bool CFile::ReadLine(CString& sData, const CString & sDelimiter) {
 
 	bool bEOF = false;
 
-	while(true) {
+	while (true) {
 		CString::size_type iFind = m_sBuffer.find(sDelimiter);
 		if (iFind != CString::npos) {
 			sData = m_sBuffer.substr(0, (iFind + 1));
@@ -312,7 +313,7 @@ bool CFile::ReadLine(CString& sData, const CString & sDelimiter) {
 		return true;
 	}
 
-	if( bEOF && m_sBuffer.size() ) {
+	if (bEOF && m_sBuffer.size()) {
 		sData = m_sBuffer;
 		m_sBuffer.clear();
 		return true;
@@ -356,18 +357,106 @@ CString CFile::GetDir() const {
 
 void CFile::SetFD(int iFD) { m_iFD = iFD; }
 
+CString CDir::ChangeDir(const CString& sPath, const CString& sAdd, const CString& sHomeDir) {
+	if (sAdd == "~") {
+		return sHomeDir;
+	}
+
+	CString sAddDir = sAdd;
+
+	if (sAddDir.Left(2) == "~/") {
+		sAddDir.LeftChomp();
+		sAddDir = sHomeDir + sAddDir;
+	}
+
+	CString sRet = ((sAddDir.size()) && (sAddDir[0] == '/')) ? "" : sPath;
+	sAddDir += "/";
+	CString sCurDir;
+
+	if (sRet.Right(1) == "/") {
+		sRet.RightChomp();
+	}
+
+	for (unsigned int a = 0; a < sAddDir.size(); a++) {
+		switch (sAddDir[a]) {
+			case '/':
+				if (sCurDir == "..") {
+					sRet = sRet.substr(0, sRet.rfind('/'));
+				} else if ((sCurDir != "") && (sCurDir != ".")) {
+					sRet += "/" + sCurDir;
+				}
+
+				sCurDir = "";
+				break;
+			default:
+				sCurDir += sAddDir[a];
+				break;
+		}
+	}
+
+	return (sRet.empty()) ? "/" : sRet;
+}
+
+bool CDir::MakeDir(const CString& sPath, mode_t iMode) {
+	CString sDir;
+	VCString dirs;
+	VCString::iterator it;
+
+	// Just in case someone tries this...
+	if (sPath.empty())
+		return false;
+
+	// If this is an absolute path, we need to handle this now!
+	if (sPath.Left(1) == "/")
+		sDir = "/";
+
+	// For every single subpath, do...
+	sPath.Split("/", dirs, false);
+	for (it = dirs.begin(); it != dirs.end(); it++) {
+		// Add this to the path we already created
+		sDir += *it;
+
+		int i = mkdir(sDir.c_str(), iMode);
+
+		if (i != 0) {
+			// All errors except EEXIST are fatal
+			if (errno != EEXIST)
+				return false;
+
+			// If it's EEXIST we have to make sure it's a dir
+			if (!CFile::IsDir(sDir))
+				return false;
+		}
+
+		sDir += "/";
+	}
+
+	// All went well
+	return true;
+}
+
 int CExecSock::popen2(int & iReadFD, int & iWriteFD, const CString & sCommand) {
 	int rpipes[2] = { -1, -1 };
 	int wpipes[2] = { -1, -1 };
 	iReadFD = -1;
 	iWriteFD = -1;
 
-	pipe(rpipes);
-	pipe(wpipes);
-	
+	if (pipe(rpipes) < 0)
+		return -1;
+
+	if (pipe(wpipes) < 0) {
+		close(rpipes[0]);
+		close(rpipes[1]);
+		return -1;
+	}
+
 	int iPid = fork();
 
 	if (iPid == -1) {
+		close(rpipes[0]);
+		close(rpipes[1]);
+		close(wpipes[0]);
+		close(wpipes[1]);
 		return -1;
 	}
 
@@ -379,14 +468,14 @@ int CExecSock::popen2(int & iReadFD, int & iWriteFD, const CString & sCommand) {
 		dup2(rpipes[1], 2);
 		close(wpipes[0]);
 		close(rpipes[1]);
-		char * const pArgv[] =
+		const char * pArgv[] =
 		{
 			"sh",
 			"-c",
-			(char *)sCommand.c_str(),
+			sCommand.c_str(),
 			NULL
 		};
-		execvp( "sh", pArgv );
+		execvp("sh", (char * const *) pArgv);
 		exit(0);
 	}
 
@@ -400,14 +489,13 @@ int CExecSock::popen2(int & iReadFD, int & iWriteFD, const CString & sCommand) {
 }
 
 void CExecSock::close2(int iPid, int iReadFD, int iWriteFD) {
-	close( iReadFD );
-	close( iWriteFD );
-	u_int iNow = time( NULL );
-	while( waitpid( iPid, NULL, WNOHANG ) == 0 )
-	{
-		if ( ( time( NULL ) - iNow ) > 5 )
+	close(iReadFD);
+	close(iWriteFD);
+	u_int iNow = time(NULL);
+	while (waitpid(iPid, NULL, WNOHANG) == 0) {
+		if ((time(NULL) - iNow) > 5)
 			break;	// giveup
-		usleep( 100 );
+		usleep(100);
 	}
 	return;
 }
