@@ -1,11 +1,13 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
+#ifdef _MODULES
+
 #include "HTTPSock.h"
 #include "znc.h"
 
@@ -82,7 +84,7 @@ void CHTTPSock::ReadLine(const CString& sData) {
 		DisableReadLine();
 
 		if (m_bPost) {
-			m_sPostData = GetInternalBuffer();
+			m_sPostData = GetInternalReadBuffer();
 			CheckPost();
 		} else {
 			GetPage();
@@ -114,7 +116,7 @@ bool CHTTPSock::PrintFile(const CString& sFileName, CString sContentType) {
 			sFilePath.LeftChomp(1);
 		}
 
-		sFilePath = CUtils::ChangeDir(m_sDocRoot, sFilePath, m_sDocRoot);
+		sFilePath = CDir::ChangeDir(m_sDocRoot, sFilePath, m_sDocRoot);
 
 		if (sFilePath.Left(m_sDocRoot.size()) != m_sDocRoot) {
 			PrintErrorPage(403, "Forbidden", "You don't have permission to access that file on this server.");
@@ -158,7 +160,7 @@ bool CHTTPSock::PrintFile(const CString& sFileName, CString sContentType) {
 	char szBuf[4096];
 	int iLen = 0;
 
-	while((iLen = File.Read(szBuf, 4096)) > 0) {
+	while ((iLen = File.Read(szBuf, 4096)) > 0) {
 		Write(szBuf, iLen);
 	}
 
@@ -389,3 +391,5 @@ Csock* CHTTPSock::GetSockObj(const CString& sHost, unsigned short uPort) {
 
 	return pSock;
 }
+
+#endif // _MODULES
