@@ -1,16 +1,17 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "znc.h"
 #include "IRCSock.h"
-#include "DCCBounce.h"
+#include "Chan.h"
 #include "Client.h"
-#include <time.h>
+#include "DCCBounce.h"
+#include "User.h"
+#include "znc.h"
 
 CIRCSock::CIRCSock(CUser* pUser) : Csock() {
 	m_pUser = pUser;
@@ -75,9 +76,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 	DEBUG_ONLY(cout << "(" << m_pUser->GetUserName() << ") IRC -> ZNC [" << sLine << "]" << endl);
 
-#ifdef _MODULES
 	MODULECALL(OnRaw(sLine), m_pUser, NULL, return);
-#endif
 
 	if (strncasecmp(sLine.c_str(), "PING ", 5) == 0) {
 		PutIRC("PONG " + sLine.substr(5));
@@ -272,7 +271,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 					CChan* pChan = m_pUser->FindChan(sLine.Token(3));
 
 					if (pChan) {
-						CString sNick = sLine.Token(4);
+						sNick = sLine.Token(4);
 						unsigned long ulDate = strtoul(sLine.Token(5).c_str(), NULL, 10);
 
 						pChan->SetTopicOwner(sNick);
@@ -283,8 +282,8 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 				case 352: {
 					// :irc.yourserver.com 352 yournick #chan ident theirhost.com irc.theirserver.com theirnick H :0 Real Name
-					CString sServer = sLine.Token(0);
-					CString sNick = sLine.Token(7);
+					sServer = sLine.Token(0);
+					sNick = sLine.Token(7);
 					CString sIdent = sLine.Token(4);
 					CString sHost = sLine.Token(5);
 
@@ -329,8 +328,12 @@ void CIRCSock::ReadLine(const CString& sData) {
 					sRest.Trim();
 					// Todo: allow for non @+= server msgs
 					CChan* pChan = m_pUser->FindChan(sRest.Token(1));
-					if (!pChan) // Todo: should this still be forwarded to clients?
+					if (!pChan) {
+						// If we don't know that channel, a client might have
+						// sent a /names on it's own -> forward it.
+						m_pUser->PutUser(sLine);
 						return;
+					}
 
 					CString sNicks = sRest.Token(2, true);
 					if (sNicks.Left(1) == ":") {
@@ -342,7 +345,6 @@ void CIRCSock::ReadLine(const CString& sData) {
 					// Get everything except the actual user list
 					CString sTmp = sLine.Token(0, false, " :") + " :";
 					vector<CClient*>& vClients = m_pUser->GetClients();
-					CString sNick;
 
 					for (unsigned int a = 0; a < vClients.size(); a++) {
 						if ((!m_bNamesx || vClients[a]->HasNamesx())
@@ -350,10 +352,13 @@ void CIRCSock::ReadLine(const CString& sData) {
 							m_pUser->PutUser(sLine, vClients[a]);
 						} else {
 							unsigned int i = 0;
-							// This loop runs once more than there are nicks,
-							// the last run sets sNick to "" and causes the break.
-							do {
-								sNick = sNicks.Token(i);
+							// This loop runs once for
+							// every nick on the channel
+							for (;;) {
+								sNick = sNicks.Token(i).Trim_n(" ");
+								if (sNick.empty())
+									break;
+
 								if (m_bNamesx && !vClients[a]->HasNamesx()
 									&& IsPermChar(sNick[0])) {
 									// Server has, client hasnt NAMESX,
@@ -372,7 +377,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 								sTmp += sNick + " ";
 								i++;
-							} while (!sNick.empty());
+							}
 							// Strip away the spaces we inserted at the end
 							sTmp.Trim(" ");
 							m_pUser->PutUser(sTmp, vClients[a]);
@@ -412,7 +417,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 			}
 		} else { //if (CUtils::wildcmp(":*!*@* * *", sLine.c_str())) {
 			CNick Nick(sLine.Token(0).LeftChomp_n());
-			CString sCmd = sLine.Token(1);
+			sCmd = sLine.Token(1);
 			CString sRest = sLine.Token(2, true);
 
 			if (sCmd.CaseCmp("NICK") == 0) {
@@ -440,6 +445,8 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				// Todo: use nick compare function here
 				if (Nick.GetNick().CaseCmp(GetNick()) == 0) {
+					// We are changing our own nick, the clients always must see this!
+					bIsVisible = true;
 					SetNick(sNewNick);
 				} else if (Nick.GetNick().CaseCmp(m_pUser->GetNick()) == 0) {
 					KeepNick(true);
@@ -549,7 +556,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 			} else if (sCmd.CaseCmp("MODE") == 0) {
 				CString sTarget = sRest.Token(0);
 				CString sModes = sRest.Token(1, true);
-				if(sModes.Left(1) == ":")
+				if (sModes.Left(1) == ":")
 					sModes = sModes.substr(1);
 
 				CChan* pChan = m_pUser->FindChan(sTarget);
@@ -564,9 +571,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 //					CString sArgs = sModes.Token(1, true); Usermode changes got no params
 					bool bAdd = true;
 /* no module call defined (yet?)
-#ifdef _MODULES
 					MODULECALL(OnRawUserMode(*pOpNick, *this, sModeArg, sArgs), m_pUser, NULL, );
-#endif
 */
 					for (unsigned int a = 0; a < sModeArg.size(); a++) {
 						const unsigned char& uMode = sModeArg[a];
@@ -576,7 +581,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 						} else if (uMode == '-') {
 							bAdd = false;
 						} else {
-							if(bAdd) {
+							if (bAdd) {
 								m_scUserModes.insert(uMode);
 							} else {
 								m_scUserModes.erase(uMode);
@@ -652,9 +657,18 @@ void CIRCSock::ReadLine(const CString& sData) {
 				if (pChan) {
 					CString sTopic = sLine.Token(3, true);
 					sTopic.LeftChomp();
+
+					MODULECALL(OnTopic(Nick, *pChan, sTopic), m_pUser, NULL, return)
+
 					pChan->SetTopicOwner(Nick.GetNick());
 					pChan->SetTopicDate((unsigned long) time(NULL));
 					pChan->SetTopic(sTopic);
+
+					if (pChan->IsDetached()) {
+						return; // Don't forward this
+					}
+
+					sLine = ":" + Nick.GetNickMask() + " TOPIC " + pChan->GetName() + " :" + sTopic;
 				}
 			} else if (sCmd.CaseCmp("PRIVMSG") == 0) {
 				// :nick!ident@host.com PRIVMSG #chan :Message
@@ -699,7 +713,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				// :blub!dummy@rox-8DBEFE92 WALLOPS :this is a test
 				CString sMsg = sRest.Token(0, true);
 
-				if(sMsg.Left(1) == ":") {
+				if (sMsg.Left(1) == ":") {
 					sMsg.LeftChomp();
 				}
 
@@ -778,35 +792,41 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 
 		return true;
 	} else {
-		const MCString& mssCTCPReplies = m_pUser->GetCTCPReplies();
-		MCString::const_iterator it = mssCTCPReplies.find(sMessage.AsUpper());
-		CString sQuery = sMessage.Token(0).AsUpper();
-		bool bHaveReply = false;
-		CString sReply;
+		return OnGeneralCTCP(Nick, sMessage);
+	}
 
-		if (it == mssCTCPReplies.end()) {
-			it = mssCTCPReplies.find(sQuery);
-		}
+	return false;
+}
 
-		if (it != mssCTCPReplies.end()) {
-			sReply = it->second;
-			bHaveReply = true;
-		}
+bool CIRCSock::OnGeneralCTCP(CNick& Nick, CString& sMessage) {
+	const MCString& mssCTCPReplies = m_pUser->GetCTCPReplies();
+	MCString::const_iterator it = mssCTCPReplies.find(sMessage.AsUpper());
+	CString sQuery = sMessage.Token(0).AsUpper();
+	bool bHaveReply = false;
+	CString sReply;
 
-		if (!bHaveReply && !m_pUser->IsUserAttached()) {
-			if (sQuery == "VERSION") {
-				sReply = CZNC::GetTag();
-			} else if (sQuery == "PING") {
-				sReply = sMessage.Token(1, true);
-			}
-		}
+	if (it == mssCTCPReplies.end()) {
+		it = mssCTCPReplies.find(sQuery);
+	}
 
-		if (!sReply.empty()) {
-			PutIRC("NOTICE " + Nick.GetNick() + " :\001" + sQuery + " " + sReply + "\001");
-			return true;
+	if (it != mssCTCPReplies.end()) {
+		sReply = m_pUser->ExpandString(it->second);
+		bHaveReply = true;
+	}
+
+	if (!bHaveReply && !m_pUser->IsUserAttached()) {
+		if (sQuery == "VERSION") {
+			sReply = CZNC::GetTag();
+		} else if (sQuery == "PING") {
+			sReply = sMessage.Token(1, true);
 		}
 	}
 
+	if (!sReply.empty()) {
+		PutIRC("NOTICE " + Nick.GetNick() + " :\001" + sQuery + " " + sReply + "\001");
+		return true;
+	}
+
 	return false;
 }
 
@@ -838,7 +858,7 @@ bool CIRCSock::OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage)
 		// Record a /me
 		if (sMessage.Left(7).CaseCmp("ACTION ") == 0) {
 			sMessage = sMessage.substr(7);
-			if(pChan->KeepBuffer() || !m_pUser->IsUserAttached()) {
+			if (pChan->KeepBuffer() || !m_pUser->IsUserAttached()) {
 				pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :\001ACTION " + m_pUser->AddTimestamp(sMessage) + "\001");
 			}
 			MODULECALL(OnChanAction(Nick, *pChan, sMessage), m_pUser, NULL, return true);
@@ -847,6 +867,9 @@ bool CIRCSock::OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage)
 		MODULECALL(OnChanCTCP(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 	}
 
+	if (OnGeneralCTCP(Nick, sMessage))
+		return true;
+
 	return (pChan && pChan->IsDetached());
 }
 
@@ -902,7 +925,8 @@ void CIRCSock::Disconnected() {
 	MODULECALL(OnIRCDisconnected(), m_pUser, NULL, );
 
 	DEBUG_ONLY(cout << GetSockName() << " == Disconnected()" << endl);
-	if (!m_pUser->IsBeingDeleted() && m_pUser->GetIRCConnectEnabled()) {
+	if (!m_pUser->IsBeingDeleted() && m_pUser->GetIRCConnectEnabled() &&
+			m_pUser->GetServers().size() != 0) {
 		m_pUser->PutStatus("Disconnected from IRC. Reconnecting...");
 	}
 	m_pUser->ClearRawBuffer();
@@ -913,9 +937,27 @@ void CIRCSock::Disconnected() {
 }
 
 void CIRCSock::SockError(int iErrno) {
-	DEBUG_ONLY(cout << GetSockName() << " == SockError(" << iErrno << ")" << endl);
+	CString sError;
+
+	if (iErrno == EDOM) {
+		sError = "Your VHost could not be resolved";
+	} else if (iErrno == EADDRNOTAVAIL) {
+		// Csocket uses this if it can't resolve the dest host name
+		sError = strerror(iErrno);
+		sError += " (Is your IRC server's host name valid?)";
+	} else {
+		sError = strerror(iErrno);
+	}
+
+	DEBUG_ONLY(cout << GetSockName() << " == SockError(" << iErrno << " "
+			<< sError << ")" << endl);
 	if (!m_pUser->IsBeingDeleted()) {
-		m_pUser->PutStatus("Disconnected from IRC (" + CString(strerror(iErrno)) + ").  Reconnecting...");
+		if (GetConState() != CST_OK)
+			m_pUser->PutStatus("Cannot connect to IRC (" +
+				sError + "). Retrying...");
+		else
+			m_pUser->PutStatus("Disconnected from IRC (" +
+				sError + ").  Reconnecting...");
 	}
 	m_pUser->ClearRawBuffer();
 	m_pUser->ClearMotdBuffer();
