@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,13 +9,12 @@
 #ifndef _IRCSOCK_H
 #define _IRCSOCK_H
 
-#include "main.h"
-#include "User.h"
-#include "Chan.h"
-#include "Buffer.h"
+#include "Csocket.h"
+#include "Nick.h"
 
 // Forward Declarations
-class CZNC;
+class CChan;
+class CUser;
 // !Forward Declarations
 
 class CIRCSock : public Csock {
@@ -34,6 +33,7 @@ public:
 	bool OnCTCPReply(CNick& Nick, CString& sMessage);
 	bool OnPrivCTCP(CNick& Nick, CString& sMessage);
 	bool OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage);
+	bool OnGeneralCTCP(CNick& Nick, CString& sMessage);
 	bool OnPrivMsg(CNick& Nick, CString& sMessage);
 	bool OnChanMsg(CNick& Nick, const CString& sChan, CString& sMessage);
 	bool OnPrivNotice(CNick& Nick, CString& sMessage);
