@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -8,13 +8,15 @@
 
 #ifdef _MODULES
 
-#include "main.h"
 #include "Modules.h"
-#include "znc.h"
-#include "Utils.h"
 #include "User.h"
-#include "Nick.h"
-#include "Chan.h"
+#include "znc.h"
+#include <dlfcn.h>
+
+#ifndef RTLD_LOCAL
+# define RTLD_LOCAL 0
+# warning "your crap box doesnt define RTLD_LOCAL !?"
+#endif
 
 #define MODUNLOADCHK(func)								\
 	for (unsigned int a = 0; a < size(); a++) {			\
@@ -201,13 +203,14 @@ CModule* CSocket::GetModule() const { return m_pModule; }
 const CString& CSocket::GetLabel() const { return m_sLabel; }
 /////////////////// !Socket ///////////////////
 
-CModule::CModule(void* pDLL, CUser* pUser, const CString& sModName) {
+CModule::CModule(void* pDLL, CUser* pUser, const CString& sModName, const CString& sDataDir) {
 	m_bFake = false;
 	m_pDLL = pDLL;
 	m_pManager = &(CZNC::Get().GetManager());;
 	m_pUser = pUser;
 	m_pClient = NULL;
 	m_sModName = sModName;
+	m_sDataDir = sDataDir;
 
 	if (m_pUser) {
 		m_sSavePath = m_pUser->GetUserPath() + "/moddata/" + m_sModName;
@@ -215,13 +218,14 @@ CModule::CModule(void* pDLL, CUser* pUser, const CString& sModName) {
 	}
 }
 
-CModule::CModule(void* pDLL, const CString& sModName) {
+CModule::CModule(void* pDLL, const CString& sModName, const CString& sDataDir) {
 	m_bFake = false;
 	m_pDLL = pDLL;
 	m_pManager = &(CZNC::Get().GetManager());
 	m_pUser = NULL;
 	m_pClient = NULL;
 	m_sModName = sModName;
+	m_sDataDir = sDataDir;
 
 	m_sSavePath = CZNC::Get().GetZNCPath() + "/moddata/" + m_sModName;
 	LoadRegistry();
@@ -277,6 +281,8 @@ bool CModule::DelNV(const CString & sName, bool bWriteToDisk) {
 
 	if (it != m_mssRegistry.end()) {
 		m_mssRegistry.erase(it);
+	} else {
+		return false;
 	}
 
 	if (bWriteToDisk) {
@@ -472,11 +478,12 @@ void CModule::ListSockets() {
 	}
 }
 
-const CString& CModule::GetModName() const { return m_sModName; }
 CString CModule::GetModNick() const { return ((m_pUser) ? m_pUser->GetStatusPrefix() : "*") + m_sModName; }
 
 bool CModule::OnLoad(const CString& sArgs, CString& sMessage) { sMessage = ""; return true; }
 bool CModule::OnBoot() { return true; }
+void CModule::OnPreRehash() {}
+void CModule::OnPostRehash() {}
 void CModule::OnIRCDisconnected() {}
 void CModule::OnIRCConnected() {}
 CModule::EModRet CModule::OnBroadcast(CString& sMessage) { return CONTINUE; }
@@ -513,6 +520,7 @@ CModule::EModRet CModule::OnUserMsg(CString& sTarget, CString& sMessage) { retur
 CModule::EModRet CModule::OnUserNotice(CString& sTarget, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnUserJoin(CString& sChannel, CString& sKey) { return CONTINUE; }
 CModule::EModRet CModule::OnUserPart(CString& sChannel, CString& sMessage) { return CONTINUE; }
+CModule::EModRet CModule::OnUserTopic(CString& sChannel, CString& sTopic) { return CONTINUE; }
 
 CModule::EModRet CModule::OnCTCPReply(CNick& Nick, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnPrivCTCP(CNick& Nick, CString& sMessage) { return CONTINUE; }
@@ -523,6 +531,7 @@ CModule::EModRet CModule::OnPrivMsg(CNick& Nick, CString& sMessage) { return CON
 CModule::EModRet CModule::OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnPrivNotice(CNick& Nick, CString& sMessage) { return CONTINUE; }
 CModule::EModRet CModule::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage) { return CONTINUE; }
+CModule::EModRet CModule::OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) { return CONTINUE; }
 
 void* CModule::GetDLL() { return m_pDLL; }
 bool CModule::PutIRC(const CString& sLine) {
@@ -535,18 +544,26 @@ bool CModule::PutStatus(const CString& sLine) {
 	return (m_pUser) ? m_pUser->PutStatus(sLine, m_pClient) : false;
 }
 bool CModule::PutModule(const CString& sLine, const CString& sIdent, const CString& sHost) {
-	return (m_pUser) ? m_pUser->PutUser(":" + GetModNick() + "!" + sIdent + "@" + sHost + " PRIVMSG " + m_pUser->GetCurNick() + " :" + sLine, m_pClient) : false;
+	if (!m_pUser)
+		return false;
+	return m_pUser->PutUser(":" + GetModNick() + "!" +
+		(sIdent.empty() ? GetModName() : sIdent) + "@" + sHost +
+		" PRIVMSG " + m_pUser->GetCurNick() + " :" + sLine,
+		m_pClient);
 }
 bool CModule::PutModNotice(const CString& sLine, const CString& sIdent, const CString& sHost) {
-	return (m_pUser) ? m_pUser->PutUser(":" + GetModNick() + "!" + sIdent + "@" + sHost + " NOTICE " + m_pUser->GetCurNick() + " :" + sLine, m_pClient) : false;
+	if (!m_pUser)
+		return false;
+	return m_pUser->PutUser(":" + GetModNick() + "!" +
+		(sIdent.empty() ? GetModName() : sIdent) + "@" + sHost +
+		" NOTICE " + m_pUser->GetCurNick() + " :" + sLine,
+		m_pClient);
 }
 
-
 ///////////////////
 // CGlobalModule //
 ///////////////////
 CModule::EModRet CGlobalModule::OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan) { return CONTINUE; }
-void CGlobalModule::OnFinishedConfig() {}
 CModule::EModRet CGlobalModule::OnDeleteUser(CUser& User) { return CONTINUE; }
 CModule::EModRet CGlobalModule::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) { return CONTINUE; }
 void CGlobalModule::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {}
@@ -584,6 +601,8 @@ bool CModules::OnBoot() {
 	return true;
 }
 
+bool CModules::OnPreRehash() { MODUNLOADCHK(OnPreRehash()); return false; }
+bool CModules::OnPostRehash() { MODUNLOADCHK(OnPostRehash()); return false; }
 bool CModules::OnIRCConnected() { MODUNLOADCHK(OnIRCConnected()); return false; }
 bool CModules::OnBroadcast(CString& sMessage) { MODHALTCHK(OnBroadcast(sMessage)); }
 bool CModules::OnIRCDisconnected() { MODUNLOADCHK(OnIRCDisconnected()); return false; }
@@ -606,6 +625,7 @@ bool CModules::OnUserMsg(CString& sTarget, CString& sMessage) { MODHALTCHK(OnUse
 bool CModules::OnUserNotice(CString& sTarget, CString& sMessage) { MODHALTCHK(OnUserNotice(sTarget, sMessage)); }
 bool CModules::OnUserJoin(CString& sChannel, CString& sKey) { MODHALTCHK(OnUserJoin(sChannel, sKey)); }
 bool CModules::OnUserPart(CString& sChannel, CString& sMessage) { MODHALTCHK(OnUserPart(sChannel, sMessage)); }
+bool CModules::OnUserTopic(CString& sChannel, CString& sTopic) { MODHALTCHK(OnUserTopic(sChannel, sTopic)); }
 
 bool CModules::OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans) { MODUNLOADCHK(OnQuit(Nick, sMessage, vChans)); return false; }
 bool CModules::OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans) { MODUNLOADCHK(OnNick(Nick, sNewNick, vChans)); return false; }
@@ -621,6 +641,7 @@ bool CModules::OnPrivMsg(CNick& Nick, CString& sMessage) { MODHALTCHK(OnPrivMsg(
 bool CModules::OnChanMsg(CNick& Nick, CChan& Channel, CString& sMessage) { MODHALTCHK(OnChanMsg(Nick, Channel, sMessage)); }
 bool CModules::OnPrivNotice(CNick& Nick, CString& sMessage) { MODHALTCHK(OnPrivNotice(Nick, sMessage)); }
 bool CModules::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMessage) { MODHALTCHK(OnChanNotice(Nick, Channel, sMessage)); }
+bool CModules::OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) { MODHALTCHK(OnTopic(Nick, Channel, sTopic)); }
 bool CModules::OnStatusCommand(const CString& sCommand) { MODHALTCHK(OnStatusCommand(sCommand)); }
 bool CModules::OnModCommand(const CString& sCommand) { MODUNLOADCHK(OnModCommand(sCommand)); return false; }
 bool CModules::OnModNotice(const CString& sMessage) { MODUNLOADCHK(OnModNotice(sMessage)); return false; }
@@ -633,10 +654,6 @@ bool CGlobalModules::OnConfigLine(const CString& sName, const CString& sValue, C
 	GLOBALMODHALTCHK(OnConfigLine(sName, sValue, pUser, pChan));
 }
 
-void CGlobalModules::OnFinishedConfig() {
-	GLOBALMODCALL(OnFinishedConfig());
-}
-
 bool CGlobalModules::OnDeleteUser(CUser& User) {
 	GLOBALMODHALTCHK(OnDeleteUser(User));
 }
@@ -678,15 +695,15 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 		return false;
 	}
 
-	CString sModPath = FindModPath(sModule);
+	CString sModPath, sDataPath;
 
-	if (sModPath.empty()) {
+	if (!CZNC::Get().FindModPath(sModule, sModPath, sDataPath)) {
 		sRetMsg = "Unable to find module [" + sModule + "]";
 		return false;
 	}
 
 	if (bFake) {
-		CModule* pModule = new CModule(NULL, sModule);
+		CModule* pModule = new CModule(NULL, sModule, sDataPath);
 		pModule->SetArgs(sArgs);
 		pModule->SetDescription("<<Fake Module>>");
 		pModule->SetFake(true);
@@ -751,7 +768,8 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	CModule* pModule = NULL;
 
 	if (pUser) {
-		typedef CModule* (*fp)(void*, CUser* pUser, const CString& sModName);
+		typedef CModule* (*fp)(void*, CUser* pUser,
+				const CString& sModName, const CString& sDataPath);
 		fp Load = (fp) dlsym(p, "Load");
 
 		if (!Load) {
@@ -760,9 +778,10 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 			return false;
 		}
 
-		pModule = Load(p, pUser, sModule);
+		pModule = Load(p, pUser, sModule, sDataPath);
 	} else {
-		typedef CModule* (*fp)(void*, const CString& sModName);
+		typedef CModule* (*fp)(void*, const CString& sModName,
+				const CString& sDataPath);
 		fp Load = (fp) dlsym(p, "Load");
 
 		if (!Load) {
@@ -771,7 +790,7 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 			return false;
 		}
 
-		pModule = Load(p, sModule);
+		pModule = Load(p, sModule, sDataPath);
 	}
 
 	pModule->SetDescription(GetDesc());
@@ -879,10 +898,6 @@ bool CModules::ReloadModule(const CString& sModule, const CString& sArgs, CUser*
 	return true;
 }
 
-CString CModules::FindModPath(const CString& sModule) {
-	return CZNC::Get().FindModPath(sModule);
-}
-
 bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule) {
 	for (unsigned int a = 0; a < sModule.length(); a++) {
 		const char& c = sModule[a];
@@ -892,9 +907,9 @@ bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule) {
 		}
 	}
 
-	CString sModPath = FindModPath(sModule);
+	CString sModPath, sTmp;
 
-	if (sModPath.empty()) {
+	if (!CZNC::Get().FindModPath(sModule, sModPath, sTmp)) {
 		return false;
 	}
 
