@@ -1,15 +1,18 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include <string.h>
 #include "String.h"
 #include "FileUtils.h"
 #include "MD5.h"
+#include "Utils.h"
+#include <sstream>
+
+using std::stringstream;
 
 const char* g_szHTMLescapes[256] = {
 	"&#0;", 0, 0, 0, 0, 0, 0, 0, 0, 0,               // 0-9
@@ -169,7 +172,10 @@ inline unsigned char* CString::strnchr(const unsigned char* src, unsigned char c
     return NULL;
 }
 
-int CString::CaseCmp(const CString& s) const {
+int CString::CaseCmp(const CString& s, u_long uLen) const {
+	if (uLen != CString::npos) {
+		return strncasecmp(c_str(), s.c_str(), uLen);
+	}
 	return strcasecmp(c_str(), s.c_str());
 }
 
@@ -481,34 +487,43 @@ unsigned int CString::Replace(CString& sStr, const CString& sReplace, const CStr
 }
 
 CString CString::Token(unsigned int uPos, bool bRest, const CString& sSep) const {
-	string sRet;
-	const char* p = c_str();
-	unsigned int uSepLen = sSep.length();
+	const char *sep_str = sSep.c_str();
+	size_t sep_len = sSep.length();
+	const char *str = c_str();
+	size_t str_len = length();
+	size_t start_pos = 0;
+	size_t end_pos;
 
-	if (uSepLen) {
-		uSepLen--;
-	}
-
-	while (*p) {
-		if (uPos) {
-			if (strncmp(p, sSep.c_str(), sSep.length()) == 0) {
-				uPos--;
-				p += uSepLen;
-			}
+	// First, find the start of our token
+	while (uPos != 0 && start_pos < str_len) {
+		if (strncmp(&str[start_pos], sep_str, sep_len) == 0) {
+			start_pos += sep_len;
+			uPos--;
 		} else {
-			if (strncmp(p, sSep.c_str(), sSep.length()) == 0) {
-				if (!bRest) {
-					return sRet;
-				}
-			}
-
-			sRet += *p;
+			start_pos++;
 		}
-
-		p++;
 	}
 
-	return sRet;
+	// String is over?
+	if (start_pos >= str_len)
+		return "";
+
+	// If they want everything from here on, give it to them
+	if (bRest) {
+		return substr(start_pos);
+	}
+
+	// Now look for the end of the token they want
+	end_pos = start_pos;
+	while (end_pos < str_len) {
+		if (strncmp(&str[end_pos], sep_str, sep_len) == 0)
+			return substr(start_pos, end_pos - start_pos);
+
+		end_pos++;
+	}
+
+	// They want the last token in the string, not something in between
+	return substr(start_pos);
 }
 
 CString CString::Ellipsize(unsigned int uLen) const {
@@ -571,6 +586,12 @@ unsigned int CString::Split(const CString& sDelim, VCString& vsRet, bool bAllowE
 	unsigned int uRightLen = sRight.length();
 	const char* p = c_str();
 
+	if (!bAllowEmpty) {
+		while (strncasecmp(p, sDelim.c_str(), uDelimLen) == 0) {
+			p += uDelimLen;
+		}
+	}
+
 	while (*p) {
 		if (uLeftLen && uRightLen && !bInside && strncasecmp(p, sLeft.c_str(), uLeftLen) == 0) {
 			p += uLeftLen;
@@ -650,10 +671,17 @@ CString CString::Format(const CString& sFormatStr, ...) {
 }
 
 CString CString::RandomString(unsigned int uLength) {
+	const char chars[] = "abcdefghijklmnopqrstuvwxyz"
+		"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+		"0123456789!?.,:;/*-+_()";
+	// -1 because sizeof() includes the trailing '\0' byte
+	const size_t len = sizeof(chars) / sizeof(chars[0]) - 1;
+	size_t p;
 	CString sRet;
 
 	for (unsigned int a = 0; a < uLength; a++) {
-		sRet += (char) (rand() % 26) + 65;
+		p = (size_t) (len * (rand() / (RAND_MAX + 1.0)));
+		sRet += chars[p];
 	}
 
 	return sRet;
@@ -840,10 +868,49 @@ CString CString::ToPercent(double d) {
 	return szRet;
 }
 
-CString CString::ToKBytes(double d) {
-	char szRet[32];
-	snprintf(szRet, 32, "%.0f K/s", d);
-	return szRet;
+CString CString::ToByteStr(unsigned long long d) {
+	const unsigned long long KiB = 1024;
+	const unsigned long long MiB = KiB * 1024;
+	const unsigned long long GiB = MiB * 1024;
+	const unsigned long long TiB = GiB * 1024;
+
+	if (d > TiB) {
+		return CString(d / (double) TiB) + " TiB";
+	} else if (d > GiB) {
+		return CString(d / (double) GiB) + " GiB";
+	} else if (d > MiB) {
+		return CString(d / (double) MiB) + " MiB";
+	} else if (d > KiB) {
+		return CString(d / (double) KiB) + " KiB";
+	}
+
+	return CString(d) + " B";
+}
+
+CString CString::ToTimeStr(unsigned long s) {
+	const unsigned long m = 60;
+	const unsigned long h = m * 60;
+	const unsigned long d = h * 24;
+	const unsigned long w = d * 7;
+	const unsigned long y = d * 365;
+	CString sRet;
+
+#define TIMESPAN(time, str)			\
+	if (s >= time) {		\
+		sRet += CString(s / time) + str " "; \
+		s = s % time;		\
+	}
+	TIMESPAN(y, "y");
+	TIMESPAN(w, "w");
+	TIMESPAN(d, "d");
+	TIMESPAN(h, "h");
+	TIMESPAN(m, "m");
+	TIMESPAN(1, "s");
+
+	if (sRet.empty())
+		return "0s";
+
+	return sRet.RightChomp_n();
 }
 
 bool CString::ToBool() const { return (!Trim_n().Trim_n("0").empty() && Trim_n().CaseCmp("false") != 0); }
@@ -938,6 +1005,9 @@ bool CString::RightChomp(unsigned int uLen) {
 
 //////////////// MCString ////////////////
 int MCString::WriteToDisk(const CString& sPath, mode_t iMode) {
+	if (this->empty())
+		return MCS_SUCCESS;
+
 	CFile cFile(sPath);
 	if (!cFile.Open(O_WRONLY|O_CREAT|O_TRUNC, iMode)) {
 		return MCS_EOPEN;
@@ -967,7 +1037,7 @@ int MCString::WriteToDisk(const CString& sPath, mode_t iMode) {
 int MCString::ReadFromDisk(const CString& sPath, mode_t iMode) {
 	clear();
 	CFile cFile(sPath);
-	if (!cFile.Open(O_RDONLY|O_CREAT, iMode)) {
+	if (!cFile.Open(O_RDONLY, iMode)) {
 		return MCS_EOPEN;
 	}
 
@@ -1014,11 +1084,11 @@ CString& MCString::Decode(CString& sValue) {
 	char *endptr;
 	CString sTmp;
 
-	while(*pTmp) {
+	while (*pTmp) {
 		if (*pTmp != '%') {
 			sTmp += *pTmp++;
 		} else {
-			char ch = (char )strtol(((const char *)pTmp + 1), &endptr, 16);
+			char ch = (char) strtol(pTmp + 1, &endptr, 16);
 			if (*endptr == ';') {
 				sTmp += ch;
 				pTmp = ++endptr;
