@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,7 +9,9 @@
 #ifndef _TIMERS_H
 #define _TIMERS_H
 
-#include "main.h"
+#include "Client.h"
+#include "IRCSock.h"
+#include "User.h"
 
 class CKeepNickTimer : public CCron {
 public:
@@ -91,4 +93,24 @@ protected:
 	CUser*	m_pUser;
 };
 
+class CClientTimeout : public CCron {
+public:
+	CClientTimeout(CClient* pClient) : CCron() {
+		m_pClient = pClient;
+		SetName("CClientTimeout::UNKNOWN");
+		StartMaxCycles(60, 1);
+	}
+	virtual ~CClientTimeout() {}
+
+protected:
+	virtual void RunJob() {
+		if (m_pClient) {
+			m_pClient->LoginTimeout();
+			m_pClient = NULL;
+		}
+	}
+
+	CClient*	m_pClient;
+};
+
 #endif // !_TIMERS_H
