@@ -1,23 +1,21 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
-#include "znc.h"
 #include "User.h"
-#include "Server.h"
-#include "IRCSock.h"
-#include "Client.h"
-#include "DCCBounce.h"
+#include "Chan.h"
 #include "DCCSock.h"
-#include "MD5.h"
+#include "IRCSock.h"
+#include "Server.h"
 #include "Timers.h"
+#include "znc.h"
 
 CUser::CUser(const CString& sUserName) {
+	m_pIRCSock = NULL;
 	m_fTimezoneOffset = 0;
 	m_uConnectTime = 0;
 	SetUserName(sUserName);
@@ -40,8 +38,9 @@ CUser::CUser(const CString& sUserName) {
 	m_bKeepNick = false;
 	m_bDenyLoadMod = false;
 	m_bAdmin= false;
+	m_bDenySetVHost= false;
 	m_sStatusPrefix = "*";
-	m_sChanPrefixes = "#&";
+	m_sChanPrefixes = "";
 	m_uBufferCount = 50;
 	m_uMaxJoinTries = 0;
 	m_bKeepBuffer = false;
@@ -62,20 +61,18 @@ CUser::CUser(const CString& sUserName) {
 }
 
 CUser::~CUser() {
-	for (unsigned int a = 0; a < m_vServers.size(); a++) {
-		delete m_vServers[a];
-	}
-
-	for (unsigned int b = 0; b < m_vChans.size(); b++) {
-		delete m_vChans[b];
-	}
-
 	DelClients();
 
 #ifdef _MODULES
 	DelModules();
 #endif
 
+	DelServers();
+
+	for (unsigned int b = 0; b < m_vChans.size(); b++) {
+		delete m_vChans[b];
+	}
+
 	CZNC::Get().GetManager().DelCronByAddr(m_pKeepNickTimer);
 	CZNC::Get().GetManager().DelCronByAddr(m_pJoinTimer);
 	CZNC::Get().GetManager().DelCronByAddr(m_pMiscTimer);
@@ -99,15 +96,18 @@ void CUser::DelClients() {
 	m_vClients.clear();
 }
 
-bool CUser::OnBoot() {
-#ifdef _MODULES
-	return GetModules().OnBoot();
-#else
-	return true;
-#endif
+void CUser::DelServers()
+{
+	for (unsigned int a = 0; a < m_vServers.size(); a++) {
+		delete m_vServers[a];
+	}
+
+	m_vServers.clear();
 }
 
 void CUser::IRCConnected(CIRCSock* pIRCSock) {
+	m_pIRCSock = pIRCSock;
+
 	for (unsigned int a = 0; a < m_vClients.size(); a++) {
 		m_vClients[a]->IRCConnected(pIRCSock);
 	}
@@ -116,6 +116,8 @@ void CUser::IRCConnected(CIRCSock* pIRCSock) {
 void CUser::IRCDisconnected() {
 	m_bIRCConnected = false;
 
+	m_pIRCSock = NULL;
+
 	for (unsigned int a = 0; a < m_vClients.size(); a++) {
 		m_vClients[a]->IRCDisconnected();
 	}
@@ -130,6 +132,17 @@ CString CUser::ExpandString(const CString& sStr) const {
 }
 
 CString& CUser::ExpandString(const CString& sStr, CString& sRet) const {
+	// offset is in hours, so * 60 * 60 gets us seconds
+	time_t iUserTime = time(NULL) + (time_t)(m_fTimezoneOffset * 60 * 60);
+	char *szTime = ctime(&iUserTime);
+	CString sTime;
+
+	if (szTime) {
+		sTime = szTime;
+		// ctime() adds a trailing newline
+		sTime.Trim();
+	}
+
 	sRet = sStr;
 	sRet.Replace("%user%", GetUserName());
 	sRet.Replace("%defnick%", GetNick());
@@ -138,6 +151,13 @@ CString& CUser::ExpandString(const CString& sStr, CString& sRet) const {
 	sRet.Replace("%ident%", GetIdent());
 	sRet.Replace("%realname%", GetRealName());
 	sRet.Replace("%vhost%", GetVHost());
+	sRet.Replace("%version%", CZNC::GetVersion());
+	sRet.Replace("%time%", sTime);
+	sRet.Replace("%uptime%", CZNC::Get().GetUptime());
+	// The following lines do not exist. You must be on DrUgS!
+	sRet.Replace("%znc%", "All your IRC are belong to ZNC");
+	// Chosen by fair zocchihedron dice roll by SilverLeo
+	sRet.Replace("%rand%", "42");
 
 	return sRet;
 }
@@ -151,7 +171,7 @@ CString& CUser::AddTimestamp(const CString& sStr, CString& sRet) const {
 	char szTimestamp[1024];
 	time_t tm;
 
-	if(GetTimestampFormat().empty() || (!m_bAppendTimestamp && !m_bPrependTimestamp)) {
+	if (GetTimestampFormat().empty() || (!m_bAppendTimestamp && !m_bPrependTimestamp)) {
 		sRet = sStr;
 	} else {
 		time(&tm);
@@ -166,7 +186,7 @@ CString& CUser::AddTimestamp(const CString& sStr, CString& sRet) const {
 		if (m_bAppendTimestamp) {
 			sRet += " ";
 			sRet += szTimestamp;
-		} 
+		}
 	}
 	return sRet;
 }
@@ -210,14 +230,14 @@ void CUser::UserConnected(CClient* pClient) {
 		}
 	}
 
-	if(GetIRCSock() != NULL) {
+	if (GetIRCSock() != NULL) {
 		CString sUserMode("");
 		const set<unsigned char>& scUserModes = GetIRCSock()->GetUserModes();
 		for (set<unsigned char>::iterator it = scUserModes.begin();
 				it != scUserModes.end(); it++) {
 			sUserMode += *it;
 		}
-		if(!sUserMode.empty()) {
+		if (!sUserMode.empty()) {
 			pClient->PutClient(":" + GetIRCNick().GetNick() + " MODE " + GetIRCNick().GetNick() + " :+" + sUserMode);
 		}
 	}
@@ -236,7 +256,7 @@ void CUser::UserConnected(CClient* pClient) {
 
 	// Tell them why they won't connect
 	if (!GetIRCConnectEnabled())
-		PutStatus("You are currently disconnected from IRC. "
+		pClient->PutStatus("You are currently disconnected from IRC. "
 				"Use 'connect' to reconnect.");
 }
 
@@ -249,7 +269,7 @@ void CUser::UserDisconnected(CClient* pClient) {
 	}
 }
 
-bool CUser::Clone(const CUser& User, CString& sErrorRet) {
+bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	unsigned int a = 0;
 	sErrorRet.clear();
 
@@ -279,6 +299,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 	SetQuitMsg(User.GetQuitMsg());
 	SetDefaultChanModes(User.GetDefaultChanModes());
 	SetBufferCount(User.GetBufferCount());
+	SetJoinTries(User.JoinTries());
 
 	// Allowed Hosts
 	m_ssAllowedHosts.clear();
@@ -287,7 +308,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 		AddAllowedHost(*it);
 	}
 
-	for (unsigned int a = 0; a < m_vClients.size(); a++) {
+	for (a = 0; a < m_vClients.size(); a++) {
 		CClient* pSock = m_vClients[a];
 
 		if (!IsHostAllowed(pSock->GetRemoteIP())) {
@@ -344,7 +365,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 		sServer = pCurServ->GetName();
 	}
 
-	m_vServers.clear();
+	DelServers();
 
 	for (a = 0; a < vServers.size(); a++) {
 		CServer* pServer = vServers[a];
@@ -384,9 +405,13 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 
 	for (a = 0; a < m_vChans.size(); a++) {
 		CChan* pChan = m_vChans[a];
+		CChan* pNewChan = User.FindChan(pChan->GetName());
 
-		if (!User.FindChan(pChan->GetName())) {
+		if (!pNewChan) {
 			pChan->SetInConfig(false);
+		} else {
+			if (bCloneChans)
+				pChan->Clone(*pNewChan);
 		}
 	}
 
@@ -410,9 +435,11 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet) {
 	SetUseClientIP(User.UseClientIP());
 	SetDenyLoadMod(User.DenyLoadMod());
 	SetAdmin(User.IsAdmin());
+	SetDenySetVHost(User.DenySetVHost());
 	SetTimestampAppend(User.GetTimestampAppend());
 	SetTimestampPrepend(User.GetTimestampPrepend());
 	SetTimestampFormat(User.GetTimestampFormat());
+	SetTimezoneOffset(User.GetTimezoneOffset());
 	// !Flags
 
 	return true;
@@ -532,11 +559,11 @@ bool CUser::PrintLine(CFile& File, const CString& sName, const CString& sValue)
 		return false;
 	}
 
-	return File.Write("\t" + sName + " = " + sValue + "\r\n");
+	return File.Write("\t" + sName + " = " + sValue + "\n");
 }
 
 bool CUser::WriteConfig(CFile& File) {
- 	File.Write("<User " + GetUserName() + ">\r\n");
+	File.Write("<User " + GetUserName() + ">\n");
 
 	PrintLine(File, "Pass", GetPass() + ((IsPassHashed()) ? " -" : ""));
 	PrintLine(File, "Nick", GetNick());
@@ -545,7 +572,8 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "RealName", GetRealName());
 	PrintLine(File, "VHost", GetVHost());
 	PrintLine(File, "QuitMsg", GetQuitMsg());
-	PrintLine(File, "StatusPrefix", GetStatusPrefix());
+	if (CZNC::Get().GetStatusPrefix() != GetStatusPrefix())
+		PrintLine(File, "StatusPrefix", GetStatusPrefix());
 	PrintLine(File, "ChanModes", GetDefaultChanModes());
 	PrintLine(File, "Buffer", CString(GetBufferCount()));
 	PrintLine(File, "KeepNick", CString((GetKeepNick()) ? "true" : "false"));
@@ -555,13 +583,14 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "AutoCycle", CString((AutoCycle()) ? "true" : "false"));
 	PrintLine(File, "DenyLoadMod", CString((DenyLoadMod()) ? "true" : "false"));
 	PrintLine(File, "Admin", CString((IsAdmin()) ? "true" : "false"));
+	PrintLine(File, "DenySetVHost", CString((DenySetVHost()) ? "true" : "false"));
 	PrintLine(File, "DCCLookupMethod", CString((UseClientIP()) ? "client" : "default"));
 	PrintLine(File, "TimestampFormat", GetTimestampFormat());
 	PrintLine(File, "AppendTimestamp", CString((GetTimestampAppend()) ? "true" : "false"));
 	PrintLine(File, "PrependTimestamp", CString((GetTimestampPrepend()) ? "true" : "false"));
 	PrintLine(File, "TimezoneOffset", CString(m_fTimezoneOffset));
 	PrintLine(File, "JoinTries", CString(m_uMaxJoinTries));
-	File.Write("\r\n");
+	File.Write("\n");
 
 	// Allow Hosts
 	if (!m_ssAllowedHosts.empty()) {
@@ -569,7 +598,7 @@ bool CUser::WriteConfig(CFile& File) {
 			PrintLine(File, "Allow", *it);
 		}
 
-		File.Write("\r\n");
+		File.Write("\n");
 	}
 
 	// CTCP Replies
@@ -578,7 +607,7 @@ bool CUser::WriteConfig(CFile& File) {
 			PrintLine(File, "CTCPReply", itb->first.AsUpper() + " " + itb->second);
 		}
 
-		File.Write("\r\n");
+		File.Write("\n");
 	}
 
 #ifdef _MODULES
@@ -596,7 +625,7 @@ bool CUser::WriteConfig(CFile& File) {
 			PrintLine(File, "LoadModule", Mods[a]->GetModName() + sArgs);
 		}
 
-		File.Write("\r\n");
+		File.Write("\n");
 	}
 #endif
 
@@ -609,14 +638,14 @@ bool CUser::WriteConfig(CFile& File) {
 	for (unsigned int c = 0; c < m_vChans.size(); c++) {
 		CChan* pChan = m_vChans[c];
 		if (pChan->InConfig()) {
-			File.Write("\r\n");
+			File.Write("\n");
 			if (!pChan->WriteConfig(File)) {
 				return false;
 			}
 		}
 	}
 
-	File.Write("</User>\r\n");
+	File.Write("</User>\n");
 
 	return true;
 }
@@ -779,7 +808,7 @@ bool CUser::CheckPass(const CString& sPass) {
 		return (sPass == m_sPass);
 	}
 
-	return (m_sPass.CaseCmp((char*) CMD5(sPass)) == 0);
+	return (m_sPass.CaseCmp(sPass.MD5()) == 0);
 }
 
 /*CClient* CUser::GetClient() {
@@ -800,13 +829,11 @@ bool CUser::CheckPass(const CString& sPass) {
 }*/
 
 CIRCSock* CUser::GetIRCSock() {
-	// Todo: optimize this by saving a pointer to the sock
-	return (CIRCSock*) CZNC::Get().GetManager().FindSockByName("IRC::" + m_sUserName);
+	return m_pIRCSock;
 }
 
 const CIRCSock* CUser::GetIRCSock() const {
-	// Todo: same as above
-	return (CIRCSock*) CZNC::Get().GetManager().FindSockByName("IRC::" + m_sUserName);
+	return m_pIRCSock;
 }
 
 CString CUser::GetLocalIP() {
@@ -912,7 +939,7 @@ bool CUser::ResumeFile(const CString& sRemoteNick, unsigned short uPort, unsigne
 }
 
 bool CUser::SendFile(const CString& sRemoteNick, const CString& sFileName, const CString& sModuleName) {
-	CString sFullPath = CUtils::ChangeDir(GetDLPath(), sFileName, CZNC::Get().GetHomePath());
+	CString sFullPath = CDir::ChangeDir(GetDLPath(), sFileName, CZNC::Get().GetHomePath());
 	CDCCSock* pSock = new CDCCSock(this, sRemoteNick, sFullPath, sModuleName);
 
 	CFile* pFile = pSock->OpenFile(false);
@@ -926,7 +953,7 @@ bool CUser::SendFile(const CString& sRemoteNick, const CString& sFileName, const
 
 	if (GetNick().CaseCmp(sRemoteNick) == 0) {
 		PutUser(":" + GetStatusPrefix() + "status!znc@znc.com PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
-			   	+ CString(uPort) + " " + CString(pFile->GetSize()) + "\001");
+				+ CString(uPort) + " " + CString(pFile->GetSize()) + "\001");
 	} else {
 		PutIRC("PRIVMSG " + sRemoteNick + " :\001DCC SEND " + pFile->GetShortName() + " " + CString(CUtils::GetLongIP(GetLocalIP())) + " "
 			    + CString(uPort) + " " + CString(pFile->GetSize()) + "\001");
@@ -982,6 +1009,15 @@ void CUser::SetUserName(const CString& s) {
 	m_sUserName = s;
 }
 
+bool CUser::IsChan(const CString& sChan) const {
+	if (sChan.empty())
+		return false; // There is no way this is a chan
+	if (GetChanPrefixes().empty())
+		return true; // We can't know, so we allow everything
+	// Thanks to the above if (empty), we can do sChan[0]
+	return GetChanPrefixes().find(sChan[0]) != CString::npos;
+}
+
 void CUser::SetNick(const CString& s) { m_sNick = s; }
 void CUser::SetAltNick(const CString& s) { m_sAltNick = s; }
 void CUser::SetIdent(const CString& s) { m_sIdent = s; }
@@ -994,6 +1030,7 @@ void CUser::SetUseClientIP(bool b) { m_bUseClientIP = b; }
 void CUser::SetKeepNick(bool b) { m_bKeepNick = b; }
 void CUser::SetDenyLoadMod(bool b) { m_bDenyLoadMod = b; }
 void CUser::SetAdmin(bool b) { m_bAdmin = b; }
+void CUser::SetDenySetVHost(bool b) { m_bDenySetVHost = b; }
 void CUser::SetDefaultChanModes(const CString& s) { m_sDefaultChanModes = s; }
 void CUser::SetIRCServer(const CString& s) { m_sIRCServer = s; m_bIRCConnected = true; }
 void CUser::SetQuitMsg(const CString& s) { m_sQuitMsg = s; }
@@ -1049,12 +1086,12 @@ bool CUser::IsPassHashed() const { return m_bPassHashed; }
 bool CUser::ConnectPaused() {
 	if (!m_uConnectTime) {
 		m_uConnectTime = time(NULL);
-		return !m_bIRCConnectEnabled;
+		return false;
 	}
 
 	if (time(NULL) - m_uConnectTime >= 5) {
 		m_uConnectTime = time(NULL);
-		return !m_bIRCConnectEnabled;
+		return false;
 	}
 
 	return true;
@@ -1064,6 +1101,7 @@ bool CUser::UseClientIP() const { return m_bUseClientIP; }
 bool CUser::GetKeepNick() const { return m_bKeepNick; }
 bool CUser::DenyLoadMod() const { return m_bDenyLoadMod; }
 bool CUser::IsAdmin() const { return m_bAdmin; }
+bool CUser::DenySetVHost() const { return m_bDenySetVHost; }
 bool CUser::MultiClients() const { return m_bMultiClients; }
 bool CUser::BounceDCCs() const { return m_bBounceDCCs; }
 const CString& CUser::GetStatusPrefix() const { return m_sStatusPrefix; }
