@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -8,20 +8,19 @@
 
 #include "Utils.h"
 #include "MD5.h"
-#include <sys/stat.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <stdio.h>
-#include <sstream>
 #include <errno.h>
 #ifdef HAVE_LIBSSL
 #include <openssl/ssl.h>
 #endif /* HAVE_LIBSSL */
+#include <sstream>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 using std::stringstream;
 
+bool CUtils::stdoutIsTTY;
+
 CUtils::CUtils() {}
 CUtils::~CUtils() {}
 
@@ -55,30 +54,30 @@ void CUtils::GenerateCert(FILE *pOut, bool bEncPrivKey, const CString& sHost) {
 	X509_NAME *pName = NULL;
 	int days = 365;
 
-	u_int iSeed = time( NULL );
-	int serial = ( rand_r( &iSeed ) % 9999 );
+	u_int iSeed = time(NULL);
+	int serial = (rand_r(&iSeed) % 9999);
 
 	RSA *pRSA = RSA_generate_key(1024, 0x10001, NULL, NULL);
-	if( ( pKey = EVP_PKEY_new() ) ) {
-		if ( !EVP_PKEY_assign_RSA( pKey, pRSA ) ) {
-		    EVP_PKEY_free( pKey );
+	if ((pKey = EVP_PKEY_new())) {
+		if (!EVP_PKEY_assign_RSA(pKey, pRSA)) {
+			EVP_PKEY_free(pKey);
 			return;
 		}
 
-		PEM_write_RSAPrivateKey( pOut, pRSA, (bEncPrivKey ? EVP_des_ede3_cbc() : NULL), NULL, 0, NULL, NULL);
+		PEM_write_RSAPrivateKey(pOut, pRSA, (bEncPrivKey ? EVP_des_ede3_cbc() : NULL), NULL, 0, NULL, NULL);
 
-		if( !( pCert = X509_new() ) ) {
-		    EVP_PKEY_free( pKey );
+		if (!(pCert = X509_new())) {
+			EVP_PKEY_free(pKey);
 			return;
 		}
 
-		X509_set_version( pCert, 2 );
-		ASN1_INTEGER_set( X509_get_serialNumber( pCert ), serial );
-		X509_gmtime_adj( X509_get_notBefore( pCert ), 0 );
-		X509_gmtime_adj( X509_get_notAfter( pCert ), (long)60*60*24*days );
-		X509_set_pubkey( pCert, pKey );
-		
-		pName = X509_get_subject_name( pCert );
+		X509_set_version(pCert, 2);
+		ASN1_INTEGER_set(X509_get_serialNumber(pCert), serial);
+		X509_gmtime_adj(X509_get_notBefore(pCert), 0);
+		X509_gmtime_adj(X509_get_notAfter(pCert), (long)60*60*24*days);
+		X509_set_pubkey(pCert, pKey);
+
+		pName = X509_get_subject_name(pCert);
 
 		const char *pLogName = getenv("LOGNAME");
 		const char *pHostName = NULL;
@@ -103,25 +102,25 @@ void CUtils::GenerateCert(FILE *pOut, bool bEncPrivKey, const CString& sHost) {
 		sEmailAddr += "@";
 		sEmailAddr += pHostName;
 
-		X509_NAME_add_entry_by_txt( pName, "C", MBSTRING_ASC, (unsigned char *)"US", -1, -1, 0);
-		X509_NAME_add_entry_by_txt( pName, "ST", MBSTRING_ASC, (unsigned char *)"SomeState", -1, -1, 0);
-		X509_NAME_add_entry_by_txt( pName, "L", MBSTRING_ASC, (unsigned char *)"SomeCity", -1, -1, 0);
-		X509_NAME_add_entry_by_txt( pName, "O", MBSTRING_ASC, (unsigned char *)"SomeCompany", -1, -1, 0);
-		X509_NAME_add_entry_by_txt( pName, "OU", MBSTRING_ASC, (unsigned char *)pLogName, -1, -1, 0);
-		X509_NAME_add_entry_by_txt( pName, "CN", MBSTRING_ASC, (unsigned char *)pHostName, -1, -1, 0);
-		X509_NAME_add_entry_by_txt( pName, "emailAddress", MBSTRING_ASC, (unsigned char *)sEmailAddr.c_str(), -1, -1, 0);
+		X509_NAME_add_entry_by_txt(pName, "C", MBSTRING_ASC, (unsigned char *)"US", -1, -1, 0);
+		X509_NAME_add_entry_by_txt(pName, "ST", MBSTRING_ASC, (unsigned char *)"SomeState", -1, -1, 0);
+		X509_NAME_add_entry_by_txt(pName, "L", MBSTRING_ASC, (unsigned char *)"SomeCity", -1, -1, 0);
+		X509_NAME_add_entry_by_txt(pName, "O", MBSTRING_ASC, (unsigned char *)"SomeCompany", -1, -1, 0);
+		X509_NAME_add_entry_by_txt(pName, "OU", MBSTRING_ASC, (unsigned char *)pLogName, -1, -1, 0);
+		X509_NAME_add_entry_by_txt(pName, "CN", MBSTRING_ASC, (unsigned char *)pHostName, -1, -1, 0);
+		X509_NAME_add_entry_by_txt(pName, "emailAddress", MBSTRING_ASC, (unsigned char *)sEmailAddr.c_str(), -1, -1, 0);
 
-		X509_set_subject_name( pCert, pName );
+		X509_set_subject_name(pCert, pName);
 
-		if ( !X509_sign( pCert, pKey, EVP_md5() ) ) {
-		    X509_free( pCert );
-		    EVP_PKEY_free( pKey );
+		if (!X509_sign(pCert, pKey, EVP_md5())) {
+			X509_free(pCert);
+			EVP_PKEY_free(pKey);
 			return;
 		}
 
-		PEM_write_X509( pOut, pCert );
-		X509_free( pCert );
-		EVP_PKEY_free( pKey );
+		PEM_write_X509(pOut, pCert);
+		X509_free(pCert);
+		EVP_PKEY_free(pKey);
 	}
 }
 #endif /* HAVE_LIBSSL */
@@ -143,119 +142,21 @@ CString CUtils::GetIP(unsigned long addr) {
 }
 
 unsigned long CUtils::GetLongIP(const CString& sIP) {
-	register int i;
-	char *addr = (char *) malloc(sIP.length() +1);
-	char ip[4][4], n;
+	unsigned long ret;
+	char ip[4][4];
+	int i;
 
-	strcpy(addr, sIP.c_str());
-
-	for (i=0; i<4; ip[0][i]=ip[1][i]=ip[2][i]=ip[3][i]='\0', i++);
-
-	if (sscanf(addr, "%3[0-9].%3[0-9].%3[0-9].%3[0-9]%[^\n]", ip[0], ip[1], ip[2], ip[3], &n) != 4) {
-		free(addr);
+	i = sscanf(sIP.c_str(), "%3[0-9].%3[0-9].%3[0-9].%3[0-9]",
+			ip[0], ip[1], ip[2], ip[3]);
+	if (i != 4)
 		return 0;
-	}
 
-	free(addr);
-	return (unsigned long) ((atoi(ip[0]) << 24) + (atoi(ip[1]) << 16) + (atoi(ip[2]) << 8) + atoi(ip[3]));
-}
+	ret  = atoi(ip[0]) << 24;
+	ret += atoi(ip[1]) << 16;
+	ret += atoi(ip[2]) << 8;
+	ret += atoi(ip[3]) << 0;
 
-CString CUtils::ChangeDir(const CString& sPath, const CString& sAdd, const CString& sHomeDir) {
-	if (sAdd == "~") {
-		return sHomeDir;
-	}
-
-	CString sAddDir = sAdd;
-
-	if (sAddDir.Left(2) == "~/") {
-		sAddDir.LeftChomp();
-		sAddDir = sHomeDir + sAddDir;
-	}
-
-	CString sRet = ((sAddDir.size()) && (sAddDir[0] == '/')) ? "" : sPath;
-	sAddDir += "/";
-	CString sCurDir;
-
-	if (sRet.Right(1) == "/") {
-		sRet.RightChomp();
-	}
-
-	for (unsigned int a = 0; a < sAddDir.size(); a++) {
-		switch (sAddDir[a]) {
-			case '/':
-				if (sCurDir == "..") {
-					sRet = sRet.substr(0, sRet.rfind('/'));
-				} else if ((sCurDir != "") && (sCurDir != ".")) {
-					sRet += "/" + sCurDir;
-				}
-
-				sCurDir = "";
-				break;
-			default:
-				sCurDir += sAddDir[a];
-				break;
-		}
-	}
-
-	return (sRet.empty()) ? "/" : sRet;
-}
-
-int CUtils::MakeDir(const CString& sPath, mode_t iMode) {
-	CString sDir = sPath;
-	CString::size_type iFind = sDir.find("/");
-
-	if (iFind == CString::npos) {
-		return mkdir(sDir.c_str(), iMode);
-	}
-	iFind++;
-
-	while ((iFind < sDir.length()) && (sDir[iFind] == '/')) {
-		iFind++; // eat up extra /'s
-	}
-
-	if (iFind >= sDir.length()) {
-		return mkdir(sDir.c_str(), iMode);
-	}
-
-	CString sWorkDir = sDir.substr(0, iFind);  // include the trailing slash
-	CString sNewDir = sDir.erase(0, iFind);
-
-	struct stat st;
-
-	if (sWorkDir.length() > 1) {
-		sWorkDir = sWorkDir.erase(sWorkDir.length() - 1, 1);  // trim off the trailing slash
-	}
-
-	if (stat(sWorkDir.c_str(), &st) == 0) {
-		int iChdir = chdir(sWorkDir.c_str());
-		if (iChdir != 0) {
-			return iChdir;   // could not change to dir
-		}
-
-		// go ahead and call the next step
-		return MakeDir(sNewDir.c_str(), iMode);
-	}
-
-	switch(errno) {
-		case ENOENT: {
-			// ok, file doesn't exists, lets create it and cd into it
-			int iMkdir = mkdir(sWorkDir.c_str(), iMode);
-			if (iMkdir != 0) {
-				return iMkdir; // could not create dir
-			}
-
-			int iChdir = chdir(sWorkDir.c_str());
-			if (iChdir != 0) {
-				return iChdir;       // could not change to dir
-			}
-
-			return MakeDir(sNewDir.c_str(), iMode);
-		}
-		default:
-			break;
-	}
-
-	return -1;
+	return ret;
 }
 
 CString CUtils::GetHashPass() {
@@ -356,7 +257,11 @@ bool CUtils::GetInput(const CString& sPrompt, CString& sRet, const CString& sDef
 	PrintPrompt(sPrompt + sExtra);
 	char szBuf[1024];
 	memset(szBuf, 0, 1024);
-	fgets(szBuf, 1024, stdin);
+	if (fgets(szBuf, 1024, stdin) == NULL) {
+		// Reading failed (Error? EOF?)
+		PrintError("Error while reading from stdin. Exiting...");
+		exit(-1);
+	}
 	sInput = szBuf;
 
 	if (sInput.Right(1) == "\n") {
@@ -373,45 +278,71 @@ bool CUtils::GetInput(const CString& sPrompt, CString& sRet, const CString& sDef
 }
 
 void CUtils::PrintError(const CString& sMessage) {
-	fprintf(stdout, "\033[1m\033[34m[\033[31m ** \033[34m]\033[39m\033[22m %s\n", sMessage.c_str());
+	if (stdoutIsTTY)
+		fprintf(stdout, "\033[1m\033[34m[\033[31m ** \033[34m]\033[39m\033[22m %s\n", sMessage.c_str());
+	else
+		fprintf(stdout, "[ ** ] %s\n", sMessage.c_str());
 	fflush(stdout);
 }
 
 void CUtils::PrintPrompt(const CString& sMessage) {
-	fprintf(stdout, "\033[1m\033[34m[\033[33m ?? \033[34m]\033[39m\033[22m %s: ", sMessage.c_str());
+	if (stdoutIsTTY)
+		fprintf(stdout, "\033[1m\033[34m[\033[33m ?? \033[34m]\033[39m\033[22m %s: ", sMessage.c_str());
+	else
+		fprintf(stdout, "[ ?? ] %s: ", sMessage.c_str());
 	fflush(stdout);
 }
 
 void CUtils::PrintMessage(const CString& sMessage, bool bStrong) {
-	fprintf(stdout, "\033[1m\033[34m[\033[33m ** \033[34m]\033[39m\033[22m %s%s%s\n",
-		((bStrong) ? "\033[1m" : ""),
-		sMessage.c_str(),
-		((bStrong) ? "\033[22m" : "")
-	);
+	if (stdoutIsTTY) {
+		if (bStrong)
+			fprintf(stdout, "\033[1m\033[34m[\033[33m ** \033[34m]\033[39m\033[22m \033[1m%s\033[22m\n",
+					sMessage.c_str());
+		else
+			fprintf(stdout, "\033[1m\033[34m[\033[33m ** \033[34m]\033[39m\033[22m %s\n",
+					sMessage.c_str());
+	} else
+		fprintf(stdout, "%s\n", sMessage.c_str());
 
 	fflush(stdout);
 }
 
 void CUtils::PrintAction(const CString& sMessage) {
-	fprintf(stdout, "\033[1m\033[34m[\033[32m    \033[34m]\033[39m\033[22m %s... ", sMessage.c_str());
+	if (stdoutIsTTY)
+		fprintf(stdout, "\033[1m\033[34m[\033[32m    \033[34m]\033[39m\033[22m %s... ", sMessage.c_str());
+	else
+		fprintf(stdout, "%s... ", sMessage.c_str());
 	fflush(stdout);
 }
 
 void CUtils::PrintStatus(bool bSuccess, const CString& sMessage) {
-	if (!sMessage.empty()) {
-		if (bSuccess) {
-			fprintf(stdout, "%s", sMessage.c_str());
-		} else {
-			fprintf(stdout, "\033[1m\033[34m[\033[31m %s \033[34m]\033[39m\033[22m", sMessage.c_str());
+	if (stdoutIsTTY) {
+		if (!sMessage.empty()) {
+			if (bSuccess) {
+				fprintf(stdout, "%s", sMessage.c_str());
+			} else {
+				fprintf(stdout, "\033[1m\033[34m[\033[31m %s \033[34m]"
+						"\033[39m\033[22m", sMessage.c_str());
+			}
 		}
-	}
 
-	fprintf(stdout, "\r");
+		fprintf(stdout, "\r");
 
-	if (bSuccess) {
-		fprintf(stdout, "\033[1m\033[34m[\033[32m ok \033[34m]\033[39m\033[22m\n");
+		if (bSuccess) {
+			fprintf(stdout, "\033[1m\033[34m[\033[32m ok \033[34m]\033[39m\033[22m\n");
+		} else {
+			fprintf(stdout, "\033[1m\033[34m[\033[31m !! \033[34m]\033[39m\033[22m\n");
+		}
 	} else {
-		fprintf(stdout, "\033[1m\033[34m[\033[31m !! \033[34m]\033[39m\033[22m\n");
+		if (bSuccess) {
+			fprintf(stdout, "%s\n", sMessage.c_str());
+		} else {
+			if (!sMessage.empty()) {
+				fprintf(stdout, "[ %s ]", sMessage.c_str());
+			}
+
+			fprintf(stdout, "\n[ !! ]\n");
+		}
 	}
 
 	fflush(stdout);
@@ -594,7 +525,7 @@ CString CBlowfish::MD5(const CString & sInput, bool bHexEncode) {
 	if (!bHexEncode) {
 		sRet.append((const char *)data, MD5_DIGEST_LENGTH);
 	} else {
-		for(int a = 0; a < MD5_DIGEST_LENGTH; a++) {
+		for (int a = 0; a < MD5_DIGEST_LENGTH; a++) {
 			sRet += g_HexDigits[data[a] >> 4];
 			sRet += g_HexDigits[data[a] & 0xf];
 		}
