@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,18 +9,16 @@
 #ifndef _UTILS_H
 #define _UTILS_H
 
-#include <unistd.h>
-#include <ctype.h>
-#include <sys/time.h>
-#include <sys/file.h>
-#include <stdio.h>
-#include <assert.h>
-
 #include "String.h"
-#include <vector>
+#include <assert.h>
 #include <map>
-using std::vector;
+#include <sys/file.h>
+#include <sys/time.h>
+#include <unistd.h>
+#include <vector>
+
 using std::map;
+using std::vector;
 
 #ifdef _DEBUG
 #define DEBUG_ONLY(f)   f
@@ -28,6 +26,15 @@ using std::map;
 #define DEBUG_ONLY(f) ((void)0)
 #endif
 
+static inline void SetFdCloseOnExec(int fd)
+{
+	int flags = fcntl(fd, F_GETFD, 0);
+	if (flags < 0)
+		return; // Ignore errors
+	// When we execve() a new process this fd is now automatically closed.
+	fcntl(fd, F_SETFD, flags | FD_CLOEXEC);
+}
+
 static const char g_HexDigits[] = "0123456789abcdef";
 
 class CUtils {
@@ -37,8 +44,8 @@ public:
 
 	static CString GetIP(unsigned long addr);
 	static unsigned long GetLongIP(const CString& sIP);
-	static CString ChangeDir(const CString& sPath, const CString& sAdd, const CString& sHomeDir);
-	static int MakeDir(const CString& sPath, mode_t iMode = 0700);
+	static void SetStdoutIsTTY(bool b) { stdoutIsTTY = b; }
+
 	static void PrintError(const CString& sMessage);
 	static void PrintMessage(const CString& sMessage, bool bStrong = false);
 	static void PrintPrompt(const CString& sMessage);
@@ -65,95 +72,78 @@ public:
 
 private:
 protected:
+	static bool stdoutIsTTY;
 };
 
 class CLockFile {
 public:
 	CLockFile() {
-		m_bCreated = false;
-		m_fd = -1;
-		m_pid = 0;
+		Init();
 	}
 
 	CLockFile(const CString& sFile) {
+		Init();
 		Open(sFile);
 	}
 
 	virtual ~CLockFile() {
-		if (getpid() == m_pid) {
-			if (m_fd > -1) {
-				UnLock();
-				close(m_fd);
-
-				if (m_bCreated) {
-					unlink(m_sFileName.c_str());
-				}
-			}
-		}
+		Close();
 	}
 
-	void Open(const CString& sFile, bool bRw = false) {
+	void Close() {
+		if (getpid() == m_pid && m_fd > -1) {
+			// This UnLock() also unlocks for all of our childs!
+			UnLock();
+			close(m_fd);
+
+			if (m_bCreated) {
+				unlink(m_sFileName.c_str());
+			}
+		} else if (m_fd > -1)
+			// Make sure we don't leak this fd
+			close(m_fd);
+
+		Init();
+	}
+
+	bool Open(const CString& sFile, bool bRw = false) {
+		Close();
+
 		m_fd = open(sFile.c_str(), bRw ? O_RDWR : O_RDONLY);
 		m_bCreated = false;
 
 		if (m_fd == -1) {
-			// i must create the file then
+			// I must create the file then
 			m_fd = open(sFile.c_str(), O_RDWR|O_CREAT, 0644);
+
+			if (m_fd == -1) {
+				return false;
+			}
+
 			m_bCreated = true;
 		}
 
+		// Thanks to broken POSIX, we shouldn't give this fd to anyone
+		SetFdCloseOnExec(m_fd);
+
 		m_pid = getpid();       // for destructor
 		m_sFileName = sFile;
+
+		return true;
 	}
 
 	//! timeout in milliseconds
-	bool TryExLock(const CString& sLockFile, unsigned long long iTimeout = 0, bool bRw = false) {
+	bool TryExLock(const CString& sLockFile, bool bRw = false) {
 		Open(sLockFile, bRw);
-		return TryExLock(iTimeout);
+		return TryExLock();
 	}
 
-	bool TryExLock(unsigned long long iTimeout = 0) {
-		if (iTimeout == 0) {
-			return Lock(LOCK_EX|LOCK_NB);
-		}
-
-		unsigned long long iNow = CUtils::GetMillTime();
-
-		while(true) {
-			if (Lock(LOCK_EX|LOCK_NB)) {
-				return true;
-			}
-
-			if ((CUtils::GetMillTime() - iNow) > iTimeout) {
-				break;
-			}
-
-			usleep(100);
-		}
-
-		return(false);
+	bool TryExLock() {
+		return Lock(LOCK_EX|LOCK_NB);
 	}
 
-	bool TryShLock(unsigned long long iTimeout = 0) {
-		if (iTimeout == 0) {
-			return(Lock(LOCK_SH|LOCK_NB));
-		}
-
-		unsigned long long iNow = CUtils::GetMillTime();
-
-		while(true) {
-			if (Lock(LOCK_SH|LOCK_NB)) {
-				return true;
-			}
-
-			if ((CUtils::GetMillTime() - iNow) > iTimeout) {
-				break;
-			}
-
-			usleep(100);
-		}
-
-		return false;
+	bool TryShLock() {
+		return(Lock(LOCK_SH|LOCK_NB));
 	}
 
 	bool LockEx() { return Lock(LOCK_EX); }
@@ -176,6 +166,12 @@ private:
 		return true;
 	}
 
+	void Init() {
+		m_bCreated = false;
+		m_fd = -1;
+		m_pid = 0;
+	}
+
 	int			m_fd;
 	int			m_pid;
 	bool		m_bCreated;
@@ -226,9 +222,9 @@ protected:
 class CBlowfish {
 public:
 	/**
-	 * @sPassword key to encrypt with
-	 * @iEncrypt encrypt method (BF_DECRYPT or BF_ENCRYPT)
-	 * @sIvec what to set the ivector to start with, default sets it all 0's
+	 * @param sPassword key to encrypt with
+	 * @param iEncrypt encrypt method (BF_DECRYPT or BF_ENCRYPT)
+	 * @param sIvec what to set the ivector to start with, default sets it all 0's
 	 */
 	CBlowfish(const CString & sPassword, int iEncrypt, const CString & sIvec = "");
 	~CBlowfish();
@@ -267,7 +263,7 @@ inline bool ReadFile(const CString & sFilename, CString & sLine) {
 		return false;
 	}
 
-	while((bytes = fread(inbuff, sizeof(char), RF_BUFF, f)) > 0) {
+	while ((bytes = fread(inbuff, sizeof(char), RF_BUFF, f)) > 0) {
 		sLine.append(inbuff, bytes);
 	}
 
@@ -319,7 +315,7 @@ inline bool ReadLine(const CString & sData, CString & sLine, CString::size_type
 
 inline CString Lower(const CString & sLine) {
 	CString sRet;
-	for(u_int a = 0; a < sLine.length(); a++) {
+	for (u_int a = 0; a < sLine.length(); a++) {
 		sRet += tolower(sLine[a]);
 	}
 
@@ -328,7 +324,7 @@ inline CString Lower(const CString & sLine) {
 
 inline CString Upper(const CString & sLine) {
 	CString sRet;
-	for(u_int a = 0; a < sLine.length(); a++) {
+	for (u_int a = 0; a < sLine.length(); a++) {
 		sRet += toupper(sLine[a]);
 	}
 
@@ -431,7 +427,14 @@ private:
 /**
  * @class CSafePtr
  * @author prozac <prozac@rottenboy.com>
- * @brief This class is intended to be created on the stack and hold a pointer which will be deleted upon destruction.  It is useful for functions where you need an allocated pointer and have many return paths.  It prevents copying to get around the exclusive ownership situation which makes std::auto_ptr invalidate the first pointer on copy.  This is intended to be used in simplistic situations such as local variables.
+ * @brief Safe deletion of a pointer.
+ *
+ * This class is intended to be created on the stack and hold a pointer which
+ * will be deleted upon destruction.  It is useful for functions where you need
+ * an allocated pointer and have many return paths.  It prevents copying to get
+ * around the exclusive ownership situation which makes std::auto_ptr
+ * invalidate the first pointer on copy.  This is intended to be used in
+ * simplistic situations such as local variables.
  */
 template<typename T>
 class CSafePtr : private CNoCopy {
@@ -500,7 +503,7 @@ public:
 
 	/**
 	 * @brief Attach() to a raw pointer
-	 * @param pRawPtr The raw pointer to keep track of, ***WARNING*** Do _NOT_ allow more than one CSmartPtr keep track of the same raw pointer
+	 * @param p The raw pointer to keep track of, ***WARNING*** Do _NOT_ allow more than one CSmartPtr keep track of the same raw pointer
 	 * @return Reference to self
 	 */
 	CSmartPtr<T>& operator =(T* p) { Attach(p); return *this; }
