@@ -1,22 +1,19 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include <unistd.h>
-#include <getopt.h>
-#include <stdlib.h>
-#include <signal.h>
 #include "znc.h"
-#include "Modules.h"
-#include "MD5.h"
+#include <getopt.h>
 
 static struct option g_LongOpts[] = {
 	{ "help",			no_argument,	0,	'h' },
 	{ "version",			no_argument,	0,	'v' },
+	{ "no-color",			no_argument,	0,	'n' },
+	{ "allow-root",			no_argument,	0,	'r' },
 	{ "makeconf",			no_argument,	0,	'c' },
 	{ "makepass",			no_argument,	0,	's' },
 #ifdef HAVE_LIBSSL
@@ -27,11 +24,13 @@ static struct option g_LongOpts[] = {
 	{ 0, 0, 0, 0 }
 };
 
-void GenerateHelp(const char *appname) {
+static void GenerateHelp(const char *appname) {
 	CUtils::PrintMessage("USAGE: " + CString(appname) + " [options] [config]");
 	CUtils::PrintMessage("Options are:");
 	CUtils::PrintMessage("\t-h, --help         List available command line options (this page)");
 	CUtils::PrintMessage("\t-v, --version      Output version information and exit");
+	CUtils::PrintMessage("\t-n, --no-color     Don't use escape sequences in the output");
+	CUtils::PrintMessage("\t-r, --allow-root   Don't complain if ZNC is run as root");
 	CUtils::PrintMessage("\t-c, --makeconf     Interactively create a new config");
 	CUtils::PrintMessage("\t-s, --makepass     Generates a password for use in config");
 #ifdef HAVE_LIBSSL
@@ -41,9 +40,7 @@ void GenerateHelp(const char *appname) {
 	CUtils::PrintMessage("\t-d, --datadir      Set a different znc repository (default is ~/.znc)");
 }
 
-void die(int sig) {
-	signal(SIGSEGV, SIG_DFL);
-	signal(SIGABRT, SIG_DFL);
+static void die(int sig) {
 	signal(SIGPIPE, SIG_DFL);
 
 #ifdef _DEBUG
@@ -57,11 +54,28 @@ void die(int sig) {
 	exit(sig);
 }
 
-int main(int argc, char** argv, char** envp) {
+static void rehash(int sig) {
+	CZNC::Get().SetNeedRehash(true);
+}
+
+static bool isRoot() {
+	uid_t u_real, u_effective, u_saved;
+
+	getresuid(&u_real, &u_effective, &u_saved);
+
+	// User root? If one of these were root, we could switch the others to root, too
+	if (u_real == 0 || u_effective == 0 || u_saved == 0)
+		return true;
+
+	return false;
+}
+
+int main(int argc, char** argv) {
 	CString sConfig;
 	CString sDataDir = "";
 
 	srand(time(NULL));
+	CUtils::SetStdoutIsTTY(isatty(1));
 
 #ifdef HAVE_LIBSSL
 	InitSSL();
@@ -70,43 +84,49 @@ int main(int argc, char** argv, char** envp) {
 	int iArg, iOptIndex = -1;
 	bool bMakeConf = false;
 	bool bMakePass = false;
+	bool bAllowRoot = false;
 #ifdef HAVE_LIBSSL
 	bool bMakePem = false;
 	bool bEncPem = false;
 
-	while ((iArg = getopt_long(argc, argv, "hvcsped:", g_LongOpts, &iOptIndex)) != -1) {
-#else	
-
-	while ((iArg = getopt_long(argc, argv, "hvcsd:", g_LongOpts, &iOptIndex)) != -1) {
+	while ((iArg = getopt_long(argc, argv, "hvnrcsped:", g_LongOpts, &iOptIndex)) != -1) {
+#else
+	while ((iArg = getopt_long(argc, argv, "hvnrcsd:", g_LongOpts, &iOptIndex)) != -1) {
 #endif /* HAVE_LIBSSL */
 	    switch (iArg) {
 		case 'h':
-			    GenerateHelp(argv[0]);
-			    return 0;
+			GenerateHelp(argv[0]);
+			return 0;
 		case 'v':
-			    cout << CZNC::GetTag() << endl;
-			    return 0;
+			cout << CZNC::GetTag() << endl;
+			return 0;
+		case 'n':
+			CUtils::SetStdoutIsTTY(false);
+			break;
+		case 'r':
+			bAllowRoot = true;
+			break;
 		case 'c':
-			    bMakeConf = true;
-			    break;
+			bMakeConf = true;
+			break;
 		case 's':
-			    bMakePass = true;
-			    break;
+			bMakePass = true;
+			break;
 #ifdef HAVE_LIBSSL
 		case 'p':
-			    bMakePem = true;
-			    break;
+			bMakePem = true;
+			break;
 		case 'e':
-			    bEncPem = true;
-			    break;
+			bEncPem = true;
+			break;
 #endif /* HAVE_LIBSSL */
 		case 'd':
-			    sDataDir = CString(optarg);
-			    break;
+			sDataDir = CString(optarg);
+			break;
 		case '?':
 		default:
-			    GenerateHelp(argv[0]);
-			    return 1;
+			GenerateHelp(argv[0]);
+			return 1;
 	    }
 	}
 
@@ -120,7 +140,7 @@ int main(int argc, char** argv, char** envp) {
 		CZNC& ZNC = CZNC::Get();
 		ZNC.InitDirs("", sDataDir);
 		if (ZNC.WriteNewConfig(sConfig)) {
-			char* args[5];
+			char const* args[5];
 
 			if (argc > 2) {
 				args[0] = argv[0];
@@ -147,8 +167,8 @@ int main(int argc, char** argv, char** envp) {
 				return 1;
 			}
 
-			chdir(ZNC.GetCurPath().c_str());
-			if (execve(*argv, args, envp) == -1) {
+			if ((chdir(ZNC.GetCurPath().c_str()) == -1)
+					|| (execv(*argv, (char *const*)args) == -1)) {
 				CUtils::PrintError("Unable to launch znc [" + CString(strerror(errno)) + "]");
 				return 1;
 			}
@@ -161,12 +181,12 @@ int main(int argc, char** argv, char** envp) {
 	if (bMakePem) {
 		CZNC* pZNC = &CZNC::Get();
 		pZNC->InitDirs("", sDataDir);
-		pZNC->WritePemFile( bEncPem );
+		pZNC->WritePemFile(bEncPem);
 
 		delete pZNC;
 		return 0;
 	}
-	if( bEncPem && !bMakePem ) {
+	if (bEncPem && !bMakePem) {
 		CUtils::PrintError("--encrypt-pem should be used along with --makepem.");
 		return 1;
 	}
@@ -189,21 +209,31 @@ int main(int argc, char** argv, char** envp) {
 		return 1;
 	}
 
-	if (!pZNC->GetListeners().size()) {
-		CUtils::PrintError("You must supply at least one Listen port in your config.");
-		delete pZNC;
-		return 1;
-	}
-
 	if (!pZNC->OnBoot()) {
 		CUtils::PrintError("Exiting due to module boot errors.");
 		delete pZNC;
 		return 1;
 	}
 
+	if (isRoot()) {
+		CUtils::PrintError("You are running ZNC as root! Don't do that! There are not many valid");
+		CUtils::PrintError("reasons for this and it can, in theory, cause great damage!");
+		if (!bAllowRoot) {
+			exit(1);
+		}
+		CUtils::PrintError("You have been warned.");
+		CUtils::PrintError("Hit CTRL+C now if you don't want to run ZNC as root.");
+		CUtils::PrintError("ZNC will start in 30 seconds.");
+		sleep(30);
+	}
+
 #ifdef _DEBUG
-	CUtils::PrintMessage("Staying open for debugging");
-#else	
+	int iPid = getpid();
+	CUtils::PrintMessage("Staying open for debugging [pid: " + CString(iPid) + "]");
+
+	pZNC->WritePidFile(iPid);
+	CUtils::PrintMessage(CZNC::GetTag());
+#else
 	CUtils::PrintAction("Forking into the background");
 
 	int iPid = fork();
@@ -215,10 +245,11 @@ int main(int argc, char** argv, char** envp) {
 	}
 
 	if (iPid > 0) {
+		// We are the parent. We are done and will go to bed.
 		CUtils::PrintStatus(true, "[pid: " + CString(iPid) + "]");
 
 		pZNC->WritePidFile(iPid);
-		CUtils::PrintMessage(CZNC::GetTag(false));
+		CUtils::PrintMessage(CZNC::GetTag());
 		exit(0);
 	}
 
@@ -226,6 +257,14 @@ int main(int argc, char** argv, char** envp) {
 	close(0); open("/dev/null", O_RDONLY);
 	close(1); open("/dev/null", O_WRONLY);
 	close(2); open("/dev/null", O_WRONLY);
+
+	CUtils::SetStdoutIsTTY(false);
+
+	// We are the child. There is no way we can be a process group
+	// leader, thus setsid() must succeed.
+	setsid();
+	// Now we are in our own process group and session (no controlling
+	// terminal). We are independent!
 #endif
 
 	struct sigaction sa;
@@ -235,6 +274,12 @@ int main(int argc, char** argv, char** envp) {
 	sa.sa_handler = SIG_IGN;
 	sigaction(SIGPIPE, &sa, (struct sigaction*) NULL);
 
+	sa.sa_handler = rehash;
+	sigaction(SIGHUP,  &sa, (struct sigaction*) NULL);
+
+	// Once this signal is caught, the signal handler is reset
+	// to SIG_DFL. This avoids endless loop with signals.
+	sa.sa_flags = SA_RESETHAND;
 	sa.sa_handler = die;
 	sigaction(SIGINT,  &sa, (struct sigaction*) NULL);
 	sigaction(SIGILL,  &sa, (struct sigaction*) NULL);
