@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -10,10 +10,19 @@
 #define _MAIN_H
 
 // Keep the number in sync with configure.in (and also with configure)
-#define VERSION 0.052
+#define VERSION 0.058
+
+// You can add -DVERSION_EXTRA="stuff" to your CXXFLAGS!
+#ifndef VERSION_EXTRA
+#define VERSION_EXTRA ""
+#endif
 
 #ifndef _MODDIR_
-#define _MODDIR_ "/usr/libexec/znc"
+#define _MODDIR_ "/usr/lib/znc"
+#endif
+
+#ifndef _DATADIR_
+#define _DATADIR_ "/usr/share/znc"
 #endif
 
 #ifdef _MODULES
@@ -53,12 +62,12 @@ using std::cout;
 using std::cerr;
 using std::endl;
 
+#include <limits.h>
 #ifndef PATH_MAX
 #define PATH_MAX	1024
+#warning PATH_MAX is undefined
 #endif
 
 #include "String.h"
-#include "Csocket.h"
-#include "Utils.h"
 
 #endif // !_MAIN_H
