@@ -1,16 +1,13 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
-#include "User.h"
-#include "Nick.h"
-#include "Modules.h"
 #include "Chan.h"
+#include "Modules.h"
 
 class CChanAttach : public CModule {
 public:
@@ -31,6 +28,12 @@ public:
 			sChan = sArgs.Token(a++);
 		}
 
+		// Load our saved settings, ignore errors
+		MCString::iterator it;
+		for (it = BeginNV(); it != EndNV(); it++) {
+			Add(it->first);
+		}
+
 		return true;
 	}
 
@@ -58,7 +61,9 @@ public:
 		if (sCommand.CaseCmp("ADD") == 0) {
 			CString sChan = sLine.Token(1);
 
-			if (Add(sChan)) {
+			if (AlreadyAdded(sChan)) {
+				PutModule(sChan + " is already added");
+			} else if (Add(sChan)) {
 				PutModule("Added " + sChan + " to list");
 			} else {
 				PutModule("Usage: Add [!]<#chan>");
@@ -67,7 +72,7 @@ public:
 			CString sChan = sLine.Token(1);
 
 			if (Del(sChan))
-				PutModule("Remove " + sChan + " from list");
+				PutModule("Removed " + sChan + " from list");
 			else
 				PutModule("Usage: Del [!]<#chan>");
 		} else if (sCommand.CaseCmp("LIST") == 0) {
@@ -86,10 +91,10 @@ public:
 
 			if (Table.size()) {
 				unsigned int uTableIdx = 0;
-				CString sLine;
+				CString sTmp;
 
-				while (Table.GetLine(uTableIdx++, sLine)) {
-					PutModule(sLine);
+				while (Table.GetLine(uTableIdx++, sTmp)) {
+					PutModule(sTmp);
 				}
 			} else {
 				PutModule("You have no entries.");
@@ -113,10 +118,10 @@ public:
 
 			if (Table.size()) {
 				unsigned int uTableIdx = 0;
-				CString sLine;
+				CString sTmp;
 
-				while (Table.GetLine(uTableIdx++, sLine)) {
-					PutModule(sLine);
+				while (Table.GetLine(uTableIdx++, sTmp)) {
+					PutModule(sTmp);
 				}
 			} else {
 				PutModule("You have no entries.");
@@ -124,6 +129,25 @@ public:
 		}
 	}
 
+	bool AlreadyAdded(const CString& sInput) {
+		vector<CString>::iterator it;
+
+		if (sInput.Left(1) == "!") {
+			CString sChan = sInput.substr(1);
+			for (it = m_vsNegChans.begin(); it != m_vsNegChans.end();
+					it++) {
+				if (*it == sChan)
+					return true;
+			}
+		} else {
+			for (it = m_vsChans.begin(); it != m_vsChans.end(); it++) {
+				if (*it == sInput)
+					return true;
+			}
+		}
+		return false;
+	}
+
 	bool Add(const CString& sChan) {
 		if (sChan.empty() || sChan == "!") {
 			return false;
@@ -135,6 +159,9 @@ public:
 			m_vsChans.push_back(sChan);
 		}
 
+		// Also save it for next module load
+		SetNV(sChan, "");
+
 		return true;
 	}
 
@@ -171,6 +198,8 @@ public:
 			m_vsChans.erase(it);
 		}
 
+		DelNV(sChan);
+
 		return true;
 	}
 
