@@ -1,19 +1,18 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
-#include "User.h"
-#include "Nick.h"
-#include "Modules.h"
 #include "Chan.h"
+#include "User.h"
 
 class CAutoOpMod;
 
+#define AUTOOP_CHALLENGE_LENGTH	32
+
 class CAutoOpTimer : public CTimer {
 public:
 
@@ -158,32 +157,17 @@ public:
 		m_msUsers.clear();
 	}
 
-	virtual void OnChanPermission(const CNick& OpNick, const CNick& Nick, CChan& Channel, unsigned char uMode, bool bAdded, bool bNoChange) {
-	}
-
-	virtual void OnOp(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) {
-	}
-
-	virtual void OnDeop(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) {
-	}
-
-	virtual void OnVoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) {
-	}
-
-	virtual void OnDevoice(const CNick& OpNick, const CNick& Nick, CChan& Channel, bool bNoChange) {
-	}
-
-	virtual void OnRawMode(const CNick& OpNick, CChan& Channel, const CString& sModes, const CString& sArgs) {
-	}
-
 	virtual void OnJoin(const CNick& Nick, CChan& Channel) {
-		if (Channel.HasPerm(CChan::Op)) {																						// If we have ops in this chan
+		// If we have ops in this chan
+		if (Channel.HasPerm(CChan::Op)) {
 			for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
-				if (it->second->HostMatches(Nick.GetHostMask()) && it->second->ChannelMatches(Channel.GetName())) {				// and the nick who joined is a valid user
+				// and the nick who joined is a valid user
+				if (it->second->HostMatches(Nick.GetHostMask()) && it->second->ChannelMatches(Channel.GetName())) {
 					if (it->second->GetUserKey().CaseCmp("__NOKEY__") == 0) {
 						PutIRC("MODE " + Channel.GetName() + " +o " + Nick.GetNick());
 					} else {
-						m_msQueue[Nick.GetNick().AsLower()] = "";		// then insert this nick into the queue
+						// then insert this nick into the queue, the timer does the rest
+						m_msQueue[Nick.GetNick().AsLower()] = "";
 					}
 
 					break;
@@ -273,11 +257,11 @@ public:
 				Table.SetCell("Channels", it->second->GetChannels());
 			}
 
-			CString sLine;
+			CString sTmp;
 			unsigned int uTableIdx = 0;
 
-			while (Table.GetLine(uTableIdx++, sLine)) {
-				PutModule(sLine);
+			while (Table.GetLine(uTableIdx++, sTmp)) {
+				PutModule(sTmp);
 			}
 		} else if (sCommand.CaseCmp("ADDCHANS") == 0 || sCommand.CaseCmp("DELCHANS") == 0) {
 			CString sUser = sLine.Token(1);
@@ -352,44 +336,6 @@ public:
 		return pUser;
 	}
 
-	/* This isn't being used yet
-	bool RequestOps(const CString& sChannel) {
-		CChan* pChan = m_pUser->FindChan(sChannel);
-		return (pChan) ? RequestOps(*pChan) : false;
-	}
-
-	bool RequestOps(const CChan& Channel) {
-		if (Channel.HasPerm(CChan::Op)) {
-			return false;					// If we already have ops then don't bother
-		}
-
-		const map<CString,CNick*>& msNicks = Channel.GetNicks();
-		VCString vsNicks;
-
-		for (map<CString,CNick*>::const_iterator it = msNicks.begin(); it != msNicks.end(); it++) {
-			const CNick& Nick = *it->second;
-
-			if (Nick.HasPerm(CChan::Op)) {								// Ok, this guy is an op, now lets make sure he matches one of our defined users
-				for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
-					cerr << "??? [" << Nick.GetHostMask() << "] [" << Channel.GetName() << "]" << endl;
-					if (it->second->HostMatches(Nick.GetHostMask()) && it->second->ChannelMatches(Channel.GetName())) {
-						vsNicks.push_back(Nick.GetNick());				// Add the nick into a vector so we chan choose one at random later
-						break;											// Ok, we found a match, on to the next op in the channel
-					}
-				}
-			}
-		}
-
-		if (!vsNicks.size()) {
-			return false;
-		}
-
-		// Need to make this random in the future, just ask the first one for now
-		PutIRC("NOTICE " + vsNicks[0] + " :!ZNCAO REQUEST " + Channel.GetName());
-
-		return true;
-	}*/
-
 	bool ChallengeRespond(const CNick& Nick, const CString& sChallenge) {
 		// Validate before responding - don't blindly trust everyone
 		bool bValid = false;
@@ -434,6 +380,11 @@ public:
 			return false;
 		}
 
+		if (sChallenge.length() != AUTOOP_CHALLENGE_LENGTH) {
+			PutModule("WARNING! [" + Nick.GetHostMask() + "] sent an invalid challenge.");
+			return false;
+		}
+
 		CString sResponse = pUser->GetUserKey() + "::" + sChallenge;
 		PutIRC("NOTICE " + Nick.GetNick() + " :!ZNCAO RESPONSE " + sResponse.MD5());
 		return false;
@@ -485,7 +436,7 @@ public:
 
 		// Now issue challenges for the new users in the queue
 		for (MCString::iterator it = m_msQueue.begin(); it != m_msQueue.end(); it++) {
-			it->second = CString::RandomString(32);
+			it->second = CString::RandomString(AUTOOP_CHALLENGE_LENGTH);
 			PutIRC("NOTICE " + it->first + " :!ZNCAO CHALLENGE " + it->second);
 		}
 	}
