@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -7,12 +7,8 @@
  */
 
 // @todo handle raw 433 (nick in use)
-#include "main.h"
-#include "User.h"
-#include "Nick.h"
-#include "Modules.h"
-#include "Chan.h"
 #include "IRCSock.h"
+#include "User.h"
 
 class CAwayNickMod;
 
@@ -63,6 +59,12 @@ public:
 
 		SetNV("nick", m_sFormat);
 
+		if (m_pUser->GetKeepNick()) {
+			sMessage = "You have KeepNick enabled. "
+				"This won't work together with awaynick.";
+			return false;
+		}
+
 		return true;
 	}
 
@@ -99,6 +101,11 @@ public:
 	}
 
 	virtual void OnUserAttached() {
+		if (m_pUser->GetKeepNick()) {
+			PutModule("WARNING: You have KeepNick enabled. "
+					"This won't work with awaynick.");
+		}
+
 		StartBackNickTimer();
 	}
 
@@ -140,7 +147,7 @@ public:
 
 				PutModule(sMsg);
 			}
-		} else if(sCommand.Token(0).CaseCmp("HELP") == 0) {
+		} else if (sCommand.Token(0).CaseCmp("HELP") == 0) {
 			PutModule("Commands are: show, timers, set [awaynick]");
 		}
 	}
