@@ -1,22 +1,34 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
-#include "znc.h"
-#include "User.h"
-#include "Nick.h"
-#include "Modules.h"
 #include "Chan.h"
+#include "User.h"
+#include "znc.h"
 
 class CChanSaverMod : public CModule {
 public:
 	MODCONSTRUCTOR(CChanSaverMod) {
+		const vector<CChan*>& vChans = m_pUser->GetChans();
+		vector<CChan*>::const_iterator it = vChans.begin();
+		vector<CChan*>::const_iterator end = vChans.end();
+
 		m_bWriteConf = false;
+
+		for (; it != end; it++) {
+			CChan *pChan = *it;
+
+			// If that channel isn't yet in the config,
+			// we'll have to add it...
+			if (!pChan->InConfig()) {
+				pChan->SetInConfig(true);
+				m_bWriteConf = true;
+			}
+		}
 	}
 
 	virtual ~CChanSaverMod() {
