@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -7,27 +7,23 @@
  */
 
 //! @author prozac@rottenboy.com
-// 
+//
 // The encryption here was designed to be compatible with mircryption's CBC mode.
-// 
+//
 // TODO:
-// 
+//
 // 1) Encrypt key storage file
 // 2) Secure key exchange using pub/priv keys and the DH algorithm
 // 3) Some way of notifying the user that the current channel is in "encryption mode" verses plain text
 // 4) Temporarily disable a target (nick/chan)
-// 
+//
 // NOTE: This module is currently NOT intended to secure you from your shell admin.
 //       The keys are currently stored in plain text, so anyone with access to your account (or root) can obtain them.
 //       It is strongly suggested that you enable SSL between znc and your client otherwise the encryption stops at znc and gets sent to your client in plain text.
-// 
+//
 
-#include "main.h"
-#include "User.h"
-#include "Nick.h"
-#include "Modules.h"
 #include "Chan.h"
-#include "String.h"
+#include "User.h"
 
 #define REQUIRESSL	1
 
@@ -48,8 +44,9 @@ public:
 
 		if (it != EndNV()) {
 			CChan* pChan = m_pUser->FindChan(sTarget);
-			if ((pChan) && (pChan->KeepBuffer())) {
-				pChan->AddBuffer(":\244" + m_pUser->GetIRCNick().GetNickMask() + " PRIVMSG " + sTarget + " :" + sMessage);
+			if (pChan) {
+				if (pChan->KeepBuffer())
+					pChan->AddBuffer(":\244" + m_pUser->GetIRCNick().GetNickMask() + " PRIVMSG " + sTarget + " :" + sMessage);
 				m_pUser->PutUser(":\244" + m_pUser->GetIRCNick().GetNickMask() + " PRIVMSG " + sTarget + " :" + sMessage, NULL, m_pClient);
 			}
 
