@@ -1,20 +1,13 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
-#include "User.h"
-#include "Nick.h"
 #include "Modules.h"
-#include "Chan.h"
 #include "znc.h"
-#include "HTTPSock.h"
-#include "Server.h"
-#include <map>
 
 using std::map;
 
