@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -12,10 +12,9 @@
  * Usage: LoadModule = rejoin [delay]
  *
  */
-#include <main.h>
-#include <Modules.h>
-#include <Chan.h>
-#include <User.h>
+
+#include "Chan.h"
+#include "User.h"
 
 class CRejoinJob: public CTimer {
 public:
