@@ -1,20 +1,12 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
 #include "User.h"
-#include "Nick.h"
-#include "Modules.h"
-#include "Chan.h"
-#include "Utils.h"
-#include <pwd.h>
-#include <map>
-#include <vector>
 
 class CNickServ : public CModule
 {
@@ -22,7 +14,7 @@ public:
 	MODCONSTRUCTOR(CNickServ)
 	{
 	}
-	
+
 	virtual ~CNickServ()
 	{
 	}
@@ -33,23 +25,23 @@ public:
 			m_sPass = GetNV("Password");
 		else
 			m_sPass = sArgs;
-		
+
 		return true;
 	}
 
-	virtual void OnModCommand( const CString& sCommand )
+	virtual void OnModCommand(const CString& sCommand)
 	{
 		CString sCmdName = sCommand.Token(0).AsLower();
-		if(sCmdName == "set") {
+		if (sCmdName == "set") {
 			CString sPass = sCommand.Token(1, true);
 			m_sPass = sPass;
 			PutModule("Password set");
-		} else if(sCmdName == "show") {
+		} else if (sCmdName == "show") {
 			if (m_sPass.empty())
 				PutModule("No password set");
 			else
 				PutModule("Current password: " + m_sPass);
-		} else if(sCmdName == "save") {
+		} else if (sCmdName == "save") {
 			SetNV("Password", m_sPass);
 			PutModule("Saved!");
 		} else {
@@ -62,12 +54,12 @@ public:
 		if (!m_sPass.empty()
 				&& Nick.GetNick().CaseCmp("NickServ") == 0
 				&& sMessage.find("msg") != CString::npos
-				&& sMessage.find("IDENTIFY") != CString::npos
+				&& sMessage.AsUpper().find("IDENTIFY") != CString::npos
 				&& sMessage.find("help") == CString::npos) {
 			PutIRC("PRIVMSG NickServ :IDENTIFY " + m_sPass);
 		}
 	}
-	
+
 	virtual EModRet OnPrivMsg(CNick& Nick, CString& sMessage)
 	{
 		HandleMessage(Nick, sMessage);
