@@ -1,19 +1,13 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
 #include "User.h"
-#include "Nick.h"
-#include "Modules.h"
-#include "Chan.h"
 #include "znc.h"
-#include "HTTPSock.h"
-#include "Server.h"
 
 class CPartylineChannel {
 public:
@@ -47,13 +41,16 @@ public:
 	GLOBALMODCONSTRUCTOR(CPartylineMod) {}
 
 	virtual ~CPartylineMod() {
-		while(m_ssChannels.size()) {
+		while (m_ssChannels.size()) {
 			delete *m_ssChannels.begin();
 			m_ssChannels.erase(m_ssChannels.begin());
 		}
 	}
 
 	virtual bool OnBoot() {
+		// The config is now read completely, so all Users are set up
+		Load();
+
 		return true;
 	}
 
@@ -82,25 +79,21 @@ public:
 
 		return true;
 	}
-	
-	void OnFinishedConfig() {
-		Load();
-	}
 
-	void Load() {		
+	void Load() {
 		VCString vsChannels;
 		for (MCString::iterator it = BeginNV(); it != EndNV(); it++) {
 			CUser* pUser = CZNC::Get().FindUser(it->first);
 			CPartylineChannel* pChannel;
 			it->second.Split(",", vsChannels, false);
 
-			if(!pUser) {
+			if (!pUser) {
 				// TODO: give some usefull message?
 				continue;
 			}
 
-			for(VCString::iterator i = vsChannels.begin(); i != vsChannels.end(); i++) {
-				if(i->Trim_n().empty())
+			for (VCString::iterator i = vsChannels.begin(); i != vsChannels.end(); i++) {
+				if (i->Trim_n().empty())
 					continue;
 				pChannel = GetChannel(*i);
 				JoinUser(pUser, pChannel);
@@ -115,7 +108,7 @@ public:
 		CString sChans;
 		const CString &sUser = pUser->GetUserName();
 
-		for(set<CPartylineChannel*>::iterator it = m_ssChannels.begin();
+		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin();
 				it != m_ssChannels.end(); it++) {
 			if ((*it)->IsFixedChan(sUser)) {
 				sChans += "," + (*it)->GetName();
@@ -395,18 +388,41 @@ public:
 			Table.AddColumn("Arguments");
 			Table.AddColumn("Description");
 
-			Table.AddRow(); Table.SetCell("Command", "Help"); Table.SetCell("Arguments", ""); Table.SetCell("Description", "List all partyline commands");
-			Table.AddRow(); Table.SetCell("Command", "List"); Table.SetCell("Arguments", ""); Table.SetCell("Description", "List all open channels");
-			Table.AddRow(); Table.SetCell("Command", "AddFixChan"); Table.SetCell("Arguments", "<user> <channel>"); Table.SetCell("Description", "Force a user into a channel which he cant part");
-			Table.AddRow(); Table.SetCell("Command", "DelFixChan"); Table.SetCell("Arguments", "<user> <channel>"); Table.SetCell("Description", "Remove a user from such a channel");
-			Table.AddRow(); Table.SetCell("Command", "ListFixChans"); Table.SetCell("Arguments", "<user>"); Table.SetCell("Description", "Show which channels a user can not part");
-			Table.AddRow(); Table.SetCell("Command", "ListFixUsers"); Table.SetCell("Arguments", "<channel>"); Table.SetCell("Description", "Show which users can not part this channel");
+			Table.AddRow();
+			Table.SetCell("Command", "Help");
+			Table.SetCell("Arguments", "");
+			Table.SetCell("Description", "List all partyline commands");
+
+			Table.AddRow();
+			Table.SetCell("Command", "List");
+			Table.SetCell("Arguments", "");
+			Table.SetCell("Description", "List all open channels");
+
+			Table.AddRow();
+			Table.SetCell("Command", "AddFixChan");
+			Table.SetCell("Arguments", "<user> <channel>");
+			Table.SetCell("Description", "Force a user into a channel which he cant part");
+
+			Table.AddRow();
+			Table.SetCell("Command", "DelFixChan");
+			Table.SetCell("Arguments", "<user> <channel>");
+			Table.SetCell("Description", "Remove a user from such a channel");
+
+			Table.AddRow();
+			Table.SetCell("Command", "ListFixChans");
+			Table.SetCell("Arguments", "<user>");
+			Table.SetCell("Description", "Show which channels a user can not part");
+
+			Table.AddRow();
+			Table.SetCell("Command", "ListFixUsers");
+			Table.SetCell("Arguments", "<channel>");
+			Table.SetCell("Description", "Show which users can not part this channel");
 
 			unsigned int uTableIdx = 0;
-			CString sLine;
+			CString sTmp;
 
-			while (Table.GetLine(uTableIdx++, sLine)) {
-				PutModule(sLine);
+			while (Table.GetLine(uTableIdx++, sTmp)) {
+				PutModule(sTmp);
 			}
 		} else if (sCommand.CaseCmp("LIST") == 0) {
 			if (!m_ssChannels.size()) {
@@ -427,13 +443,13 @@ public:
 			}
 
 			unsigned int uTableIdx = 0;
-			CString sLine;
+			CString sTmp;
 
-			while (Table.GetLine(uTableIdx++, sLine)) {
-				PutModule(sLine);
+			while (Table.GetLine(uTableIdx++, sTmp)) {
+				PutModule(sTmp);
 			}
 		} else if (sCommand.CaseCmp("ADDFIXCHAN") == 0) {
-			if(!m_pUser->IsAdmin()) {
+			if (!m_pUser->IsAdmin()) {
 				PutModule("Access denied");
 				return;
 			}
@@ -442,12 +458,12 @@ public:
 			CUser* pUser = CZNC::Get().FindUser(sUser);
 			CPartylineChannel* pChan;
 
-			if(sChan.Left(2) != "~#") {
+			if (sChan.Left(2) != "~#") {
 				PutModule("Invalid channel name");
 				return;
 			}
 
-			if(pUser == NULL) {
+			if (pUser == NULL) {
 				PutModule("Unknown User '" + sUser + "'");
 				return;
 			}
@@ -460,7 +476,7 @@ public:
 
 			PutModule("Fixed " + sUser + " to channel " + sChan);
 		} else if (sCommand.CaseCmp("DELFIXCHAN") == 0) {
-			if(!m_pUser->IsAdmin()) {
+			if (!m_pUser->IsAdmin()) {
 				PutModule("Access denied");
 				return;
 			}
@@ -469,12 +485,12 @@ public:
 			CUser* pUser = CZNC::Get().FindUser(sUser);
 			CPartylineChannel* pChan = FindChannel(sChan);
 
-			if(pUser == NULL) {
+			if (pUser == NULL) {
 				PutModule("Unknown User '" + sUser + "'");
 				return;
 			}
 
-			if(!pChan || !pChan->IsFixedChan(sUser)) {
+			if (!pChan || !pChan->IsFixedChan(sUser)) {
 				PutModule(sUser + " is not in " + sChan + " or isnt fixed to it");
 				return;
 			}
@@ -485,38 +501,38 @@ public:
 
 			PutModule("Removed " + sUser + " from " + sChan);
 		} else if (sCommand.CaseCmp("LISTFIXCHANS") == 0) {
-			if(!m_pUser->IsAdmin()) {
+			if (!m_pUser->IsAdmin()) {
 				PutModule("Access denied");
 				return;
 			}
 			CString sUser = sLine.Token(1);
 			CUser* pUser = CZNC::Get().FindUser(sUser);
-			if(!pUser) {
+			if (!pUser) {
 				PutModule("User not found!");
 				return;
 			}
 
 			for (set<CPartylineChannel*>::const_iterator a = m_ssChannels.begin(); a != m_ssChannels.end(); a++) {
-				if((*a)->IsFixedChan(sUser)) {
+				if ((*a)->IsFixedChan(sUser)) {
 					PutModule((*a)->GetName());
 				}
 			}
-			PutModule("--- End of list");				
+			PutModule("--- End of list");
 		} else if (sCommand.CaseCmp("LISTFIXUSERS") == 0) {
-			if(!m_pUser->IsAdmin()) {
+			if (!m_pUser->IsAdmin()) {
 				PutModule("Access denied");
 				return;
 			}
 			CString sChan = sLine.Token(1).Left(32);
 			CPartylineChannel* pChan = FindChannel(sChan);
 
-			if(!pChan) { 
+			if (!pChan) {
 				PutModule("Channel does not exist!");
 				return;
 			}
 			const set<CString>& sNicks = pChan->GetNicks();
-			for(set<CString>::const_iterator it = sNicks.begin(); it != sNicks.end(); it++) {
-				if(pChan->IsFixedChan(*it)) {
+			for (set<CString>::const_iterator it = sNicks.begin(); it != sNicks.end(); it++) {
+				if (pChan->IsFixedChan(*it)) {
 					PutModule(*it);
 				}
 			}
@@ -540,9 +556,9 @@ public:
 	void PutChan(const set<CString>& ssNicks, const CString& sLine, bool bIncludeCurUser = true, bool bIncludeClient = true, CUser* pUser = NULL, CClient* pClient = NULL) {
 		const map<CString, CUser*>& msUsers = CZNC::Get().GetUserMap();
 
-		if(!pUser)
+		if (!pUser)
 			pUser = m_pUser;
-		if(!pClient)
+		if (!pClient)
 			pClient = m_pClient;
 
 		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
@@ -561,10 +577,10 @@ public:
 	void SendNickList(CUser* pUser, const set<CString>& ssNicks, const CString& sChan) {
 		CString sNickList;
 		for (set<CString>::iterator it = ssNicks.begin(); it != ssNicks.end(); it++) {
-			CUser* pUser = CZNC::Get().FindUser(*it);
+			CUser* pChanUser = CZNC::Get().FindUser(*it);
 
-			if (pUser && pUser->IsUserAttached()) {
-				sNickList += (pUser->IsAdmin()) ? "@" : "+";
+			if (pChanUser && pChanUser->IsUserAttached()) {
+				sNickList += (pChanUser->IsAdmin()) ? "@" : "+";
 			}
 
 			sNickList += "?" + (*it) + " ";
@@ -585,8 +601,8 @@ public:
 	CPartylineChannel* FindChannel(const CString& sChan) {
 		CString sChannel = sChan.AsLower();
 
-		for(set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
-			if((*it)->GetName().AsLower() == sChannel)
+		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
+			if ((*it)->GetName().AsLower() == sChannel)
 				return *it;
 		}
 
@@ -596,7 +612,7 @@ public:
 	CPartylineChannel* GetChannel(const CString& sChannel) {
 		CPartylineChannel* pChannel = FindChannel(sChannel);
 
-		if(pChannel == NULL) {
+		if (pChannel == NULL) {
 			pChannel = new CPartylineChannel(sChannel.AsLower());
 			m_ssChannels.insert(pChannel);
 		}
