@@ -1,16 +1,14 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
-#include "User.h"
-#include "Nick.h"
-#include "Modules.h"
 #include "Chan.h"
+#include "User.h"
+#include "Modules.h"
 
 class CSampleTimer : public CTimer {
 public:
@@ -168,9 +166,21 @@ public:
 		return CONTINUE;
 	}
 
+	virtual EModRet OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) {
+		PutModule("* " + Nick.GetNick() + " changes topic on " + Channel.GetName() + " to '" + sTopic + "'");
+
+		return CONTINUE;
+	}
+
+	virtual EModRet OnUserTopic(CString& sTarget, CString& sTopic) {
+		PutModule("* " + m_pUser->GetCurNick() + " changed topic on " + sTarget + " to '" + sTopic + "'");
+
+		return CONTINUE;
+	}
+
 	virtual EModRet OnUserMsg(CString& sTarget, CString& sMessage) {
 		PutModule("[" + sTarget + "] usermsg [" + sMessage + "]");
-		sMessage = "\0034" + sMessage + "\003";
+		sMessage = "Sample: \0034" + sMessage + "\003";
 
 		return CONTINUE;
 	}
