@@ -1,32 +1,27 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#include "main.h"
-#include "User.h"
-#include "Nick.h"
-#include "Modules.h"
 #include "Chan.h"
-#include "znc.h"
 #include "HTTPSock.h"
-#include "Client.h"
 #include "Server.h"
 #include "Template.h"
+#include "User.h"
+#include "znc.h"
+#include <sstream>
+
+using std::stringstream;
 
 class CWebAdminMod;
 class CWebAdminSock;
 
-
 class CWebAdminAuth : public CAuthBase {
 public:
-	CWebAdminAuth(CWebAdminSock* pWebAdminSock, const CString& sUsername, const CString& sPassword)
-		: CAuthBase(sUsername, sPassword) {
-			m_pWebAdminSock = pWebAdminSock;
-	}
+	CWebAdminAuth(CWebAdminSock* pWebAdminSock, const CString& sUsername, const CString& sPassword);
 
 	virtual ~CWebAdminAuth() {}
 
@@ -85,8 +80,17 @@ public:
 	}
 
 	// Setters
-	void SetSessionUser(CUser* p) { m_pSessionUser = p; m_bAdmin = p->IsAdmin(); }
-	void SetAdmin(bool b) { m_bAdmin = b; }
+	void SetSessionUser(CUser* p) {
+		m_pSessionUser = p;
+		m_bAdmin = p->IsAdmin();
+
+		// If m_pUser is not NULL, only that user can be edited.
+		if (m_bAdmin) {
+			m_pUser = NULL;
+		} else {
+			m_pUser = m_pSessionUser;
+		}
+	}
 	// !Setters
 
 	virtual Csock* GetSockObj(const CString& sHost, unsigned short uPort);
@@ -104,7 +108,7 @@ protected:
 
 class CWebAdminMod : public CGlobalModule {
 public:
-	CWebAdminMod(void *pDLL, const CString& sModName) : CGlobalModule(pDLL, sModName) {
+	GLOBALMODCONSTRUCTOR(CWebAdminMod) {
 		m_uPort = 8080;
 		m_sSkinName = GetNV("SkinName");
 	}
@@ -219,25 +223,16 @@ private:
 	CString						m_sListenHost;
 	map<CString, unsigned int>	m_suSwitchCounters;
 };
- 
+
 CString CWebAdminSock::GetSkinDir() {
-	CString sModPath = CZNC::Get().FindModPath(m_pModule->GetModName());	// @todo store the path to the module at load time and store it in a member var which can be used here
+	CString sSkinDir = m_pModule->GetModDataDir() + "/skins/"
+		+ m_pModule->GetSkinName() + "/";
 
-	// Remove the file.so portion from the path
-	while (!sModPath.empty() && sModPath.Right(1) != "/") {
-		sModPath.RightChomp();
-	}
-
-	// Remove the last /
-	sModPath.RightChomp();
-
-	CString sSkinDir = sModPath + "/" + m_pModule->GetModName() + "/skins/" + m_pModule->GetSkinName() + "/";
-
-	if (CDir::Exists(sSkinDir)) {
+	if (CFile::IsDir(sSkinDir)) {
 		return sSkinDir;
 	}
 
-	return sModPath + "/" + m_pModule->GetModName() + "/skins/default/";
+	return m_pModule->GetModDataDir() + "/skins/default/";
 }
 
 void CWebAdminSock::PrintPage(CString& sPageRet, const CString& sTmplName) {
@@ -269,35 +264,13 @@ void CWebAdminSock::PrintPage(CString& sPageRet, const CString& sTmplName) {
 bool CWebAdminSock::OnLogin(const CString& sUser, const CString& sPass) {
 	m_spAuth = new CWebAdminAuth(this, sUser, sPass);
 
-	if (CZNC::Get().GetModules().OnLoginAttempt(m_spAuth)) {
-		if (IsLoggedIn()) {
-			return true;
-		}
+	// Some authentication module could need some time, block this socket
+	// until then. CWebAdminAuth will UnPauseRead().
+	PauseRead();
+	CZNC::Get().AuthUser(m_spAuth);
 
-		PauseRead();
-
-		return false;
-	}
-
-	CUser* pUser = CZNC::Get().FindUser(GetUser());
-
-	if (pUser) {
-		CString sHost = GetRemoteIP();
-
-		if (pUser->IsHostAllowed(sHost) && pUser->CheckPass(GetPass())) {
-			if (pUser->IsAdmin()) {
-				m_bAdmin = true;
-			} else {
-				m_pUser = pUser;
-			}
-
-			m_pSessionUser = pUser;
-
-			return true;
-		}
-	}
-
-	return false;
+	// If CWebAdminAuth already set this, don't change it.
+	return IsLoggedIn();
 }
 
 void CWebAdminSock::ListUsersPage(CString& sPageRet) {
@@ -567,7 +540,7 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 
 		CString sDir(GetSkinDir() + "/..");
 
-		if (CDir::Exists(sDir)) {
+		if (CFile::IsDir(sDir)) {
 			CDir Dir(sDir);
 
 			m_Template.AddRow("SkinLoop")["Name"] = "default";
@@ -575,7 +548,9 @@ bool CWebAdminSock::SettingsPage(CString& sPageRet) {
 			for (unsigned int d = 0; d < Dir.size(); d++) {
 				const CFile& SubDir = *Dir[d];
 
-				if (SubDir.IsDir() && SubDir.GetShortName() != "CVS" && SubDir.GetShortName() != "default") {
+				if (SubDir.IsDir() &&
+						SubDir.GetShortName() != ".svn" &&
+						SubDir.GetShortName() != "default") {
 					CTemplate& l = m_Template.AddRow("SkinLoop");
 					l["Name"] = SubDir.GetShortName();
 
@@ -705,20 +680,20 @@ bool CWebAdminSock::ChanPage(CString& sPageRet, CChan* pChan) {
 		} else {
 			m_Template["Action"] = "addchan";
 			m_Template["Title"] = "Add Channel" + CString(" for User [" + m_pUser->GetUserName() + "]");
-			m_Template["BufferCount"] = "50";
-			m_Template["DefModes"] = "+stn";
+			m_Template["BufferCount"] = CString(m_pUser->GetBufferCount());
+			m_Template["DefModes"] = CString(m_pUser->GetDefaultChanModes());
 			m_Template["InConfig"] = "true";
 		}
 
 		CTemplate& o1 = m_Template.AddRow("OptionLoop");
 		o1["Name"] = "autocycle";
 		o1["DisplayName"] = "Auto Cycle";
-		if (!pChan || pChan->AutoCycle()) { o1["Checked"] = "true"; }
+		if ((pChan && pChan->AutoCycle()) || (!pChan && m_pUser->AutoCycle())) { o1["Checked"] = "true"; }
 
 		CTemplate& o2 = m_Template.AddRow("OptionLoop");
 		o2["Name"] = "keepbuffer";
 		o2["DisplayName"] = "Keep Buffer";
-		if (!pChan || pChan->KeepBuffer()) { o2["Checked"] = "true"; }
+		if ((pChan && pChan->KeepBuffer()) || (!pChan && m_pUser->KeepBuffer())) { o2["Checked"] = "true"; }
 
 		CTemplate& o3 = m_Template.AddRow("OptionLoop");
 		o3["Name"] = "detached";
@@ -816,6 +791,7 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 			m_Template["DefaultChanModes"] = pUser->GetDefaultChanModes();
 			m_Template["BufferCount"] = CString(pUser->GetBufferCount());
 			m_Template["TimestampFormat"] = pUser->GetTimestampFormat();
+			m_Template["TimezoneOffset"] = CString(pUser->GetTimezoneOffset());
 
 			const set<CString>& ssAllowedHosts = pUser->GetAllowedHosts();
 			for (set<CString>::const_iterator it = ssAllowedHosts.begin(); it != ssAllowedHosts.end(); it++) {
@@ -866,14 +842,27 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 			m_Template["StatusPrefix"] = "*";
 		}
 
+		// To change VHosts be admin or don't have DenySetVHost
 		const VCString& vsVHosts = CZNC::Get().GetVHosts();
-		for (unsigned int b = 0; b < vsVHosts.size(); b++) {
-			const CString& sVHost = vsVHosts[b];
-			CTemplate& l = m_Template.AddRow("VHostLoop");
+		bool bFoundVHost = false;
+		if (IsAdmin() || !m_pSessionUser->DenySetVHost()) {
+			for (unsigned int b = 0; b < vsVHosts.size(); b++) {
+				const CString& sVHost = vsVHosts[b];
+				CTemplate& l = m_Template.AddRow("VHostLoop");
 
-			l["VHost"] = sVHost;
+				l["VHost"] = sVHost;
 
-			if (pUser && pUser->GetVHost() == sVHost) {
+				if (pUser && pUser->GetVHost() == sVHost) {
+					l["Checked"] = "true";
+					bFoundVHost = true;
+				}
+			}
+
+			// If our current vhost is not in the global list...
+			if (pUser && !bFoundVHost && !pUser->GetVHost().empty()) {
+				CTemplate& l = m_Template.AddRow("VHostLoop");
+
+				l["VHost"] = pUser->GetVHost();
 				l["Checked"] = "true";
 			}
 		}
@@ -949,12 +938,19 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 			o10["DisplayName"] = "Admin";
 			if (pUser && pUser->IsAdmin()) { o10["Checked"] = "true"; }
 			if (pUser && pUser == CZNC::Get().FindUser(GetUser())) { o10["Disabled"] = "true"; }
+
+			CTemplate& o11 = m_Template.AddRow("OptionLoop");
+			o11["Name"] = "denysetvhost";
+			o11["DisplayName"] = "Deny SetVHost";
+			if (pUser && pUser->DenySetVHost()) { o11["Checked"] = "true"; }
 		}
 
 		PrintPage(sPageRet, "UserPage.tmpl");
 		return true;
 	}
 
+	/* If pUser is NULL, we are adding a user, else we are editing this one */
+
 	CString sUsername = GetParam("user");
 	if (!pUser && CZNC::Get().FindUser(sUsername)) {
 		GetErrorPage(sPageRet, "Invalid Submission [User " + sUsername + " already exists]");
@@ -982,7 +978,7 @@ bool CWebAdminSock::UserPage(CString& sPageRet, CUser* pUser) {
 		}
 	} else {
 		// Edit User Submission
-		if (!pUser->Clone(*pNewUser, sErr)) {
+		if (!pUser->Clone(*pNewUser, sErr, false)) {
 			delete pNewUser;
 			GetErrorPage(sPageRet, "Invalid Submission [" + sErr + "]");
 			return true;
@@ -1016,6 +1012,13 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 		return NULL;
 	}
 
+	if (pUser) {
+		/* If we are editing a user we must not change the user name */
+		sUsername = pUser->GetUserName();
+	}
+
+	CUser* pNewUser = new CUser(sUsername);
+
 	CString sArg = GetParam("password");
 
 	if (sArg != GetParam("password2")) {
@@ -1023,8 +1026,6 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 		return NULL;
 	}
 
-	CUser* pNewUser = new CUser(sUsername);
-
 	if (!sArg.empty()) {
 		pNewUser->SetPass(sArg.MD5(), true);
 	}
@@ -1098,11 +1099,19 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 	sArg = GetParam("statusprefix"); if (!sArg.empty()) { pNewUser->SetStatusPrefix(sArg); }
 	sArg = GetParam("ident"); if (!sArg.empty()) { pNewUser->SetIdent(sArg); }
 	sArg = GetParam("realname"); if (!sArg.empty()) { pNewUser->SetRealName(sArg); }
-	sArg = GetParam("vhost"); if (!sArg.empty()) { pNewUser->SetVHost(sArg); }
 	sArg = GetParam("quitmsg"); if (!sArg.empty()) { pNewUser->SetQuitMsg(sArg); }
 	sArg = GetParam("chanmodes"); if (!sArg.empty()) { pNewUser->SetDefaultChanModes(sArg); }
 	sArg = GetParam("timestampformat"); if (!sArg.empty()) { pNewUser->SetTimestampFormat(sArg); }
 
+	sArg = GetParam("vhost");
+	// To change VHosts be admin or don't have DenySetVHost
+	if (IsAdmin() || !m_pSessionUser->DenySetVHost()) {
+		if (!sArg.empty())
+			pNewUser->SetVHost(sArg);
+	} else if (pUser){
+		pNewUser->SetVHost(pUser->GetVHost());
+	}
+
 	pNewUser->SetBufferCount(GetParam("bufsize").ToUInt());
 	pNewUser->SetKeepBuffer(GetParam("keepbuffer").ToBool());
 	pNewUser->SetMultiClients(GetParam("multiclients").ToBool());
@@ -1112,11 +1121,14 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 	pNewUser->SetUseClientIP(GetParam("useclientip").ToBool());
 	pNewUser->SetTimestampAppend(GetParam("appendtimestamp").ToBool());
 	pNewUser->SetTimestampPrepend(GetParam("prependtimestamp").ToBool());
+	pNewUser->SetTimezoneOffset(GetParam("timezoneoffset").ToDouble());
 
 	if (IsAdmin()) {
 		pNewUser->SetDenyLoadMod(GetParam("denyloadmod").ToBool());
+		pNewUser->SetDenySetVHost(GetParam("denysetvhost").ToBool());
 	} else if (pUser) {
 		pNewUser->SetDenyLoadMod(pUser->DenyLoadMod());
+		pNewUser->SetDenySetVHost(pUser->DenySetVHost());
 	}
 
 	if (pUser && pUser != CZNC::Get().FindUser(GetUser())) {
@@ -1134,6 +1146,12 @@ CUser* CWebAdminSock::GetNewUser(CString& sPageRet, CUser* pUser) {
 	return pNewUser;
 }
 
+CWebAdminAuth::CWebAdminAuth(CWebAdminSock* pWebAdminSock, const CString& sUsername,
+		const CString& sPassword)
+		: CAuthBase(sUsername, sPassword, pWebAdminSock->GetRemoteIP()) {
+	m_pWebAdminSock = pWebAdminSock;
+}
+
 void CWebAdminAuth::AcceptLogin(CUser& User) {
 	if (m_pWebAdminSock) {
 		m_pWebAdminSock->SetSessionUser(&User);
@@ -1144,6 +1162,7 @@ void CWebAdminAuth::AcceptLogin(CUser& User) {
 
 void CWebAdminAuth::RefuseLogin(const CString& sReason) {
 	if (m_pWebAdminSock) {
+		m_pWebAdminSock->SetLoggedIn(false);
 		m_pWebAdminSock->UnPauseRead();
 	}
 }
