@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -7,19 +7,23 @@
  */
 
 #include "znc.h"
-#include "User.h"
-#include "Server.h"
-#include "Client.h"
+#include "Chan.h"
 #include "IRCSock.h"
-#include "Utils.h"
-
+#include "Server.h"
+#include "User.h"
 #include <pwd.h>
-#include <signal.h>
-#include <sys/types.h>
+#include <list>
 
-#ifdef _MODULES
-#include "Modules.h"
-#endif
+namespace
+{ // private namespace for local things
+	struct CGlobalModuleConfigLine
+	{
+		CString	m_sName;
+		CString	m_sValue;
+		CUser	*m_pUser;
+		CChan	*m_pChan;
+	};
+};
 
 CZNC::CZNC() {
 #ifdef _MODULES
@@ -31,10 +35,12 @@ CZNC::CZNC() {
 	m_uBytesRead = 0;
 	m_uBytesWritten = 0;
 	m_pConnectUserTimer = NULL;
+	m_bNeedRehash = false;
+	m_TimeStarted = time(NULL);
 }
 
 CZNC::~CZNC() {
-	if(m_pISpoofLockFile)
+	if (m_pISpoofLockFile)
 		ReleaseISpoof();
 
 #ifdef _MODULES
@@ -53,6 +59,7 @@ CZNC::~CZNC() {
 		a->second->SetBeingDeleted(true);
 	}
 
+	m_pConnectUserTimer = NULL;
 	// This deletes m_pConnectUserTimer
 	m_Manager.Cleanup();
 	DeleteUsers();
@@ -60,44 +67,66 @@ CZNC::~CZNC() {
 #ifdef _MODULES
 	delete m_pModules;
 #endif
+
+	DeletePidFile();
+}
+
+CString CZNC::GetVersion() {
+	char szBuf[128];
+
+	snprintf(szBuf, sizeof(szBuf), "%1.3f"VERSION_EXTRA, VERSION);
+	// If snprintf overflows (which I doubt), we want to be on the safe side
+	szBuf[sizeof(szBuf) - 1] = '\0';
+
+	return szBuf;
 }
 
 CString CZNC::GetTag(bool bIncludeVersion) {
 	if (!bIncludeVersion) {
-		return "ZNC by prozac - http://znc.sourceforge.net";
+		return "ZNC - http://znc.sourceforge.net";
 	}
 
 	char szBuf[128];
-	memset(szBuf, 0, 128);
-	snprintf(szBuf, 127, "ZNC %1.3f by prozac - http://znc.sourceforge.net", VERSION);
+	snprintf(szBuf, sizeof(szBuf), "ZNC %1.3f"VERSION_EXTRA" - http://znc.sourceforge.net", VERSION);
+	// If snprintf overflows (which I doubt), we want to be on the safe side
+	szBuf[sizeof(szBuf) - 1] = '\0';
 
 	return szBuf;
 }
 
+CString CZNC::GetUptime() {
+	time_t now = time(NULL);
+	return CString::ToTimeStr(now - TimeStarted());
+}
+
 bool CZNC::OnBoot() {
 #ifdef _MODULES
 	if (!GetModules().OnBoot()) {
 		return false;
 	}
-#endif
 
 	for (map<CString,CUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
-		if (!it->second->OnBoot()) {
+		if (!it->second->GetModules().OnBoot()) {
 			return false;
 		}
 	}
+#endif
 
 	return true;
 }
 
 bool CZNC::ConnectUser(CUser *pUser) {
 	CString sSockName = "IRC::" + pUser->GetUserName();
+	// Don't use pUser->GetIRCSock(), as that only returns something if the
+	// CIRCSock is already connected, not when it's still connecting!
 	CIRCSock* pIRCSock = (CIRCSock*) m_Manager.FindSockByName(sSockName);
 
 	if (m_pISpoofLockFile != NULL) {
 		return false;
 	}
 
+	if (!pUser->GetIRCConnectEnabled())
+		return false;
 
 	if (pIRCSock || !pUser->HasServers())
 		return false;
@@ -129,7 +158,7 @@ bool CZNC::ConnectUser(CUser *pUser) {
 	}
 #endif
 
-	if (!m_Manager.Connect(pServer->GetName(), pServer->GetPort(), sSockName, 20, bSSL, pUser->GetVHost(), pIRCSock)) {
+	if (!m_Manager.Connect(pServer->GetName(), pServer->GetPort(), sSockName, 120, bSSL, pUser->GetVHost(), pIRCSock)) {
 		ReleaseISpoof();
 		pUser->PutStatus("Unable to connect. (Bad host?)");
 	}
@@ -137,59 +166,86 @@ bool CZNC::ConnectUser(CUser *pUser) {
 	return true;
 }
 
+bool CZNC::HandleUserDeletion()
+{
+	map<CString, CUser*>::iterator it;
+	map<CString, CUser*>::iterator end;
+
+	if (m_msDelUsers.size() == 0)
+		return false;
+
+	end = m_msDelUsers.end();
+	for (it = m_msDelUsers.begin(); it != end; it++) {
+		CUser* pUser = it->second;
+		pUser->SetBeingDeleted(true);
+
+#ifdef _MODULES
+		if (GetModules().OnDeleteUser(*pUser)) {
+			pUser->SetBeingDeleted(false);
+			continue;
+		}
+#endif
+		m_msUsers.erase(pUser->GetUserName());
+
+		CIRCSock* pIRCSock = pUser->GetIRCSock();
+
+		if (pIRCSock) {
+			m_Manager.DelSockByAddr(pIRCSock);
+		}
+
+		pUser->DelClients();
+#ifdef _MODULES
+		pUser->DelModules();
+#endif
+		AddBytesRead(pUser->BytesRead());
+		AddBytesWritten(pUser->BytesWritten());
+		delete pUser;
+	}
+
+	m_msDelUsers.clear();
+	RestartConnectUser();
+
+	return true;
+}
+
 int CZNC::Loop() {
-	EnableConnectUser();
-
 	while (true) {
-		// Check for users that need to be deleted
-		if (m_ssDelUsers.size()) {
-			for (set<CUser*>::iterator it = m_ssDelUsers.begin(); it != m_ssDelUsers.end(); it++) {
-				CUser* pUser = *it;
-				pUser->SetBeingDeleted(true);
+		CString sError;
+		map<CString, CUser*>::iterator it;
+		map<CString, CUser*>::iterator end;
 
-#ifdef _MODULES
-				if (GetModules().OnDeleteUser(*pUser)) {
-					pUser->SetBeingDeleted(false);
-					continue;
-				}
-#endif
-				m_msUsers.erase(pUser->GetUserName());
+		if (GetNeedRehash()) {
+			SetNeedRehash(false);
 
-				CIRCSock* pIRCSock = pUser->GetIRCSock();
-
-				if (pIRCSock) {
-					m_Manager.DelSockByAddr(pIRCSock);
-				}
-
-				pUser->DelClients();
-#ifdef _MODULES
-				pUser->DelModules();
-#endif
-				AddBytesRead(pUser->BytesRead());
-				AddBytesWritten(pUser->BytesWritten());
-				delete pUser;
+			if (RehashConfig(sError)) {
+				Broadcast("Rehashing succeeded", true);
+			} else {
+				Broadcast("Rehashing failed: " + sError, true);
+				Broadcast("ZNC is in some possibly inconsistent state!", true);
 			}
+		}
 
-			m_ssDelUsers.clear();
-			RestartConnectUser();
+		// Check for users that need to be deleted
+		if (HandleUserDeletion()) {
+			// Also remove those user(s) from the config file
 			WriteConfig();
 		}
 
 		// Csocket wants micro seconds
-		// between 5 to 600 secs
-		m_Manager.DynamicSelectLoop(5 * 1000 * 1000, 600 * 1000 * 1000);
+		// 500 msec to 600 sec
+		m_Manager.DynamicSelectLoop(500 * 1000, 600 * 1000 * 1000);
 	}
 
 	return 0;
 }
 
 bool CZNC::WriteISpoof(CUser* pUser) {
-	if(m_pISpoofLockFile != NULL)
+	if (m_pISpoofLockFile != NULL)
 		return false;
 
 	if (!m_sISpoofFile.empty()) {
 		m_pISpoofLockFile = new CLockFile;
-		if(!m_pISpoofLockFile->TryExLock(m_sISpoofFile, 50, true)) {
+		if (!m_pISpoofLockFile->TryExLock(m_sISpoofFile, true)) {
 			delete m_pISpoofLockFile;
 			m_pISpoofLockFile = NULL;
 			return false;
@@ -215,7 +271,7 @@ bool CZNC::WriteISpoof(CUser* pUser) {
 }
 
 void CZNC::ReleaseISpoof() {
-	if(m_pISpoofLockFile == NULL)
+	if (m_pISpoofLockFile == NULL)
 		return;
 
 	if (!m_sISpoofFile.empty()) {
@@ -258,10 +314,30 @@ bool CZNC::WritePidFile(int iPid) {
 	return false;
 }
 
-bool CZNC::WritePemFile( bool bEncPem ) {
+bool CZNC::DeletePidFile() {
+	if (!m_sPidFile.empty()) {
+		CString sFile;
+		// absolute path or relative to the data dir?
+		if (m_sPidFile[0] != '/')
+			sFile = GetZNCPath() + "/" + m_sPidFile;
+		else
+			sFile = m_sPidFile;
+
+		CFile File(sFile);
+		CUtils::PrintAction("Deleting pid file [" + sFile + "]");
+		if (File.Delete()) {
+			CUtils::PrintStatus(true);
+			return true;
+		}
+		CUtils::PrintStatus(false);
+	}
+	return false;
+}
+
+bool CZNC::WritePemFile(bool bEncPem) {
 #ifndef HAVE_LIBSSL
 	CUtils::PrintError("ZNC was not compiled with ssl support.");
-	return false; 
+	return false;
 #else
 	CString sPemFile = GetPemLocation();
 	const char* pHostName = getenv("HOSTNAME");
@@ -276,7 +352,7 @@ bool CZNC::WritePemFile( bool bEncPem ) {
 		return false;
 	}
 
-	while (!CUtils::GetInput("hostname of your shell", sHost, sHost, "including the '.com' portion"));
+	while (!CUtils::GetInput("hostname of your shell", sHost, sHost, "including the '.com' portion")) ;
 
 	CUtils::PrintAction("Writing Pem file [" + sPemFile + "]");
 	FILE *f = fopen(sPemFile.c_str(), "w");
@@ -301,7 +377,7 @@ void CZNC::DeleteUsers() {
 	}
 
 	m_msUsers.clear();
-	RestartConnectUser();
+	DisableConnectUser();
 }
 
 CUser* CZNC::GetUser(const CString& sUser) {
@@ -326,17 +402,31 @@ bool CZNC::IsHostAllowed(const CString& sHostMask) {
 
 void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
 	char buf[PATH_MAX];
-	getcwd(buf, PATH_MAX);
+	char *home;
+
+	if (getcwd(buf, PATH_MAX) == NULL) {
+		CUtils::PrintError("getcwd() failed, can't read my current dir");
+		exit(-1);
+	}
 
 	// If the bin was not ran from the current directory, we need to add that dir onto our cwd
 	CString::size_type uPos = sArgvPath.rfind('/');
-	m_sCurPath = (uPos == CString::npos) ? CString(buf) : CUtils::ChangeDir(buf, sArgvPath.substr(0, uPos), "");
+	m_sCurPath = (uPos == CString::npos) ? CString(buf) : CDir::ChangeDir(buf, sArgvPath.substr(0, uPos), "");
 
 	// Try to set the user's home dir, default to binpath on failure
-	struct passwd* pUserInfo = getpwuid(getuid());
+	home = getenv("HOME");
 
-	if (pUserInfo) {
-		m_sHomePath = pUserInfo->pw_dir;
+	m_sHomePath.clear();
+	if (home) {
+		m_sHomePath = home;
+	}
+
+	if (m_sHomePath.empty()) {
+		struct passwd* pUserInfo = getpwuid(getuid());
+
+		if (pUserInfo) {
+			m_sHomePath = pUserInfo->pw_dir;
+		}
 	}
 
 	if (m_sHomePath.empty()) {
@@ -351,7 +441,6 @@ void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
 
 	// Other dirs that we use
 	m_sConfPath = m_sZNCPath + "/configs";
-	m_sConfBackupPath = m_sConfPath + "/backups";
 	m_sModPath = m_sZNCPath + "/modules";
 	m_sUserPath = m_sZNCPath + "/users";
 }
@@ -375,10 +464,25 @@ CString CZNC::ExpandConfigPath(const CString& sConfigFile) {
 	return sRetPath;
 }
 
-bool CZNC::WriteConfig() {
-	CFile File(m_sConfigFile);
+bool CZNC::BackupConfig() const {
+	CString sBackup = GetConfigFile() + "-backup";
 
-	if (!File.Copy(GetConfBackupPath() + "/" + File.GetShortName() + "-" + CString(time(NULL)))) {
+	// Create a new backup overwriting an old one we might have
+	if (CFile::Copy(m_sConfigFile, sBackup, true))
+		return true;
+
+	// Don't abort if no config file exists
+	if (!CFile::Exists(m_sConfigFile))
+		// No backup because we got nothing to backup
+		return true;
+
+	return false;
+}
+
+bool CZNC::WriteConfig() {
+	CFile File(GetConfigFile());
+
+	if (!BackupConfig()) {
 		return false;
 	}
 
@@ -396,25 +500,25 @@ bool CZNC::WriteConfig() {
 
 		CString s6 = (pListener->IsIPV6()) ? "6" : " ";
 
-		File.Write("Listen" + s6 + "      = " + sHostPortion + CString((pListener->IsSSL()) ? "+" : "") + CString(pListener->GetPort()) + "\r\n");
+		File.Write("Listen" + s6 + "      = " + sHostPortion + CString((pListener->IsSSL()) ? "+" : "") + CString(pListener->GetPort()) + "\n");
 	}
 
-	File.Write("ConnectDelay = " + CString(m_uiConnectDelay) + "\r\n");
+	File.Write("ConnectDelay = " + CString(m_uiConnectDelay) + "\n");
 
 	if (!m_sISpoofFile.empty()) {
-		File.Write("ISpoofFile   = " + m_sISpoofFile + "\r\n");
-		if (!m_sISpoofFormat.empty()) { File.Write("ISpoofFormat = " + m_sISpoofFormat + "\r\n"); }
+		File.Write("ISpoofFile   = " + m_sISpoofFile + "\n");
+		if (!m_sISpoofFormat.empty()) { File.Write("ISpoofFormat = " + m_sISpoofFormat + "\n"); }
 	}
 
-	if (!m_sPidFile.empty()) { File.Write("PidFile      = " + m_sPidFile + "\r\n"); }
-	if (!m_sStatusPrefix.empty()) { File.Write("StatusPrefix = " + m_sStatusPrefix + "\r\n"); }
+	if (!m_sPidFile.empty()) { File.Write("PidFile      = " + m_sPidFile + "\n"); }
+	if (!m_sStatusPrefix.empty()) { File.Write("StatusPrefix = " + m_sStatusPrefix + "\n"); }
 
 	for (unsigned int m = 0; m < m_vsMotd.size(); m++) {
-		File.Write("Motd         = " + m_vsMotd[m] + "\r\n");
+		File.Write("Motd         = " + m_vsMotd[m] + "\n");
 	}
 
 	for (unsigned int v = 0; v < m_vsVHosts.size(); v++) {
-		File.Write("VHost        = " + m_vsVHosts[v] + "\r\n");
+		File.Write("VHost        = " + m_vsVHosts[v] + "\n");
 	}
 
 #ifdef _MODULES
@@ -427,7 +531,7 @@ bool CZNC::WriteConfig() {
 			sArgs = " " + sArgs;
 		}
 
-		File.Write("LoadModule   = " + Mods[a]->GetModName() + sArgs + "\r\n");
+		File.Write("LoadModule   = " + Mods[a]->GetModName() + sArgs + "\n");
 	}
 #endif
 
@@ -439,7 +543,7 @@ bool CZNC::WriteConfig() {
 			continue;
 		}
 
-		File.Write("\r\n");
+		File.Write("\n");
 
 		if (!it->second->WriteConfig(File)) {
 			DEBUG_ONLY(cerr << "** Error writing config for user [" << it->first << "]" << endl);
@@ -457,7 +561,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 	vector<CString> vsLines;
 
 	if (CFile::Exists(sConfigFile)) {
-		if (!m_LockFile.TryExLock(sConfigFile, 50)) {
+		if (!m_LockFile.TryExLock(sConfigFile)) {
 			CUtils::PrintError("ZNC is currently running on this config.");
 			return false;
 		}
@@ -475,8 +579,8 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 	CUtils::PrintMessage("");
 
 	// Listen
-	unsigned int uPort = 0;
-	while(!CUtils::GetNumInput("What port would you like ZNC to listen on?", uPort, 1, 65535));
+	unsigned int uListenPort = 0;
+	while (!CUtils::GetNumInput("What port would you like ZNC to listen on?", uListenPort, 1, 65535)) ;
 
 	CString sSSL;
 #ifdef HAVE_LIBSSL
@@ -499,7 +603,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 		sListenHost += " ";
 	}
 
-	vsLines.push_back("Listen" + s6 + "    = " + sListenHost + sSSL + CString(uPort));
+	vsLines.push_back("Listen" + s6 + "    = " + sListenHost + sSSL + CString(uListenPort));
 	// !Listen
 
 #ifdef _MODULES
@@ -562,7 +666,8 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 		} while (!CUser::IsValidUserName(sUser));
 
 		vsLines.push_back("<User " + sUser + ">");
-		sAnswer = CUtils::GetHashPass();					vsLines.push_back("\tPass       = " + sAnswer + " -");
+		sAnswer = CUtils::GetHashPass();
+		vsLines.push_back("\tPass       = " + sAnswer + " -");
 
 		if (CUtils::GetBoolInput("Would you like this user to be an admin?", bFirstUser)) {
 			vsLines.push_back("\tAdmin      = true");
@@ -570,14 +675,23 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 			vsLines.push_back("\tAdmin      = false");
 		}
 
-		CUtils::GetInput("Nick", sNick, CUser::MakeCleanUserName(sUser));			vsLines.push_back("\tNick       = " + sNick);
-		CUtils::GetInput("Alt Nick", sAnswer, sNick + "_");	if (!sAnswer.empty()) { vsLines.push_back("\tAltNick    = " + sAnswer); }
-		CUtils::GetInput("Ident", sAnswer, sNick);			vsLines.push_back("\tIdent      = " + sAnswer);
-		CUtils::GetInput("Real Name", sAnswer, "Got ZNC?");	vsLines.push_back("\tRealName   = " + sAnswer);
-		CUtils::GetInput("VHost", sAnswer, "", "optional");	if (!sAnswer.empty()) { vsLines.push_back("\tVHost      = " + sAnswer); }
+		CUtils::GetInput("Nick", sNick, CUser::MakeCleanUserName(sUser));
+		vsLines.push_back("\tNick       = " + sNick);
+		CUtils::GetInput("Alt Nick", sAnswer, sNick + "_");
+		if (!sAnswer.empty()) {
+			vsLines.push_back("\tAltNick    = " + sAnswer);
+		}
+		CUtils::GetInput("Ident", sAnswer, sNick);
+		vsLines.push_back("\tIdent      = " + sAnswer);
+		CUtils::GetInput("Real Name", sAnswer, "Got ZNC?");
+		vsLines.push_back("\tRealName   = " + sAnswer);
+		CUtils::GetInput("VHost", sAnswer, "", "optional");
+		if (!sAnswer.empty()) {
+			vsLines.push_back("\tVHost      = " + sAnswer);
+		}
 		// todo: Possibly add motd
 
-		if (CUtils::GetBoolInput("Would you like ZNC to keep trying for your primary nick?", true)) {
+		if (CUtils::GetBoolInput("Would you like ZNC to keep trying for your primary nick?", false)) {
 			vsLines.push_back("\tKeepNick   = true");
 		} else {
 			vsLines.push_back("\tKeepNick   = false");
@@ -585,8 +699,11 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 
 		unsigned int uBufferCount = 0;
 
-		CUtils::GetNumInput("Number of lines to buffer per channel", uBufferCount, 0, ~0, 50);	if (uBufferCount) { vsLines.push_back("\tBuffer     = " + CString(uBufferCount)); }
-		if (CUtils::GetBoolInput("Would you like your buffer to be sticky?", true)) {
+		CUtils::GetNumInput("Number of lines to buffer per channel", uBufferCount, 0, ~0, 50);
+		if (uBufferCount) {
+			vsLines.push_back("\tBuffer     = " + CString(uBufferCount));
+		}
+		if (CUtils::GetBoolInput("Would you like to keep buffers after replay?", false)) {
 			vsLines.push_back("\tKeepBuffer = true");
 		} else {
 			vsLines.push_back("\tKeepBuffer = false");
@@ -647,17 +764,17 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 		do {
 			CString sHost, sPass;
 			bool bSSL = false;
-			unsigned int uPort = 0;
+			unsigned int uServerPort = 0;
 
-			while(!CUtils::GetInput("IRC server", sHost, "", "host only") || !CServer::IsValidHostName(sHost));
-			while(!CUtils::GetNumInput("[" + sHost + "] Port", uPort, 1, 65535, 6667));
+			while (!CUtils::GetInput("IRC server", sHost, "", "host only") || !CServer::IsValidHostName(sHost)) ;
+			while (!CUtils::GetNumInput("[" + sHost + "] Port", uServerPort, 1, 65535, 6667)) ;
 			CUtils::GetInput("[" + sHost + "] Password (probably empty)", sPass);
 
 #ifdef HAVE_LIBSSL
 			bSSL = CUtils::GetBoolInput("Does this server use SSL? (probably no)", false);
 #endif
 
-			vsLines.push_back("\tServer     = " + sHost + ((bSSL) ? " +" : " ") + CString(uPort) + " " + sPass);
+			vsLines.push_back("\tServer     = " + sHost + ((bSSL) ? " +" : " ") + CString(uServerPort) + " " + sPass);
 		} while (CUtils::GetBoolInput("Would you like to add another server?", false));
 
 		vsLines.push_back("");
@@ -670,7 +787,7 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 		bool bDefault = true;
 
 		while (CUtils::GetBoolInput("Would you like to add " + sArg + " channel" + sPost, bDefault)) {
-			while (!CUtils::GetInput("Channel name", sAnswer));
+			while (!CUtils::GetInput("Channel name", sAnswer)) ;
 			vsLines.push_back("\t<Chan " + sAnswer + ">");
 			vsLines.push_back("\t</Chan>");
 			sArg = "another";
@@ -737,48 +854,132 @@ bool CZNC::WriteNewConfig(const CString& sConfig) {
 	CUtils::PrintMessage("as the irc server password like so.. user:pass.", true);
 	CUtils::PrintMessage("");
 	CUtils::PrintMessage("Try something like this in your IRC client...", true);
-	CUtils::PrintMessage("/server <znc_server_ip> " + CString(uPort) + " " + sUser + ":<pass>", true);
+	CUtils::PrintMessage("/server <znc_server_ip> " + CString(uListenPort) + " " + sUser + ":<pass>", true);
 	CUtils::PrintMessage("");
 
 	m_LockFile.UnLock();
 	return CUtils::GetBoolInput("Launch znc now?", true);
 }
 
-bool CZNC::ParseConfig(const CString& sConfig) {
+bool CZNC::ParseConfig(const CString& sConfig)
+{
+	CString s;
+
 	m_sConfigFile = ExpandConfigPath(sConfig);
 
+	return DoRehash(s);
+}
+
+bool CZNC::RehashConfig(CString& sError)
+{
+#ifdef _MODULES
+	GetModules().OnPreRehash();
+	for (map<CString, CUser*>::iterator itb = m_msUsers.begin();
+			itb != m_msUsers.end(); itb++) {
+		itb->second->GetModules().OnPreRehash();
+	}
+#endif
+
+	// This clears m_msDelUsers
+	HandleUserDeletion();
+
+	// Mark all users as going-to-be deleted
+	m_msDelUsers = m_msUsers;
+	m_msUsers.clear();
+
+	// Make sure that timer doesn't have a stale iterator
+	DisableConnectUser();
+
+	if (DoRehash(sError)) {
+#ifdef _MODULES
+		GetModules().OnPostRehash();
+		for (map<CString, CUser*>::iterator it = m_msUsers.begin();
+				it != m_msUsers.end(); it++) {
+			it->second->GetModules().OnPostRehash();
+		}
+#endif
+
+		return true;
+	}
+
+	// Rehashing failed, try to recover
+	CString s;
+	while (m_msDelUsers.size()) {
+		AddUser(m_msDelUsers.begin()->second, s);
+		m_msDelUsers.erase(m_msDelUsers.begin());
+	}
+
+	// Make sure that users that want to connect do so
+	RestartConnectUser();
+
+	return false;
+}
+
+bool CZNC::DoRehash(CString& sError)
+{
+	sError.clear();
+
 	CUtils::PrintAction("Opening Config [" + m_sConfigFile + "]");
 
 	if (!CFile::Exists(m_sConfigFile)) {
-		CUtils::PrintStatus(false, "No such file");
+		sError = "No such file";
+		CUtils::PrintStatus(false, sError);
 		CUtils::PrintMessage("Restart znc with the --makeconf option if you wish to create this config.");
 		return false;
 	}
 
 	if (!CFile::IsReg(m_sConfigFile)) {
-		CUtils::PrintStatus(false, "Not a file");
+		sError = "Not a file";
+		CUtils::PrintStatus(false, sError);
 		return false;
 	}
 
-	if (!m_LockFile.TryExLock(m_sConfigFile, 50)) {
-		CUtils::PrintStatus(false, "ZNC is already running on this config.");
+	if (!m_LockFile.Open(m_sConfigFile)) {
+		sError = "Can not open config file";
+		CUtils::PrintStatus(false, sError);
 		return false;
 	}
 
-	CFile File(m_sConfigFile);
+	if (!m_LockFile.TryExLock()) {
+		sError = "ZNC is already running on this config.";
+		CUtils::PrintStatus(false, sError);
+		return false;
+	}
 
-	if (!File.Open(O_RDONLY)) {
-		CUtils::PrintStatus(false);
+	CFile File(m_LockFile.GetFD(), m_sConfigFile);
+
+	// This fd is re-used for rehashing, so we must seek back to the beginning!
+	if (!File.Seek(0)) {
+		sError = "Could not seek to the beginning of the config.";
+		CUtils::PrintStatus(false, sError);
 		return false;
 	}
 
 	CUtils::PrintStatus(true);
 
+	m_vsVHosts.clear();
+	m_vsMotd.clear();
+
+	// Delete all listeners
+	while (m_vpListeners.size()) {
+		delete m_vpListeners[0];
+		m_vpListeners.erase(m_vpListeners.begin());
+	}
+
+	// Make sure that timer doesn't have a stale iterator
+	DisableConnectUser();
+
 	CString sLine;
 	bool bCommented = false;	// support for /**/ style comments
 	CUser* pUser = NULL;	// Used to keep track of which user block we are in
+	CUser* pRealUser = NULL;	// If we rehash a user, this is the real one
 	CChan* pChan = NULL;	// Used to keep track of which chan block we are in
 	unsigned int uLineNum = 0;
+#ifdef _MODULES
+	MCString msModules;	// Modules are queued for later loading
+#endif
+
+	std::list<CGlobalModuleConfigLine> lGlobalModuleConfigLine;
 
 	while (File.ReadLine(sLine)) {
 		uLineNum++;
@@ -819,51 +1020,90 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 			sValue.Trim();
 
 			if (sLine.Left(1) == "/") {
-				CString sTag = sLine.substr(1);
+				sTag = sTag.substr(1);
 
 				if (pUser) {
 					if (pChan) {
 						if (sTag.CaseCmp("Chan") == 0) {
-							pUser->AddChan(pChan);
+							// Save the channel name, because AddChan
+							// deletes the CChannel*, if adding fails
+							sError = pChan->GetName();
+							if (!pUser->AddChan(pChan)) {
+								sError = "Channel [" + sError + "] defined more than once";
+								CUtils::PrintError(sError);
+								return false;
+							}
+							sError.clear();
 							pChan = NULL;
 							continue;
 						}
 					} else if (sTag.CaseCmp("User") == 0) {
 						CString sErr;
 
-						if (!AddUser(pUser, sErr)) {
-							CUtils::PrintError("Invalid user [" + pUser->GetUserName() + "] " + sErr);
+						if (pRealUser) {
+							if (!pRealUser->Clone(*pUser, sErr)
+									|| !AddUser(pRealUser, sErr)) {
+								sError = "Invalid user [" + pUser->GetUserName() + "] " + sErr;
+								DEBUG_ONLY(cout << "CUser::Clone() failed in rehash" << endl);
+							}
 							pUser->SetBeingDeleted(true);
 							delete pUser;
+							pUser = NULL;
+						} else if (!AddUser(pUser, sErr)) {
+							sError = "Invalid user [" + pUser->GetUserName() + "] " + sErr;
+						}
+
+						if (!sError.empty()) {
+							CUtils::PrintError(sError);
+							if (pUser) {
+								pUser->SetBeingDeleted(true);
+								delete pUser;
+								pUser = NULL;
+							}
 							return false;
 						}
 
 						pUser = NULL;
+						pRealUser = NULL;
 						continue;
 					}
 				}
 			} else if (sTag.CaseCmp("User") == 0) {
 				if (pUser) {
-					CUtils::PrintError("You may not nest <User> tags inside of other <User> tags.");
+					sError = "You may not nest <User> tags inside of other <User> tags.";
+					CUtils::PrintError(sError);
 					return false;
 				}
 
 				if (sValue.empty()) {
-					CUtils::PrintError("You must supply a username in the <User> tag.");
+					sError = "You must supply a username in the <User> tag.";
+					CUtils::PrintError(sError);
 					return false;
 				}
 
 				if (m_msUsers.find(sValue) != m_msUsers.end()) {
-					CUtils::PrintError("User [" + sValue + "] defined more than once.");
+					sError = "User [" + sValue + "] defined more than once.";
+					CUtils::PrintError(sError);
 					return false;
 				}
 
-				pUser = new CUser(sValue);
 				CUtils::PrintMessage("Loading user [" + sValue + "]");
 
+				// Either create a CUser* or use an existing one
+				map<CString, CUser*>::iterator it = m_msDelUsers.find(sValue);
+
+				if (it != m_msDelUsers.end()) {
+					pRealUser = it->second;
+					m_msDelUsers.erase(it);
+				} else
+					pRealUser = NULL;
+
+				pUser = new CUser(sValue);
+
 				if (!m_sStatusPrefix.empty()) {
 					if (!pUser->SetStatusPrefix(m_sStatusPrefix)) {
-						CUtils::PrintError("Invalid StatusPrefix [" + m_sStatusPrefix + "] Must be 1-5 chars, no spaces.");
+						sError = "Invalid StatusPrefix [" + m_sStatusPrefix + "] Must be 1-5 chars, no spaces.";
+						CUtils::PrintError(sError);
 						return false;
 					}
 				}
@@ -871,12 +1111,14 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 				continue;
 			} else if (sTag.CaseCmp("Chan") == 0) {
 				if (!pUser) {
-					CUtils::PrintError("<Chan> tags must be nested inside of a <User> tag.");
+					sError = "<Chan> tags must be nested inside of a <User> tag.";
+					CUtils::PrintError(sError);
 					return false;
 				}
 
 				if (pChan) {
-					CUtils::PrintError("You may not nest <Chan> tags inside of other <Chan> tags.");
+					sError = "You may not nest <Chan> tags inside of other <Chan> tags.";
+					CUtils::PrintError(sError);
 					return false;
 				}
 
@@ -891,12 +1133,6 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 		sName.Trim();
 		sValue.Trim();
 
-#ifdef _MODULES
-		if (GetModules().OnConfigLine(sName, sValue, pUser, pChan)) {
-			continue;
-		}
-#endif
-
 		if ((!sName.empty()) && (!sValue.empty())) {
 			if (pUser) {
 				if (pChan) {
@@ -969,9 +1205,13 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 					} else if (sName.CaseCmp("Admin") == 0) {
 						pUser->SetAdmin((sValue.CaseCmp("TRUE") == 0));
 						continue;
+					} else if (sName.CaseCmp("DenySetVHost") == 0) {
+						pUser->SetDenySetVHost((sValue.CaseCmp("TRUE") == 0));
+						continue;
 					} else if (sName.CaseCmp("StatusPrefix") == 0) {
 						if (!pUser->SetStatusPrefix(sValue)) {
-							CUtils::PrintError("Invalid StatusPrefix [" + sValue + "] Must be 1-5 chars, no spaces.");
+							sError = "Invalid StatusPrefix [" + sValue + "] Must be 1-5 chars, no spaces.";
+							CUtils::PrintError(sError);
 							return false;
 						}
 						continue;
@@ -1010,14 +1250,14 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 						pUser->SetTimestampPrepend(sValue.ToBool());
 						continue;
 					} else if (sName.CaseCmp("Timestamp") == 0) {
-						if(sValue.Trim_n().CaseCmp("true") != 0) {
-							if(sValue.Trim_n().CaseCmp("append") == 0) {
+						if (sValue.Trim_n().CaseCmp("true") != 0) {
+							if (sValue.Trim_n().CaseCmp("append") == 0) {
 								pUser->SetTimestampAppend(true);
 								pUser->SetTimestampPrepend(false);
-							} else if(sValue.Trim_n().CaseCmp("prepend") == 0) {
+							} else if (sValue.Trim_n().CaseCmp("prepend") == 0) {
 								pUser->SetTimestampAppend(false);
 								pUser->SetTimestampPrepend(true);
-							} else if(sValue.Trim_n().CaseCmp("false") == 0) {
+							} else if (sValue.Trim_n().CaseCmp("false") == 0) {
 								pUser->SetTimestampAppend(false);
 								pUser->SetTimestampPrepend(false);
 							} else {
@@ -1048,14 +1288,17 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 
 							CUtils::PrintStatus(bModRet, sModRet);
 							if (!bModRet) {
+								sError = sModRet;
 								return false;
 							}
 						} catch (CException e) {
+							sError = "Caught exception while loading [" + sModName + "]";
 							CUtils::PrintStatus(false, sModRet);
 							return false;
 						}
 #else
-						CUtils::PrintStatus(false, "Modules are not enabled.");
+						sError = "Modules are not enabled.";
+						CUtils::PrintStatus(false, sError);
 #endif
 						continue;
 					}
@@ -1101,23 +1344,29 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 
 #ifndef HAVE_IPV6
 					if (bIPV6) {
-						CUtils::PrintStatus(false, "IPV6 is not enabled");
+						sError = "IPV6 is not enabled";
+						CUtils::PrintStatus(false, sError);
 						return false;
 					}
 #endif
 
 #ifndef HAVE_LIBSSL
 					if (bSSL) {
-						CUtils::PrintStatus(false, "SSL is not enabled");
+						sError = "SSL is not enabled";
+						CUtils::PrintStatus(false, sError);
 						return false;
 					}
 #else
 					CString sPemFile = GetPemLocation();
 
 					if (bSSL && !CFile::Exists(sPemFile)) {
-						CUtils::PrintStatus(false, "Unable to locate pem file: [" + sPemFile + "]");
+						sError = "Unable to locate pem file: [" + sPemFile + "]";
+						CUtils::PrintStatus(false, sError);
 
-						if (CUtils::GetBoolInput("Would you like to create a new pem file?", true)) {
+						// If stdin is e.g. /dev/null and we call GetBoolInput(),
+						// we are stuck in an endless loop!
+						if (isatty(0) && CUtils::GetBoolInput("Would you like to create a new pem file?", true)) {
+							sError.clear();
 							WritePemFile();
 						} else {
 							return false;
@@ -1127,14 +1376,16 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 					}
 #endif
 					if (!uPort) {
-						CUtils::PrintStatus(false, "Invalid port");
+						sError = "Invalid port";
+						CUtils::PrintStatus(false, sError);
 						return false;
 					}
 
 					CListener* pListener = new CListener(uPort, sBindHost, bSSL, bIPV6);
 
 					if (!pListener->Listen()) {
-						CUtils::PrintStatus(false, "Unable to bind");
+						sError = "Unable to bind";
+						CUtils::PrintStatus(false, sError);
 						delete pListener;
 						return false;
 					}
@@ -1144,37 +1395,26 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 
 					continue;
 				} else if (sName.CaseCmp("LoadModule") == 0) {
-					CString sModName = sValue.Token(0);
-					CUtils::PrintAction("Loading Global Module [" + sModName + "]");
 #ifdef _MODULES
-					CString sModRet;
+					CString sModName = sValue.Token(0);
 					CString sArgs = sValue.Token(1, true);
 
-					try {
-						bool bModRet = GetModules().LoadModule(sModName, sArgs, NULL, sModRet);
-
-						// If the module was loaded, sModRet contains
-						// "Loaded Module [name] ..." and we strip away this beginning.
-						if (bModRet)
-							sModRet = sModRet.Token(1, true, sModName + "] ");
-
-						CUtils::PrintStatus(bModRet, sModRet);
-						if (!bModRet) {
-							return false;
-						}
-					} catch (CException e) {
-						CUtils::PrintStatus(false, sModRet);
+					if (msModules.find(sModName) != msModules.end()) {
+						sError = "Module [" + sModName +
+							"] already loaded";
+						CUtils::PrintError(sError);
 						return false;
 					}
+					msModules[sModName] = sArgs;
 #else
-					CUtils::PrintStatus(false, "Modules are not enabled.");
+					CUtils::PrintError("Modules are not enabled.");
 #endif
 					continue;
 				} else if (sName.CaseCmp("ISpoofFormat") == 0) {
 					m_sISpoofFormat = sValue;
 					continue;
 				} else if (sName.CaseCmp("ISpoofFile") == 0) {
-					if(sValue.Left(2) == "~/") {
+					if (sValue.Left(2) == "~/") {
 						sValue.LeftChomp(2);
 						sValue = GetHomePath() + "/" + sValue;
 					}
@@ -1197,26 +1437,135 @@ bool CZNC::ParseConfig(const CString& sConfig) {
 					continue;
 				}
 			}
+
 		}
 
-		CUtils::PrintError("Unhandled line " + CString(uLineNum) + " in config: [" + sLine + "]");
-		return false;
+		if (sName.CaseCmp("GM:", 3) == 0)
+		{ // GM: prefix is a pass through to config lines for global modules
+			CGlobalModuleConfigLine cTmp;
+			cTmp.m_sName = sName.substr(3, CString::npos);
+			cTmp.m_sValue = sValue;
+			cTmp.m_pChan = pChan;
+			cTmp.m_pUser = pUser;
+			lGlobalModuleConfigLine.push_back(cTmp);
+		}
+		else
+		{
+			sError = "Unhandled line " + CString(uLineNum) + " in config: [" + sLine + "]";
+			CUtils::PrintError(sError);
+			return false;
+		}
 	}
 
+#ifdef _MODULES
+	// First step: Load and reload new modules or modules with new arguments
+	for (MCString::iterator it = msModules.begin(); it != msModules.end(); it++) {
+		CString sModName = it->first;
+		CString sArgs = it->second;
+		CString sModRet;
+		CModule *pOldMod;
+
+		pOldMod = GetModules().FindModule(sModName);
+		if (!pOldMod) {
+			CUtils::PrintAction("Loading Global Module [" + sModName + "]");
+
+			try {
+				bool bModRet = GetModules().LoadModule(sModName, sArgs, NULL, sModRet);
+
+				// If the module was loaded, sModRet contains
+				// "Loaded Module [name] ..." and we strip away this beginning.
+				if (bModRet)
+					sModRet = sModRet.Token(1, true, sModName + "] ");
+
+				CUtils::PrintStatus(bModRet, sModRet);
+				if (!bModRet) {
+					sError = sModRet;
+					return false;
+				}
+			} catch (CException e) {
+				sError = "Caught an exception while loading [" + sModName + "]";
+				CUtils::PrintStatus(false, sModRet);
+				return false;
+			}
+		} else if (pOldMod->GetArgs() != sArgs) {
+			CUtils::PrintAction("Reloading Global Module [" + sModName + "]");
+
+			try {
+				bool bModRet = GetModules().ReloadModule(sModName, sArgs, NULL, sModRet);
+
+				// If the module was loaded, sModRet contains
+				// "Loaded Module [name] ..." and we strip away this beginning.
+				if (bModRet)
+					sModRet = sModRet.Token(1, true, sModName + "] ");
+
+				CUtils::PrintStatus(bModRet, sModRet);
+				if (!bModRet) {
+					sError = sModRet;
+					return false;
+				}
+			} catch (CException e) {
+				sError = "Caught an exception while reloading [" + sModName + "]";
+				CUtils::PrintStatus(false, sModRet);
+				return false;
+			}
+		} else
+			CUtils::PrintMessage("Module [" + sModName + "] already loaded.");
+	}
+
+	// Second step: Unload modules which are no longer in the config
+	set<CString> ssUnload;
+	for (size_t i = 0; i < GetModules().size(); i++) {
+		CModule *pCurMod = GetModules()[i];
+
+		if (msModules.find(pCurMod->GetModName()) == msModules.end())
+			ssUnload.insert(pCurMod->GetModName());
+	}
+
+	for (set<CString>::iterator it = ssUnload.begin(); it != ssUnload.end(); it++) {
+		if (GetModules().UnloadModule(*it))
+			CUtils::PrintMessage("Unloaded Global Module [" + *it + "]");
+		else
+			CUtils::PrintMessage("Could not unload [" + *it + "]");
+	}
+
+	// last step, throw unhandled config items at global config
+	for (std::list<CGlobalModuleConfigLine>::iterator it = lGlobalModuleConfigLine.begin(); it != lGlobalModuleConfigLine.end(); it++)
+	{
+		if ((pChan && pChan == it->m_pChan) || (pUser && pUser == it->m_pUser))
+			continue; // skip unclosed user or chan
+		if (!GetModules().OnConfigLine(it->m_sName, it->m_sValue, it->m_pUser, it->m_pChan))
+		{
+			CUtils::PrintMessage("unhandled global module config line [GM:" + it->m_sName + "] = [" + it->m_sValue + "]");
+		}
+	}
+#endif
+
 	if (pChan) {
+		// TODO last <Chan> not closed
 		delete pChan;
 	}
 
+	if (pUser) {
+		// TODO last <User> not closed
+		delete pUser;
+	}
+
 	File.Close();
 
-	if (!m_msUsers.size()) {
-		CUtils::PrintError("You must define at least one user in your config.");
+	if (m_msUsers.size() == 0) {
+		sError = "You must define at least one user in your config.";
+		CUtils::PrintError(sError);
 		return false;
 	}
 
-#ifdef _MODULES
-	GetModules().OnFinishedConfig();
-#endif
+	if (m_vpListeners.size() == 0) {
+		sError = "You must supply at least one Listen port in your config.";
+		CUtils::PrintError(sError);
+		return false;
+	}
+
+	// Make sure that users that want to connect do so
+	RestartConnectUser();
 
 	return true;
 }
@@ -1245,40 +1594,48 @@ bool CZNC::RemVHost(const CString& sHost) {
 	return true;
 }
 
-void CZNC::Broadcast(const CString& sMessage, CUser* pUser) {
+void CZNC::Broadcast(const CString& sMessage, bool bAdminOnly,
+		CUser* pSkipUser, CClient *pSkipClient) {
 	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
-		if (a->second != pUser) {
+		if (bAdminOnly && !a->second->IsAdmin())
+			continue;
+
+		if (a->second != pSkipUser) {
 			CString sMsg = sMessage;
-#ifdef _MODULES
+
 			MODULECALL(OnBroadcast(sMsg), a->second, NULL, continue);
-#endif
-			a->second->PutStatusNotice("*** " + sMsg);
+			a->second->PutStatusNotice("*** " + sMsg, NULL, pSkipClient);
 		}
 	}
 }
 
-CString CZNC::FindModPath(const CString& sModule) const {
-	CString sModPath = GetCurPath() + "/modules/" + sModule;
-	sModPath += (sModule.find(".") == CString::npos) ? ".so" : "";
+bool CZNC::FindModPath(const CString& sModule, CString& sModPath,
+		CString& sDataPath) const {
+	CString sMod = sModule;
+	CString sDir = sMod;
+	if (sModule.find(".") == CString::npos)
+		sMod += ".so";
+
+	sDataPath = GetCurPath() + "/modules/";
+	sModPath = sDataPath + sMod;
 
 	if (!CFile::Exists(sModPath)) {
-		//DEBUG_ONLY(cout << "[" << sModPath << "] Not found..." << endl);
-		sModPath = GetModPath() + "/" + sModule;
-		sModPath += (sModule.find(".") == CString::npos) ? ".so" : "";
+		sDataPath = GetModPath() + "/";
+		sModPath = sDataPath + sMod;
 
 		if (!CFile::Exists(sModPath)) {
-			//DEBUG_ONLY(cout << "[" << sModPath << "] Not found..." << endl);
-			sModPath = _MODDIR_ + CString("/") + sModule;
-			sModPath += (sModule.find(".") == CString::npos) ? ".so" : "";
+			sDataPath = _DATADIR_ + CString("/");
+			sModPath = _MODDIR_ + CString("/") + sMod;
 
 			if (!CFile::Exists(sModPath)) {
-				//DEBUG_ONLY(cout << "[" << sModPath << "] Not found... giving up!" << endl);
-				return "";
+				return false;
 			}
 		}
 	}
 
-	return sModPath;
+	sDataPath += sDir;
+
+	return true;
 }
 
 CUser* CZNC::FindUser(const CString& sUsername) {
@@ -1298,7 +1655,7 @@ bool CZNC::DeleteUser(const CString& sUsername) {
 		return false;
 	}
 
-	m_ssDelUsers.insert(pUser);
+	m_msDelUsers[pUser->GetUserName()] = pUser;
 	return true;
 }
 
@@ -1336,6 +1693,35 @@ void CZNC::UpdateTrafficStats() {
 	}
 }
 
+void CZNC::AuthUser(CSmartPtr<CAuthBase> AuthClass) {
+#ifdef _MODULES
+	// TODO unless the auth module calls it, CUser::IsHostAllowed() is not honoured
+	if (GetModules().OnLoginAttempt(AuthClass)) {
+		return;
+	}
+#endif
+
+	CUser* pUser = GetUser(AuthClass->GetUsername());
+
+	if (!pUser || !pUser->CheckPass(AuthClass->GetPassword())) {
+		if (pUser) {
+			pUser->PutStatus("Another client attempted to login as you, with a bad password.");
+		}
+
+		AuthClass->RefuseLogin("Invalid Password");
+		return;
+	}
+
+	CString sHost = AuthClass->GetRemoteIP();
+
+	if (!pUser->IsHostAllowed(sHost)) {
+		AuthClass->RefuseLogin("Your host [" + sHost + "] is not allowed");
+		return;
+	}
+
+	AuthClass->AcceptLogin(*pUser);
+}
+
 class CConnectUserTimer : public CCron {
 public:
 	CConnectUserTimer(int iSecs) : CCron() {
