@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007  See the AUTHORS file for details.
+ * Copyright (C) 2004-2008  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,19 +9,17 @@
 #ifndef _ZNC_H
 #define _ZNC_H
 
-#include "main.h"
-#include "FileUtils.h"
 #include "Client.h"
+#include "Csocket.h"
+#include "FileUtils.h"
 #ifdef _MODULES
 #include "Modules.h"
 #endif
-#include <set>
 #include <map>
+
 using std::map;
-using std::set;
 
 class CUser;
-class CClient;
 class CListener;
 
 class CSockManager : public TSocketManager<Csock> {
@@ -71,7 +69,7 @@ public:
 	}
 
 	virtual u_short ListenAllRand(const CString& sSockName, int isSSL = false, int iMaxConns = SOMAXCONN, Csock *pcSock = NULL, u_int iTimeout = 0, bool bIsIPv6 = false) {
-		return( ListenRand( sSockName, "", isSSL, iMaxConns, pcSock, iTimeout, bIsIPv6 ) );
+		return(ListenRand(sSockName, "", isSSL, iMaxConns, pcSock, iTimeout, bIsIPv6));
 	}
 
 	virtual bool Connect(const CString& sHostname, u_short iPort , const CString& sSockName, int iTimeout = 60, bool isSSL = false, const CString& sBindHost = "", Csock *pcSock = NULL) {
@@ -99,53 +97,68 @@ public:
 	bool WriteISpoof(CUser* pUser);
 	void ReleaseISpoof();
 	bool WritePidFile(int iPid);
+	bool DeletePidFile();
 	CUser* GetUser(const CString& sUser);
 	Csock* FindSockByName(const CString& sSockName);
-	bool ParseConfig(const CString& sConfig);
 	bool IsHostAllowed(const CString& sHostMask);
 	void InitDirs(const CString& sArgvPath, const CString& sDataDir);
 	bool OnBoot();
 	CString ExpandConfigPath(const CString& sConfigFile);
 	bool WriteNewConfig(const CString& sConfig);
 	bool WriteConfig();
+	bool ParseConfig(const CString& sConfig);
+	bool RehashConfig(CString& sError);
+	static CString GetVersion();
 	static CString GetTag(bool bIncludeVersion = true);
-	CString FindModPath(const CString& sModule) const;
+	CString GetUptime();
+	// This returns the path to the .so and to the data dir
+	// which is where static data (webadmin skins) are saved
+	bool FindModPath(const CString& sModule, CString& sModPath,
+			CString& sDataPath) const;
 	void ClearVHosts();
 	bool AddVHost(const CString& sHost);
 	bool RemVHost(const CString& sHost);
-	void Broadcast(const CString& sMessage, CUser* pUser = NULL);
+	void Broadcast(const CString& sMessage, bool bAdminOnly = false,
+			CUser* pSkipUser = NULL, CClient* pSkipClient = NULL);
 	void AddBytesRead(unsigned long long u) { m_uBytesRead += u; }
 	void AddBytesWritten(unsigned long long u) { m_uBytesWritten += u; }
 	unsigned long long BytesRead() const { return m_uBytesRead; }
 	unsigned long long BytesWritten() const { return m_uBytesWritten; }
 	void UpdateTrafficStats();
 
+	// Authenticate a user.
+	// The result is passed back via callbacks to CAuthBase.
+	// CSmartPtr handles freeing this pointer!
+	void AuthUser(CSmartPtr<CAuthBase> AuthClass);
+
 	// Setters
+	void SetNeedRehash(bool b) { m_bNeedRehash = b; }
 	void SetStatusPrefix(const CString& s) { m_sStatusPrefix = (s.empty()) ? "*" : s; }
 	void SetISpoofFile(const CString& s) { m_sISpoofFile = s; }
 	void SetISpoofFormat(const CString& s) { m_sISpoofFormat = (s.empty()) ? "global { reply \"%\" }" : s; }
 	// !Setters
 
 	// Getters
+	bool GetNeedRehash() { return m_bNeedRehash; }
 	CSockManager& GetManager() { return m_Manager; }
 #ifdef _MODULES
 	CGlobalModules& GetModules() { return *m_pModules; }
 #endif
 	const CString& GetStatusPrefix() const { return m_sStatusPrefix; }
-	const CString& GetCurPath() const { if (!CFile::Exists(m_sCurPath)) { CUtils::MakeDir(m_sCurPath); } return m_sCurPath; }
-	const CString& GetModPath() const { if (!CFile::Exists(m_sModPath)) { CUtils::MakeDir(m_sModPath); } return m_sModPath; }
-	const CString& GetHomePath() const { if (!CFile::Exists(m_sHomePath)) { CUtils::MakeDir(m_sHomePath); } return m_sHomePath; }
-	const CString& GetZNCPath() const { if (!CFile::Exists(m_sZNCPath)) { CUtils::MakeDir(m_sZNCPath); } return m_sZNCPath; }
-	const CString& GetConfPath() const { if (!CFile::Exists(m_sConfPath)) { CUtils::MakeDir(m_sConfPath); } return m_sConfPath; }
-	const CString& GetConfBackupPath() const { if (!CFile::Exists(m_sConfBackupPath)) { CUtils::MakeDir(m_sConfBackupPath); } return m_sConfBackupPath; }
-	const CString& GetUserPath() const { if (!CFile::Exists(m_sUserPath)) { CUtils::MakeDir(m_sUserPath); } return m_sUserPath; }
+	const CString& GetCurPath() const { if (!CFile::Exists(m_sCurPath)) { CDir::MakeDir(m_sCurPath); } return m_sCurPath; }
+	const CString& GetModPath() const { if (!CFile::Exists(m_sModPath)) { CDir::MakeDir(m_sModPath); } return m_sModPath; }
+	const CString& GetHomePath() const { if (!CFile::Exists(m_sHomePath)) { CDir::MakeDir(m_sHomePath); } return m_sHomePath; }
+	const CString& GetZNCPath() const { if (!CFile::Exists(m_sZNCPath)) { CDir::MakeDir(m_sZNCPath); } return m_sZNCPath; }
+	const CString& GetConfPath() const { if (!CFile::Exists(m_sConfPath)) { CDir::MakeDir(m_sConfPath); } return m_sConfPath; }
+	const CString& GetUserPath() const { if (!CFile::Exists(m_sUserPath)) { CDir::MakeDir(m_sUserPath); } return m_sUserPath; }
 	CString GetPemLocation() const { return GetZNCPath() + "/znc.pem"; }
 	const CString& GetConfigFile() const { return m_sConfigFile; }
-	bool WritePemFile( bool bEncPem = false );
+	bool WritePemFile(bool bEncPem = false);
 	const CString& GetISpoofFile() const { return m_sISpoofFile; }
 	const CString& GetISpoofFormat() const { return m_sISpoofFormat; }
 	const VCString& GetVHosts() const { return m_vsVHosts; }
 	const vector<CListener*>& GetListeners() const { return m_vpListeners; }
+	time_t TimeStarted() const { return m_TimeStarted; }
 	// !Getters
 
 	// Static allocator
@@ -153,7 +166,7 @@ public:
 	CUser* FindUser(const CString& sUsername);
 	bool DeleteUser(const CString& sUsername);
 	bool AddUser(CUser* pUser, CString& sErrorRet);
-	const map<CString,CUser*> & GetUserMap() const { return( m_msUsers ); }
+	const map<CString,CUser*> & GetUserMap() const { return(m_msUsers); }
 
 	// Message of the Day
 	void SetMotd(const CString& sMessage) { ClearMotd(); AddMotd(sMessage); }
@@ -171,10 +184,19 @@ public:
 	void RestartConnectUser();
 
 private:
+	bool DoRehash(CString& sError);
+	// Returns true if something was done
+	bool HandleUserDeletion();
+	// Backup znc.conf
+	bool BackupConfig() const;
+
 protected:
+	time_t				m_TimeStarted;
+
+	bool				m_bNeedRehash;
 	vector<CListener*>		m_vpListeners;
 	map<CString,CUser*>		m_msUsers;
-	set<CUser*>				m_ssDelUsers;
+	map<CString,CUser*>		m_msDelUsers;
 	CSockManager			m_Manager;
 
 	CString					m_sCurPath;
@@ -182,7 +204,6 @@ protected:
 	CString					m_sHomePath;
 	CString					m_sZNCPath;
 	CString					m_sConfPath;
-	CString					m_sConfBackupPath;
 	CString					m_sUserPath;
 
 	CString					m_sConfigFile;
@@ -211,9 +232,13 @@ public:
 		m_sBindHost = sBindHost;
 		m_bSSL = bSSL;
 		m_bIPV6 = bIPV6;
+		m_pClient = NULL;
 	}
 
-	virtual ~CListener() {}
+	virtual ~CListener() {
+		if (m_pClient)
+			CZNC::Get().GetManager().DelSockByAddr(m_pClient);
+	}
 
 	// Setters
 	void SetSSL(bool b) { m_bSSL = b; }
@@ -229,22 +254,22 @@ public:
 	const CString& GetBindHost() const { return m_sBindHost; }
 	// !Getters
 
-	bool Listen() const {
-		if (!m_uPort) {
+	bool Listen() {
+		if (!m_uPort || m_pClient) {
 			return false;
 		}
 
-		CClient* pClient = new CClient;
+		m_pClient = new CClient;
 
 		bool bSSL = false;
 #ifdef HAVE_LIBSSL
 		if (IsSSL()) {
 			bSSL = true;
-			pClient->SetPemLocation(CZNC::Get().GetPemLocation());
+			m_pClient->SetPemLocation(CZNC::Get().GetPemLocation());
 		}
 #endif
 
-		return CZNC::Get().GetManager().ListenHost(m_uPort, "_LISTENER", m_sBindHost, bSSL, SOMAXCONN, pClient, 0, m_bIPV6);
+		return CZNC::Get().GetManager().ListenHost(m_uPort, "_LISTENER", m_sBindHost, bSSL, SOMAXCONN, m_pClient, 0, m_bIPV6);
 	}
 private:
 protected:
@@ -252,6 +277,7 @@ protected:
 	bool			m_bIPV6;
 	unsigned short	m_uPort;
 	CString			m_sBindHost;
+	CClient*		m_pClient;
 };
 
 #endif // !_ZNC_H
